@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -32,9 +32,9 @@ Module::Module(LexicalScope*parent, perm_string n)
 {
       library_flag = false;
       is_cell = false;
+      is_interface = false;
       program_block = false;
       uc_drive = UCD_NONE;
-      timescale_warn_done = false;
 }
 
 Module::~Module()
@@ -122,3 +122,13 @@ const list<PGate*>& Module::get_gates() const
 {
       return gates_;
 }
+
+PNamedItem::SymbolType Module::symbol_type() const
+{
+      if (program_block)
+            return PROGRAM;
+      if (is_interface)
+            return INTERFACE;
+
+      return MODULE;
+}
