@@ -1,7 +1,7 @@
 #ifndef IVL_Module_H
 #define IVL_Module_H
 /*
- * Copyright (c) 1998-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -28,7 +28,7 @@
 # include  "HName.h"
 # include  "named.h"
 # include  "PScope.h"
-# include  "LineInfo.h"
+# include  "PNamedItem.h"
 # include  "netlist.h"
 # include  "pform_types.h"
 class PExpr;
@@ -54,7 +54,7 @@ class NetScope;
  * these containers as well.
  */
 
-class Module : public PScopeExtra, public LineInfo {
+class Module : public PScopeExtra, public PNamedItem {
 
 	/* The module ports are in general a vector of port_t
 	   objects. Each port has a name and an ordered list of
@@ -97,7 +97,7 @@ class Module : public PScopeExtra, public LineInfo {
 	/* specparams are simpler than other parameters, in that they
 	   can have a range, but not an explicit type. The restrictions
 	   are enforced by the parser. */
-      map<perm_string,param_expr_t>specparams;
+      map<perm_string,param_expr_t*>specparams;
 
 	/* The module also has defparam assignments which don't create
 	   new parameters within the module, but may be used to set
@@ -121,8 +121,6 @@ class Module : public PScopeExtra, public LineInfo {
 
       map<perm_string,PExpr*> attributes;
 
-      bool timescale_warn_done;
-
 	/* The module has a list of generate schemes that appear in
 	   the module definition. These are used at elaboration time. */
       list<PGenerate*> generate_schemes;
@@ -162,6 +160,8 @@ class Module : public PScopeExtra, public LineInfo {
 
       bool elaborate_sig(Design*, NetScope*scope) const;
 
+      SymbolType symbol_type() const;
+
     private:
       void dump_specparams_(ostream&out, unsigned indent) const;
       list<PGate*> gates_;
