@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -187,9 +187,15 @@ ostream& operator << (ostream&fd, NetCaseCmp::kind_t that)
 	  case NetCaseCmp::NEQ:
 	    fd << "!==";
 	    break;
-	  case NetCaseCmp::XEQ:
+	  case NetCaseCmp::WEQ:
 	    fd << "==?";
 	    break;
+	  case NetCaseCmp::WNE:
+	    fd << "!=?";
+	    break;
+	  case NetCaseCmp::XEQ:
+	    fd << "==x?";
+	    break;
 	  case NetCaseCmp::ZEQ:
 	    fd << "==z?";
 	    break;
@@ -226,13 +232,15 @@ ostream& netdarray_t::debug_dump(ostream&o) const
 
 ostream& netqueue_t::debug_dump(ostream&fd) const
 {
-      fd << "queue of " << *element_type();
+      fd << "queue of ";
+      if (max_idx_ >= 0) fd << "(maximum of " << max_idx_+1 << " elements) ";
+      fd << *element_type();
       return fd;
 }
 
 ostream& netvector_t::debug_dump(ostream&o) const
 {
-      o << type_ << (signed_? " signed" : " unsigned") << packed_dims_;
+      o << "netvector_t:" << type_ << (signed_? " signed" : " unsigned") << packed_dims_;
       return o;
 }
 
@@ -344,6 +352,10 @@ void NetDelaySrc::dump(ostream&o, unsigned ind) const
 	    if (has_condit()) o << " if";
 	    else o << " ifnone";
       }
+      if (parallel_)
+	    o << " parallel";
+      else
+	    o << " full";
       o << " src "
 	<< "(" << transition_delays_[IVL_PE_01]
 	<< "," << transition_delays_[IVL_PE_10]
@@ -688,6 +700,14 @@ void NetFF::dump_node(ostream&o, unsigned ind) const
       dump_obj_attr(o, ind+4);
 }
 
+void NetLatch::dump_node(ostream&o, unsigned ind) const
+{
+      o << setw(ind) << "" << "LPM_LATCH: " << name()
+	<< " scope=" << scope_path(scope()) << endl;
+      dump_node_pins(o, ind+4);
+      dump_obj_attr(o, ind+4);
+}
+
 void NetLiteral::dump_node(ostream&o, unsigned ind) const
 {
       o << setw(ind) << "" << "constant real " << real_
@@ -721,6 +741,12 @@ void NetLogic::dump_node(ostream&o, unsigned ind) const
 	  case CMOS:
 	    o << "cmos";
 	    break;
+	  case EQUIV:
+	    o << "<->";
+	    break;
+	  case IMPL:
+	    o << "->";
+	    break;
 	  case NAND:
 	    o << "nand";
 	    break;
@@ -976,6 +1002,18 @@ void NetProcTop::dump(ostream&o, unsigned ind) const
 	    o << "always  /* " << get_fileline() << " in "
 	      << scope_path(scope_) << " */" << endl;
 	    break;
+	  case IVL_PR_ALWAYS_COMB:
+	    o << "always_comb  /* " << get_fileline() << " in "
+	      << scope_path(scope_) << " */" << endl;
+	    break;
+	  case IVL_PR_ALWAYS_FF:
+	    o << "always_ff  /* " << get_fileline() << " in "
+	      << scope_path(scope_) << " */" << endl;
+	    break;
+	  case IVL_PR_ALWAYS_LATCH:
+	    o << "always_latch  /* " << get_fileline() << " in "
+	      << scope_path(scope_) << " */" << endl;
+	    break;
 	  case IVL_PR_FINAL:
 	    o << "final  /* " << get_fileline() << " in "
 	      << scope_path(scope_) << " */" << endl;
@@ -1003,6 +1041,13 @@ void NetAnalogTop::dump(ostream&o, unsigned ind) const
 	      << scope_path(scope_) << " */" << endl;
 	    break;
 
+	    // These are not used in an analog context.
+	  case IVL_PR_ALWAYS_COMB:
+	  case IVL_PR_ALWAYS_FF:
+	  case IVL_PR_ALWAYS_LATCH:
+	    assert(0);
+	    break;
+
 	  case IVL_PR_FINAL:
 	    o << "analog final /* " << get_fileline() << " in "
 	      << scope_path(scope_) << " */" << endl;
@@ -1020,7 +1065,11 @@ void NetAlloc::dump(ostream&o, unsigned ind) const
 void NetAssign_::dump_lval(ostream&o) const
 {
       if (sig_) o << sig_->name();
-      else if (nest_) nest_->dump_lval(o);
+      else if (nest_) {
+	    o << "(";
+	    nest_->dump_lval(o);
+	    o << ")";
+      }
       else o << "<?>";
 
       if (! member_.nil()) {
@@ -1113,15 +1162,29 @@ void NetBlock::dump(ostream&o, unsigned ind) const
 
 void NetCase::dump(ostream&o, unsigned ind) const
 {
+      o << setw(ind) << "";
+      switch (quality_) {
+	  case IVL_CASE_QUALITY_BASIC:
+	    break;
+	  case IVL_CASE_QUALITY_UNIQUE:
+	    o << "unique ";
+	    break;
+	  case IVL_CASE_QUALITY_UNIQUE0:
+	    o << "unique0 ";
+	    break;
+	  case IVL_CASE_QUALITY_PRIORITY:
+	    o << "priority ";
+	    break;
+      }
       switch (type_) {
 	  case EQ:
-	    o << setw(ind) << "" << "case (" << *expr_ << ")" << endl;
+	    o << "case (" << *expr_ << ")" << endl;
 	    break;
 	  case EQX:
-	    o << setw(ind) << "" << "casex (" << *expr_ << ")" << endl;
+	    o << "casex (" << *expr_ << ")" << endl;
 	    break;
 	  case EQZ:
-	    o << setw(ind) << "" << "casez (" << *expr_ << ")" << endl;
+	    o << "casez (" << *expr_ << ")" << endl;
 	    break;
       }
 
@@ -1382,6 +1445,8 @@ void NetScope::dump(ostream&o) const
       if (is_interface()) o << " (interface)";
       o << " " << children_.size() << " children, "
 	<< classes_.size() << " classes" << endl;
+      if (unit() && !is_unit())
+	    o << "    in compilation unit " << unit()->basename() << endl;
 
       for (unsigned idx = 0 ;  idx < attr_cnt() ;  idx += 1)
 	    o << "    (* " << attr_key(idx) << " = "
@@ -1621,11 +1686,14 @@ void NetEBinary::dump(ostream&o) const
 	  case 'A':
 	    o << "~&";
 	    break;
+	  case 'e':
+	    o << "==";
+	    break;
 	  case 'E':
 	    o << "===";
 	    break;
-	  case 'e':
-	    o << "==";
+	  case 'w':
+	    o << "==?";
 	    break;
 	  case 'G':
 	    o << ">=";
@@ -1642,6 +1710,9 @@ void NetEBinary::dump(ostream&o) const
 	  case 'N':
 	    o << "!==";
 	    break;
+	  case 'W':
+	    o << "!=?";
+	    break;
 	  case 'o':
 	    o << "||";
 	    break;
@@ -1651,6 +1722,12 @@ void NetEBinary::dump(ostream&o) const
 	  case 'p':
 	    o << "**";
 	    break;
+	  case 'q':
+	    o << "->";
+	    break;
+	  case 'Q':
+	    o << "<->";
+	    break;
 	  case 'r':
 	    o << ">>";
 	    break;
@@ -1670,7 +1747,7 @@ void NetECast::dump(ostream&fd) const
 {
       if (op_=='2')
 	    fd << "bool<" << expr_width() << ">(" << *expr_ << ")";
-      else if (op_=='4')
+      else if (op_=='v')
 	    fd << "logic<" << expr_width() << ">(" << *expr_ << ")";
       else
 	    NetEUnary::dump(fd);
@@ -1894,18 +1971,6 @@ void Design::dump(ostream&o) const
 	    cur->second->dump(o);
       }
 
-      o << "$ROOT CLASSES:" << endl;
-      for (map<perm_string,netclass_t*>::const_iterator cur = classes_.begin()
-		 ; cur != classes_.end() ; ++cur) {
-	    cur->second->dump_scope(o);
-      }
-
-      o << "$ROOT TASKS/FUNCTIONS:" << endl;
-      for (map<NetScope*,PTaskFunc*>::const_iterator cur = root_tasks_.begin()
-		 ; cur != root_tasks_.end() ; ++ cur) {
-	    cur->first->dump(o);
-      }
-
       o << "SCOPES:" << endl;
       for (list<NetScope*>::const_iterator scope = root_scopes_.begin();
 	   scope != root_scopes_.end(); ++ scope ) {
