@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -78,9 +78,13 @@ static void elaborate_array_ranges(Design*des, NetScope*scope,
  */
 ivl_type_s* data_type_t::elaborate_type(Design*des, NetScope*scope)
 {
+	// User-defined types must be elaborated in the context
+	// where they were defined.
+      if (!name.nil())
+	    scope = scope->find_typedef_scope(des, this);
+
+      ivl_assert(*this, scope);
       Definitions*use_definitions = scope;
-      if (use_definitions == 0)
-	    use_definitions = des;
 
       map<Definitions*,ivl_type_s*>::iterator pos = cache_type_elaborate_.lower_bound(use_definitions);
 	  if (pos != cache_type_elaborate_.end() && pos->first == use_definitions)
@@ -147,13 +151,13 @@ ivl_type_s* class_type_t::elaborate_type_raw(Design*, NetScope*) const
  * available at the right time. At that time, the netenum_t* object is
  * stashed in the scope so that I can retrieve it here.
  */
-ivl_type_s* enum_type_t::elaborate_type_raw(Design*des, NetScope*scope) const
+ivl_type_s* enum_type_t::elaborate_type_raw(Design*, NetScope*scope) const
 {
       ivl_assert(*this, scope);
       ivl_type_s*tmp = scope->enumeration_for_key(this);
-      if (tmp) return tmp;
-
-      tmp = des->enumeration_for_key(this);
+      if (tmp == 0 && scope->unit()) {
+	    tmp = scope->unit()->enumeration_for_key(this);
+}
       return tmp;
 }
 
@@ -216,9 +220,9 @@ netstruct_t* struct_type_t::elaborate_type_raw(Design*des, NetScope*scope) const
 	      // There may be several names that are the same type:
 	      //   <data_type> name1, name2, ...;
 	      // Process all the member, and give them a type.
-	    for (list<decl_assignment_t*>::iterator name = curp->names->begin()
-		       ; name != curp->names->end() ;  ++ name) {
-		  decl_assignment_t*namep = *name;
+	    for (list<decl_assignment_t*>::iterator cur_name = curp->names->begin()
+		       ; cur_name != curp->names->end() ;  ++ cur_name) {
+		  decl_assignment_t*namep = *cur_name;
 
 		  netstruct_t::member_t memb;
 		  memb.name = namep->name;
@@ -248,19 +252,20 @@ ivl_type_s* uarray_type_t::elaborate_type_raw(Design*des, NetScope*scope) const
       }
 
 	// Special case: if the dimension is null:nil. this is a queue.
-      if (cur->second==0 && dynamic_cast<PENull*>(cur->first)) {
+      if (dynamic_cast<PENull*>(cur->first)) {
 	    cerr << get_fileline() << ": sorry: "
 		 << "SV queues inside classes are not yet supported." << endl;
 	    des->errors += 1;
 
-	    ivl_type_s*res = new netqueue_t(btype);
+	      // FIXME: Need to set the max size if cur->second is defined
+	    ivl_type_s*res = new netqueue_t(btype, -1);
 	    return res;
       }
 
       vector<netrange_t> dimensions;
-      bool bad_range = evaluate_ranges(des, scope, dimensions, *dims);
+      bool dimensions_ok = evaluate_ranges(des, scope, this, dimensions, *dims);
 
-      if (bad_range) {
+      if (!dimensions_ok) {
 	    cerr << get_fileline() << " : warning: "
 		 << "Bad dimensions for type here." << endl;
       }
