@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -274,7 +274,18 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 
       if (op_ == 'E' || op_ == 'N') {
 	    NetCaseCmp*gate = new NetCaseCmp(scope, scope->local_symbol(),
-					     width, op_=='E'?NetCaseCmp::EEQ:NetCaseCmp::NEQ);
+					     width, op_=='E' ? NetCaseCmp::EEQ : NetCaseCmp::NEQ);
+	    gate->set_line(*this);
+	    connect(gate->pin(0), osig->pin(0));
+	    connect(gate->pin(1), lsig->pin(0));
+	    connect(gate->pin(2), rsig->pin(0));
+	    des->add_node(gate);
+	    return osig;
+      }
+
+      if (op_ == 'w' || op_ == 'W') {
+	    NetCaseCmp*gate = new NetCaseCmp(scope, scope->local_symbol(),
+					     width, op_=='w' ? NetCaseCmp::WEQ : NetCaseCmp::WNE);
 	    gate->set_line(*this);
 	    connect(gate->pin(0), osig->pin(0));
 	    connect(gate->pin(1), lsig->pin(0));
@@ -335,6 +346,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 		  des->errors += 1;
 		  return 0;
 	    }
+	    // fallthrough
 	  case 'e': // ==
 	    connect(dev->pin_AEB(), osig->pin(0));
 	    break;
@@ -353,6 +365,7 @@ NetNet* NetEBComp::synthesize(Design*des, NetScope*scope, NetExpr*root)
 		  des->errors += 1;
 		  return 0;
 	    }
+	    // fallthrough
 	  case 'n': // !=
 	    connect(dev->pin_ANEB(), osig->pin(0));
 	    break;
@@ -386,7 +399,8 @@ NetNet* NetEBPow::synthesize(Design*des, NetScope*scope, NetExpr*root)
       powr->set_line(*this);
       des->add_node(powr);
 
-      powr->set_signed( has_sign() );
+        // The lpm_pwr object only cares about the signedness of the exponent.
+      powr->set_signed( right_->has_sign() );
 
       connect(powr->pin_DataA(), lsig->pin(0));
       connect(powr->pin_DataB(), rsig->pin(0));
@@ -531,27 +545,40 @@ NetNet* NetEBLogic::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    return 0;
       }
 
-      netvector_t*osig_tmp = new netvector_t(expr_type());
-      NetNet*osig = new NetNet(scope, scope->local_symbol(),
-			       NetNet::IMPLICIT, osig_tmp);
-      osig->set_line(*this);
-      osig->local_flag(true);
-
       NetLogic*olog;
       perm_string oname = scope->local_symbol();
 
 	/* Create the logic OR/AND gate. This has a single bit output,
 	 * with single bit inputs for the two operands. */
-      if (op() == 'o') {
-	    olog = new NetLogic(scope, oname, 3, NetLogic::OR, 1, true);
-      } else {
-	    assert(op() == 'a');
+      switch (op()) {
+	case 'a':
 	    olog = new NetLogic(scope, oname, 3, NetLogic::AND, 1, true);
+	    break;
+	case 'o':
+	    olog = new NetLogic(scope, oname, 3, NetLogic::OR, 1, true);
+	    break;
+	case 'q':
+	    olog = new NetLogic(scope, oname, 3, NetLogic::IMPL, 1, true);
+	    break;
+	case 'Q':
+	    olog = new NetLogic(scope, oname, 3, NetLogic::EQUIV, 1, true);
+	    break;
+	default:
+	    cerr << get_fileline() << ": sorry: "
+	         << human_readable_op(op_)
+	         << " is not currently supported." << endl;
+	    des->errors += 1;
+	    return 0;
       }
-
       olog->set_line(*this);
       des->add_node(olog);
 
+      netvector_t*osig_tmp = new netvector_t(expr_type());
+      NetNet*osig = new NetNet(scope, scope->local_symbol(),
+			       NetNet::IMPLICIT, osig_tmp);
+      osig->set_line(*this);
+      osig->local_flag(true);
+
       connect(osig->pin(0), olog->pin(0));
 
 	/* The left and right operands have already been reduced to a
@@ -733,12 +760,18 @@ NetNet* NetEConcat::synthesize(Design*des, NetScope*scope, NetExpr*root)
 	    }
       }
 
-      if (flag == false) return 0;
+      if (flag == false) {
+	    delete[]tmp;
+	    return 0;
+      }
 
       ivl_assert(*this, data_type != IVL_VT_NO_TYPE);
 
 	/* If this is a replication of zero just return 0. */
-      if (expr_width() == 0) return 0;
+      if (expr_width() == 0) {
+	    delete[]tmp;
+	    return 0;
+      }
 
 	/* Make a NetNet object to carry the output vector. */
       perm_string path = scope->local_symbol();
@@ -1341,6 +1374,7 @@ static NetEvWait* make_func_trigger(Design*des, NetScope*scope, NetExpr*root)
       if (nset && (nset->size() > 0)) {
             NetEvent*ev = new NetEvent(scope->local_symbol());
             ev->set_line(*root);
+            ev->local_flag(true);
 
             NetEvProbe*pr = new NetEvProbe(scope, scope->local_symbol(),
                                            ev, NetEvProbe::ANYEDGE,
