@@ -73,7 +73,8 @@ bool load_module(const char*type, int&parser_errors)
 	    if (cur == lcur->name_map.end())
 		  continue;
 
-	    sprintf(path, "%s%c%s", lcur->dir, dir_character, (*cur).second);
+	    snprintf(path, sizeof(path), "%s%c%s",
+		     lcur->dir, dir_character, (*cur).second);
 
 	    if(depend_file) {
                   if (depfile_mode == 'p') {
@@ -84,36 +85,10 @@ bool load_module(const char*type, int&parser_errors)
 		  fflush(depend_file);
 	    }
 
-	    if (ivlpp_string) {
-		  char*cmdline = (char*)malloc(strlen(ivlpp_string) +
-					       strlen(path) + 4);
-		  strcpy(cmdline, ivlpp_string);
-		  strcat(cmdline, " \"");
-		  strcat(cmdline, path);
-		  strcat(cmdline, "\"");
+	    if (verbose_flag)
+		  cerr << "Loading library file " << path << "." << endl;
 
-		  if (verbose_flag)
-			cerr << "Executing: " << cmdline << endl<< flush;
-
-		  FILE*file = popen(cmdline, "r");
-
-		  if (verbose_flag)
-			cerr << "...parsing output from preprocessor..." << endl << flush;
-
-		  parser_errors = pform_parse(path, file);
-		  pclose(file);
-		  free(cmdline);
-
-	    } else {
-		  if (verbose_flag)
-			cerr << "Loading library file "
-			     << path << "." << endl;
-
-		  FILE*file = fopen(path, "r");
-		  assert(file);
-		  parser_errors = pform_parse(path, file);
-		  fclose(file);
-	    }
+	    parser_errors = pform_parse(path);
 
 	    if (verbose_flag)
 		  cerr << "... Load module complete." << endl << flush;
@@ -121,7 +96,6 @@ bool load_module(const char*type, int&parser_errors)
 	    return parser_errors == 0;
       }
 
-
       return false;
 }
 
