@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2013 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2016 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -481,6 +481,17 @@ NetNet* Nexus::pick_any_net()
       return 0;
 }
 
+NetNode* Nexus::pick_any_node()
+{
+      for (Link*cur = first_nlink() ; cur ; cur = cur->next_nlink()) {
+	    NetNode*node = dynamic_cast<NetNode*>(cur->get_obj());
+	    if (node != 0)
+		  return node;
+      }
+
+      return 0;
+}
+
 const char* Nexus::name() const
 {
       if (name_)
