@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -19,37 +19,37 @@
 
 # include "config.h"
 
-# include <iostream>
-
+# include  <iostream>
+# include  <set>
 # include  <cassert>
 # include  <typeinfo>
 # include  "compiler.h"
 # include  "netlist.h"
 # include  "netmisc.h"
 
-NexusSet* NetExpr::nex_input(bool)
+NexusSet* NetExpr::nex_input(bool, bool, bool) const
 {
       cerr << get_fileline()
 	   << ": internal error: nex_input not implemented: "
 	   << *this << endl;
-      return 0;
+      return new NexusSet;
 }
 
-NexusSet* NetProc::nex_input(bool)
+NexusSet* NetProc::nex_input(bool, bool, bool) const
 {
       cerr << get_fileline()
 	   << ": internal error: NetProc::nex_input not implemented"
 	   << endl;
-      return 0;
+      return new NexusSet;
 }
 
-NexusSet* NetEArrayPattern::nex_input(bool rem_out)
+NexusSet* NetEArrayPattern::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
       NexusSet*result = new NexusSet;
       for (size_t idx = 0 ; idx < items_.size() ; idx += 1) {
 	    if (items_[idx]==0) continue;
 
-	    NexusSet*tmp = items_[idx]->nex_input(rem_out);
+	    NexusSet*tmp = items_[idx]->nex_input(rem_out, always_sens, nested_func);
 	    if (tmp == 0) continue;
 
 	    result->add(*tmp);
@@ -58,32 +58,32 @@ NexusSet* NetEArrayPattern::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetEBinary::nex_input(bool rem_out)
+NexusSet* NetEBinary::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = left_->nex_input(rem_out);
-      NexusSet*tmp = right_->nex_input(rem_out);
+      NexusSet*result = left_->nex_input(rem_out, always_sens, nested_func);
+      NexusSet*tmp = right_->nex_input(rem_out, always_sens, nested_func);
       result->add(*tmp);
       delete tmp;
       return result;
 }
 
-NexusSet* NetEConcat::nex_input(bool rem_out)
+NexusSet* NetEConcat::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      if (parms_[0] == NULL) return NULL;
-      NexusSet*result = parms_[0]->nex_input(rem_out);
+      if (parms_[0] == NULL) return new NexusSet;
+      NexusSet*result = parms_[0]->nex_input(rem_out, always_sens, nested_func);
       for (unsigned idx = 1 ;  idx < parms_.size() ;  idx += 1) {
 	    if (parms_[idx] == NULL) {
 		  delete result;
-		  return NULL;
+		  return new NexusSet;
 	    }
-	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
       return result;
 }
 
-NexusSet* NetEAccess::nex_input(bool)
+NexusSet* NetEAccess::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
@@ -91,63 +91,73 @@ NexusSet* NetEAccess::nex_input(bool)
 /*
  * A constant has not inputs, so always return an empty set.
  */
-NexusSet* NetEConst::nex_input(bool)
+NexusSet* NetEConst::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetECReal::nex_input(bool)
+NexusSet* NetECReal::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetEEvent::nex_input(bool)
+NexusSet* NetEEvent::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetELast::nex_input(bool)
+NexusSet* NetELast::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetENetenum::nex_input(bool)
+NexusSet* NetENetenum::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetENew::nex_input(bool)
+NexusSet* NetENew::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetENull::nex_input(bool)
+NexusSet* NetENull::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetEProperty::nex_input(bool)
+NexusSet* NetEProperty::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetEScope::nex_input(bool)
+NexusSet* NetEScope::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetESelect::nex_input(bool rem_out)
+NexusSet* NetESelect::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = base_? base_->nex_input(rem_out) : new NexusSet();
-      NexusSet*tmp = expr_->nex_input(rem_out);
-      if (tmp == NULL) {
-	    delete result;
-	    return NULL;
+      NexusSet*result = base_? base_->nex_input(rem_out, always_sens, nested_func) : new NexusSet();
+      NexusSet*tmp = expr_->nex_input(rem_out, always_sens, nested_func);
+      bool const_select = result->size() == 0;
+      if (always_sens && const_select) {
+	    if (NetEConst *val = dynamic_cast <NetEConst*> (base_)) {
+		  assert(select_type() == IVL_SEL_OTHER);
+		  if (NetESignal *sig = dynamic_cast<NetESignal*> (expr_)) {
+			delete tmp;
+			tmp = sig->nex_input_base(rem_out, always_sens, nested_func,
+                                                  val->value().as_unsigned(), expr_width());
+		  } else {
+			cerr << get_fileline() << ": Sorry, cannot determine the sensitivity "
+			     << "for the select of " << *expr_ << ", using all bits." << endl;
+		  }
+	    }
       }
       result->add(*tmp);
       delete tmp;
 	/* See the comment for NetESignal below. */
-      if (base_ && warn_sens_entire_vec) {
+      if (base_ && ! always_sens && warn_sens_entire_vec) {
 	    cerr << get_fileline() << ": warning: @* is sensitive to all "
 	            "bits in '" << *expr_ << "'." << endl;
       }
@@ -157,30 +167,35 @@ NexusSet* NetESelect::nex_input(bool rem_out)
 /*
  * The $fread, etc. system functions can have NULL arguments.
  */
-NexusSet* NetESFunc::nex_input(bool rem_out)
+NexusSet* NetESFunc::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      if (parms_.empty())
-	    return new NexusSet;
+      NexusSet*result = new NexusSet;
 
-      NexusSet*result;
-      if (parms_[0]) result = parms_[0]->nex_input(rem_out);
-      else result = new NexusSet;
-      for (unsigned idx = 1 ;  idx < parms_.size() ;  idx += 1) {
+      if (parms_.empty()) return result;
+
+      for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
 	    if (parms_[idx]) {
-		  NexusSet*tmp = parms_[idx]->nex_input(rem_out);
+		  NexusSet*tmp = parms_[idx]->nex_input(rem_out, always_sens, nested_func);
 		  result->add(*tmp);
 		  delete tmp;
 	    }
       }
+
       return result;
 }
 
-NexusSet* NetEShallowCopy::nex_input(bool)
+NexusSet* NetEShallowCopy::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetESignal::nex_input(bool rem_out)
+NexusSet* NetESignal::nex_input(bool rem_out, bool always_sens, bool nested_func) const
+{
+      return nex_input_base(rem_out, always_sens, nested_func, 0, 0);
+}
+
+NexusSet* NetESignal::nex_input_base(bool rem_out, bool always_sens, bool nested_func,
+                                     unsigned base, unsigned width) const
 {
 	/*
 	 * This is not what I would expect for the various selects (bit,
@@ -188,48 +203,88 @@ NexusSet* NetESignal::nex_input(bool rem_out)
 	 * instead of building the appropriate select and then using it
 	 * as the trigger. Other simulators also add everything.
 	 */
+      bool const_select = false;
+      unsigned const_word = 0;
       NexusSet*result = new NexusSet;
 	/* Local signals are not added to the sensitivity list. */
       if (net_->local_flag()) return result;
 	/* If we have an array index add it to the sensitivity list. */
       if (word_) {
 	    NexusSet*tmp;
-	    tmp = word_->nex_input(rem_out);
+	    tmp = word_->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
-            if (warn_sens_entire_arr) {
+            if (!always_sens && warn_sens_entire_arr) {
                   cerr << get_fileline() << ": warning: @* is sensitive to all "
                        << net_->unpacked_count() << " words in array '"
                        << name() << "'." << endl;
             }
+	    if (always_sens) if (NetEConst *val = dynamic_cast <NetEConst*> (word_)) {
+		  const_select = true;
+		  const_word = val->value().as_unsigned();
+	    }
+      }
+
+      if ((base == 0) && (width == 0)) width = net_->vector_width();
+
+      if (const_select) {
+	    result->add(net_->pin(const_word).nexus(), base, width);
+      } else {
+	    for (unsigned idx = 0 ;  idx < net_->pin_count() ;  idx += 1)
+		  result->add(net_->pin(idx).nexus(), base, width);
       }
-      for (unsigned idx = 0 ;  idx < net_->pin_count() ;  idx += 1)
-	    result->add(net_->pin(idx).nexus(), 0, net_->vector_width());
 
       return result;
 }
 
-NexusSet* NetETernary::nex_input(bool rem_out)
+NexusSet* NetETernary::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
       NexusSet*tmp;
-      NexusSet*result = cond_->nex_input(rem_out);
+      NexusSet*result = cond_->nex_input(rem_out, always_sens, nested_func);
 
-      tmp = true_val_->nex_input(rem_out);
+      tmp = true_val_->nex_input(rem_out, always_sens, nested_func);
       result->add(*tmp);
       delete tmp;
 
-      tmp = false_val_->nex_input(rem_out);
+      tmp = false_val_->nex_input(rem_out, always_sens, nested_func);
       result->add(*tmp);
       delete tmp;
 
       return result;
 }
 
-NexusSet* NetEUFunc::nex_input(bool rem_out)
+NexusSet* NetEUFunc::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
       NexusSet*result = new NexusSet;
+
       for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
-	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      if (always_sens) {
+	    NetFuncDef*func = func_->func_def();
+
+	      // Avoid recursive function calls.
+	    static set<NetFuncDef*> func_set;
+	    if (!nested_func)
+		  func_set.clear();
+
+	    if (!func_set.insert(func).second)
+		  return result;
+
+	    NexusSet*tmp = func->proc()->nex_input(rem_out, always_sens, true);
+	      // Remove the function inputs
+	    NexusSet*in = new NexusSet;
+	    for (unsigned idx = 0 ;  idx < func->port_count() ;  idx += 1) {
+		  NetNet*net = func->port(idx);
+		  assert(net->pin_count() == 1);
+		  in->add(net->pin(0).nexus(), 0, net->vector_width());
+	    }
+	    tmp->rem(*in);
+	    delete in;
+
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -237,21 +292,28 @@ NexusSet* NetEUFunc::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetEUnary::nex_input(bool rem_out)
+NexusSet* NetEUnary::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      return expr_->nex_input(rem_out);
+      return expr_->nex_input(rem_out, always_sens, nested_func);
 }
 
-NexusSet* NetAssign_::nex_input(bool rem_out)
+NexusSet* NetAlloc::nex_input(bool, bool, bool) const
 {
+      return new NexusSet;
+}
+
+NexusSet* NetAssign_::nex_input(bool rem_out, bool always_sens, bool nested_func) const
+{
+      assert(! nest_);
       NexusSet*result = new NexusSet;
+
       if (word_) {
-	    NexusSet*tmp = word_->nex_input(rem_out);
+	    NexusSet*tmp = word_->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
       if (base_) {
-	    NexusSet*tmp = base_->nex_input(rem_out);
+	    NexusSet*tmp = base_->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -259,15 +321,21 @@ NexusSet* NetAssign_::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetAssignBase::nex_input(bool rem_out)
+NexusSet* NetAssignBase::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = rval_->nex_input(rem_out);
+      NexusSet*result = new NexusSet;
+	// For the deassign and release statements there is no R-value.
+      if (rval_) {
+	    NexusSet*tmp = rval_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
 
 	/* It is possible that the lval_ can have nex_input values. In
 	   particular, index expressions are statement inputs as well,
 	   so should be addressed here. */
       for (NetAssign_*cur = lval_ ;  cur ;  cur = cur->more) {
-	    NexusSet*tmp = cur->nex_input(rem_out);
+	    NexusSet*tmp = cur->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -292,16 +360,15 @@ NexusSet* NetAssignBase::nex_input(bool rem_out)
  * In this example, "t" should not be in the input set because it is
  * used by the sequence as a temporary value.
  */
-NexusSet* NetBlock::nex_input(bool rem_out)
+NexusSet* NetBlock::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      if (last_ == 0)
-	    return new NexusSet;
+      if (last_ == 0) return new NexusSet;
 
-      if (type_ != SEQU) {
+      if (! always_sens && (type_ != SEQU)) {
 	    cerr << get_fileline() << ": internal error: Sorry, "
 		 << "I don't know how to synthesize fork/join blocks."
 		 << endl;
-	    return 0;
+	    return new NexusSet;
       }
 
       NetProc*cur = last_->next_;
@@ -312,10 +379,10 @@ NexusSet* NetBlock::nex_input(bool rem_out)
 
       do {
 	      /* Get the inputs for the current statement. */
-	    NexusSet*tmp = cur->nex_input(rem_out);
+	    NexusSet*tmp = cur->nex_input(rem_out, always_sens, nested_func);
 
 	      /* Add the current input set to the accumulated input set. */
-	    if (tmp != 0) result->add(*tmp);
+	    result->add(*tmp);
 	    delete tmp;
 
 	      /* Add the current outputs to the accumulated output set if
@@ -339,11 +406,9 @@ NexusSet* NetBlock::nex_input(bool rem_out)
  * the inputs to all the guards, and the inputs to all the guarded
  * statements.
  */
-NexusSet* NetCase::nex_input(bool rem_out)
+NexusSet* NetCase::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = expr_->nex_input(rem_out);
-      if (result == 0)
-	    return 0;
+      NexusSet*result = expr_->nex_input(rem_out, always_sens, nested_func);
 
       for (size_t idx = 0 ;  idx < items_.size() ;  idx += 1) {
 
@@ -351,8 +416,7 @@ NexusSet* NetCase::nex_input(bool rem_out)
 	    if (items_[idx].statement == 0)
 		  continue;
 
-	    NexusSet*tmp = items_[idx].statement->nex_input(rem_out);
-	    assert(tmp);
+	    NexusSet*tmp = items_[idx].statement->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
 
@@ -360,8 +424,7 @@ NexusSet* NetCase::nex_input(bool rem_out)
 		 case is special and is identified by a null
 		 guard. The default guard obviously has no input. */
 	    if (items_[idx].guard) {
-		  tmp = items_[idx].guard->nex_input(rem_out);
-		  assert(tmp);
+		  tmp = items_[idx].guard->nex_input(rem_out, always_sens, nested_func);
 		  result->add(*tmp);
 		  delete tmp;
 	    }
@@ -370,24 +433,18 @@ NexusSet* NetCase::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetCAssign::nex_input(bool)
+NexusSet* NetCondit::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      cerr << get_fileline() << ": internal warning: NetCAssign::nex_input()"
-	   << " not implemented." << endl;
-      return new NexusSet;
-}
+      NexusSet*result = expr_->nex_input(rem_out, always_sens, nested_func);
 
-NexusSet* NetCondit::nex_input(bool rem_out)
-{
-      NexusSet*result = expr_->nex_input(rem_out);
       if (if_ != 0) {
-	    NexusSet*tmp = if_->nex_input(rem_out);
+	    NexusSet*tmp = if_->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
 
       if (else_ != 0) {
-	    NexusSet*tmp = else_->nex_input(rem_out);
+	    NexusSet*tmp = else_->nex_input(rem_out, always_sens, nested_func);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -395,51 +452,85 @@ NexusSet* NetCondit::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetDoWhile::nex_input(bool rem_out)
+NexusSet* NetDisable::nex_input(bool, bool, bool) const
 {
-      NexusSet*result = proc_->nex_input(rem_out);
-      NexusSet*tmp = cond_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
-      return result;
-}
-
-NexusSet* NetEvWait::nex_input(bool rem_out)
-{
-      NexusSet*result;
-      if (statement_)
-	    result = statement_->nex_input(rem_out);
-      else
-	    result = new NexusSet;
-
-      return result;
-}
-
-NexusSet* NetForce::nex_input(bool)
-{
-      cerr << get_fileline() << ": internal warning: NetForce::nex_input()"
-	   << " not implemented." << endl;
       return new NexusSet;
 }
 
-NexusSet* NetForLoop::nex_input(bool rem_out)
+NexusSet* NetDoWhile::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = init_expr_->nex_input(rem_out);
+      NexusSet*result = cond_->nex_input(rem_out, always_sens, nested_func);
 
-      NexusSet*tmp = condition_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
+      if (proc_) {
+	    NexusSet*tmp = proc_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
 
-      tmp = statement_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
+      return result;
+}
 
-      tmp = step_statement_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
+NexusSet* NetEvTrig::nex_input(bool, bool, bool) const
+{
+      return new NexusSet;
+}
+
+NexusSet* NetEvWait::nex_input(bool rem_out, bool always_sens, bool nested_func) const
+{
+      NexusSet*result = new NexusSet;
+
+      if (statement_) {
+	    NexusSet*tmp = statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      return result;
+}
+
+NexusSet* NetForever::nex_input(bool rem_out, bool always_sens, bool nested_func) const
+{
+      NexusSet*result = new NexusSet;
+
+      if (statement_) {
+	    NexusSet*tmp = statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      return result;
+}
+
+NexusSet* NetForLoop::nex_input(bool rem_out, bool always_sens, bool nested_func) const
+{
+      NexusSet*result = new NexusSet;
+
+      if (init_expr_) {
+	    NexusSet*tmp = init_expr_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      if (condition_) {
+	    NexusSet*tmp = condition_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      if (step_statement_) {
+	    NexusSet*tmp = step_statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
+      if (statement_) {
+	    NexusSet*tmp = statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
 
       if (gn_shared_loop_index_flag) {
-	    tmp = new NexusSet();
+	    NexusSet*tmp = new NexusSet();
 	    for (unsigned idx = 0 ; idx < index_->pin_count() ; idx += 1)
 		tmp->add(index_->pin(idx).nexus(), 0, index_->vector_width());
 
@@ -450,10 +541,9 @@ NexusSet* NetForLoop::nex_input(bool rem_out)
       return result;
 }
 
-NexusSet* NetForever::nex_input(bool rem_out)
+NexusSet* NetFree::nex_input(bool, bool, bool) const
 {
-      NexusSet*result = statement_->nex_input(rem_out);
-      return result;
+      return new NexusSet;
 }
 
 /*
@@ -465,36 +555,44 @@ NexusSet* NetForever::nex_input(bool rem_out)
  * include the input set of the <expr> because it does not affect the
  * result. The statement can be omitted.
  */
-NexusSet* NetPDelay::nex_input(bool rem_out)
+NexusSet* NetPDelay::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      if (statement_ == 0) return 0;
-      NexusSet*result = statement_->nex_input(rem_out);
+      NexusSet*result = new NexusSet;
+
+      if (statement_) {
+	    NexusSet*tmp = statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
       return result;
 }
 
-NexusSet* NetRepeat::nex_input(bool rem_out)
+NexusSet* NetRepeat::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = statement_->nex_input(rem_out);
-      NexusSet*tmp = expr_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
+      NexusSet*result = expr_->nex_input(rem_out, always_sens, nested_func);
+
+      if (statement_) {
+	    NexusSet*tmp = statement_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
       return result;
 }
 
 /*
  * The $display, etc. system tasks can have NULL arguments.
  */
-NexusSet* NetSTask::nex_input(bool rem_out)
+NexusSet* NetSTask::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      if (parms_.empty())
-	    return new NexusSet;
+      NexusSet*result = new NexusSet;
 
-      NexusSet*result;
-      if (parms_[0]) result = parms_[0]->nex_input(rem_out);
-      else result = new NexusSet;
-      for (unsigned idx = 1 ;  idx < parms_.size() ;  idx += 1) {
+      if (parms_.empty()) return result;
+
+      for (unsigned idx = 0 ;  idx < parms_.size() ;  idx += 1) {
 	    if (parms_[idx]) {
-		  NexusSet*tmp = parms_[idx]->nex_input(rem_out);
+		  NexusSet*tmp = parms_[idx]->nex_input(rem_out, always_sens, nested_func);
 		  result->add(*tmp);
 		  delete tmp;
 	    }
@@ -508,16 +606,20 @@ NexusSet* NetSTask::nex_input(bool rem_out)
  * parameters to consider, because the compiler already removed them
  * and converted them to blocking assignments.
  */
-NexusSet* NetUTask::nex_input(bool)
+NexusSet* NetUTask::nex_input(bool, bool, bool) const
 {
       return new NexusSet;
 }
 
-NexusSet* NetWhile::nex_input(bool rem_out)
+NexusSet* NetWhile::nex_input(bool rem_out, bool always_sens, bool nested_func) const
 {
-      NexusSet*result = proc_->nex_input(rem_out);
-      NexusSet*tmp = cond_->nex_input(rem_out);
-      result->add(*tmp);
-      delete tmp;
+      NexusSet*result = cond_->nex_input(rem_out, always_sens, nested_func);
+
+      if (proc_) {
+	    NexusSet*tmp = proc_->nex_input(rem_out, always_sens, nested_func);
+	    result->add(*tmp);
+	    delete tmp;
+      }
+
       return result;
 }
