@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2014 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2015 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -83,8 +83,8 @@ const NetProc* NetBlock::proc_next(const NetProc*cur) const
       return cur->next_;
 }
 
-NetCase::NetCase(NetCase::TYPE c, NetExpr*ex, unsigned cnt)
-: type_(c), expr_(ex), items_(cnt)
+NetCase::NetCase(ivl_case_quality_t q, NetCase::TYPE c, NetExpr*ex, unsigned cnt)
+: quality_(q), type_(c), expr_(ex), items_(cnt)
 {
       ivl_assert(*this, expr_);
 }
@@ -199,6 +199,7 @@ NetForever::~NetForever()
 NetForLoop::NetForLoop(NetNet*ind, NetExpr*iexpr, NetExpr*cond, NetProc*sub, NetProc*step)
 : index_(ind), init_expr_(iexpr), condition_(cond), statement_(sub), step_statement_(step)
 {
+      as_block_ = NULL;
 }
 
 void NetForLoop::wrap_up()
