@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2000-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2016 CERN Michele Castellana (michele.castellana@cern.ch)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -23,6 +24,8 @@
 # include  "netlist.h"
 # include  "netclass.h"
 # include  "netenum.h"
+# include  "netvector.h"
+# include  "PPackage.h"
 # include  <cstring>
 # include  <cstdlib>
 # include  <sstream>
@@ -110,11 +113,13 @@ void Definitions::add_class(netclass_t*net_class)
  * in question.
  */
 
-NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, bool nest,
-		   bool program, bool interface)
+NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, NetScope*in_unit,
+		   bool nest, bool program, bool interface, bool compilation_unit)
 : type_(t), name_(n), nested_module_(nest), program_block_(program),
-  is_interface_(interface), up_(up)
+  is_interface_(interface), is_unit_(compilation_unit), unit_(in_unit), up_(up)
 {
+      imports_ = 0;
+      typedefs_ = 0;
       events_ = 0;
       lcounter_ = 0;
       is_auto_ = false;
@@ -122,8 +127,10 @@ NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, bool nest,
       calls_stask_ = false;
       in_final_ = false;
 
+      if (compilation_unit)
+	    unit_ = this;
+
       if (up) {
-	    assert(t!=CLASS);
 	    need_const_func_ = up->need_const_func_;
 	    is_const_func_ = up->is_const_func_;
 	    time_unit_ = up->time_unit();
@@ -131,6 +138,8 @@ NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, bool nest,
 	    time_from_timescale_ = up->time_from_timescale();
 	      // Need to check for duplicate names?
 	    up_->children_[name_] = this;
+	    if (unit_ == 0)
+		  unit_ = up_->unit_;
       } else {
 	    need_const_func_ = false;
 	    is_const_func_ = false;
@@ -162,6 +171,8 @@ NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, bool nest,
       lineno_ = 0;
       def_lineno_ = 0;
       genvar_tmp_val = 0;
+      tie_hi_ = 0;
+      tie_lo_ = 0;
 }
 
 NetScope::~NetScope()
@@ -196,16 +207,72 @@ void NetScope::set_line(perm_string file, perm_string def_file,
       def_lineno_ = def_lineno;
 }
 
+void NetScope::add_imports(const map<perm_string,PPackage*>*imports)
+{
+      if (!imports->empty())
+	    imports_ = imports;
+}
+
+NetScope*NetScope::find_import(const Design*des, perm_string name)
+{
+      if (imports_ == 0)
+	    return 0;
+
+      map<perm_string,PPackage*>::const_iterator cur = imports_->find(name);
+      if (cur != imports_->end()) {
+            return des->find_package(cur->second->pscope_name());
+      } else
+            return 0;
+}
+
+void NetScope::add_typedefs(const map<perm_string,data_type_t*>*typedefs)
+{
+      if (!typedefs->empty())
+	    typedefs_ = typedefs;
+}
+
+NetScope*NetScope::find_typedef_scope(const Design*des, data_type_t*type)
+{
+      assert(type);
+
+      NetScope *cur_scope = this;
+      while (cur_scope) {
+	    if (cur_scope->typedefs_ && cur_scope->typedefs_->find(type->name) != cur_scope->typedefs_->end())
+		  return cur_scope;
+	    NetScope*import_scope = cur_scope->find_import(des, type->name);
+	    if (import_scope)
+		  cur_scope = import_scope;
+	    else if (cur_scope == unit_)
+		  return 0;
+	    else
+		  cur_scope = cur_scope->parent();
+
+	    if (cur_scope == 0)
+		  cur_scope = unit_;
+      }
+
+      return 0;
+}
+
 /*
  * Look for the enumeration in the current scope and any parent scopes.
  */
-const netenum_t*NetScope::find_enumeration_for_name(perm_string name)
+const netenum_t*NetScope::find_enumeration_for_name(const Design*des, perm_string name)
 {
       NetScope *cur_scope = this;
       while (cur_scope) {
 	    NetEConstEnum*tmp = cur_scope->enum_names_[name];
 	    if (tmp) break;
-	    cur_scope = cur_scope->parent();
+	    NetScope*import_scope = cur_scope->find_import(des, name);
+	    if (import_scope)
+		  cur_scope = import_scope;
+	    else if (cur_scope == unit_)
+		  return 0;
+	    else
+		  cur_scope = cur_scope->parent();
+
+	    if (cur_scope == 0)
+		  cur_scope = unit_;
       }
 
       assert(cur_scope);
@@ -360,12 +427,7 @@ const NetExpr* NetScope::get_parameter(Design*des,
       msb = 0;
       lsb = 0;
       const NetExpr*tmp = enumeration_expr(key);
-      if (tmp) return tmp;
-
-      tmp = des->enumeration_expr(key);
-      if (tmp) return tmp;
-
-      return 0;
+      return tmp;
 }
 
 NetScope::param_ref_t NetScope::find_parameter(perm_string key)
@@ -382,11 +444,6 @@ NetScope::param_ref_t NetScope::find_parameter(perm_string key)
       return idx;
 }
 
-NetScope::TYPE NetScope::type() const
-{
-      return type_;
-}
-
 void NetScope::print_type(ostream&stream) const
 {
       switch (type_) {
@@ -505,6 +562,7 @@ void NetScope::add_module_port_info( unsigned idx, perm_string name, PortType::E
                                 unsigned long width )
 {
       assert(type_ == MODULE);
+      assert(ports_.size() > idx);
       PortInfo &info = ports_[idx];
       info.name = name;
       info.type = ptype;
@@ -652,15 +710,11 @@ netclass_t*NetScope::find_class(perm_string name)
       if (type_==CLASS && name_==hname_t(name))
 	    return class_def_;
 
-	// Look for the class that directly within this scope.
+	// Look for the class directly within this scope.
       map<perm_string,netclass_t*>::const_iterator cur = classes_.find(name);
       if (cur != classes_.end())
 	    return cur->second;
 
-	// If this is a module scope, then look no further.
-      if (type_==MODULE)
-	    return 0;
-
       if (up_==0 && type_==CLASS) {
 	    assert(class_def_);
 
@@ -668,12 +722,16 @@ netclass_t*NetScope::find_class(perm_string name)
 	    return def_parent->find_class(name);
       }
 
-	// If there is no further to look, ...
-      if (up_ == 0)
-	    return 0;
-
 	// Try looking up for the class.
-      return up_->find_class(name);
+      if (up_!=0 && type_!=MODULE)
+	    return up_->find_class(name);
+
+	// Try the compilation unit.
+      if (unit_ != 0)
+	    return unit_->find_class(name);
+
+	// Nowhere left to try...
+      return 0;
 }
 
 /*
@@ -741,7 +799,56 @@ const NetScope* NetScope::child_byname(perm_string name) const
 
 perm_string NetScope::local_symbol()
 {
-      ostringstream res;
-      res << "_s" << (lcounter_++);
-      return lex_strings.make(res.str());
+      perm_string sym;
+      do {
+	    ostringstream res;
+	    res << "_ivl_" << (lcounter_++);
+	    perm_string sym_tmp = lex_strings.make(res.str());
+
+	      // If the name already exists as a signal, try again.
+	    if (signals_map_.find(sym_tmp) != signals_map_.end())
+		  continue;
+	      // If the name already exists as a parameter, try again.
+	    if (parameters.find(sym_tmp) != parameters.end())
+		  continue;
+	    if (genvars_.find(sym_tmp) != genvars_.end())
+		  continue;
+	      // If the name already exists as a class, try again.
+	    if (classes_.find(sym_tmp) != classes_.end())
+		  continue;
+
+	      // No collisions, this is the one.
+	    sym = sym_tmp;
+      } while (sym.nil());
+      return sym;
+}
+
+void NetScope::add_tie_hi(Design*des)
+{
+      if (tie_hi_ == 0) {
+	    NetNet*sig = new NetNet(this, lex_strings.make("_LOGIC1"),
+				    NetNet::WIRE, &netvector_t::scalar_logic);
+	    sig->local_flag(true);
+
+	    tie_hi_ = new NetLogic(this, local_symbol(),
+				   1, NetLogic::PULLUP, 1);
+	    des->add_node(tie_hi_);
+
+	    connect(sig->pin(0), tie_hi_->pin(0));
+      }
+}
+
+void NetScope::add_tie_lo(Design*des)
+{
+      if (tie_lo_ == 0) {
+	    NetNet*sig = new NetNet(this, lex_strings.make("_LOGIC0"),
+				    NetNet::WIRE, &netvector_t::scalar_logic);
+	    sig->local_flag(true);
+
+	    tie_lo_ = new NetLogic(this, local_symbol(),
+				   1, NetLogic::PULLDOWN, 1);
+	    des->add_node(tie_lo_);
+
+	    connect(sig->pin(0), tie_lo_->pin(0));
+      }
 }
