@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2014-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -22,8 +22,8 @@
 
 using namespace std;
 
-netqueue_t::netqueue_t(ivl_type_t vec)
-: netdarray_t(vec)
+netqueue_t::netqueue_t(ivl_type_t vec, long max_idx)
+: netdarray_t(vec), max_idx_(max_idx)
 {
 }
 
@@ -38,9 +38,16 @@ ivl_variable_type_t netqueue_t::base_type() const
 
 bool netqueue_t::test_compatibility(ivl_type_t that) const
 {
-      const netqueue_t*that_q = dynamic_cast<const netqueue_t*>(that);
-      if (that_q == 0)
+      ivl_type_t elem_type = 0;
+
+      if (const netqueue_t*that_q = dynamic_cast<const netqueue_t*>(that))
+	    elem_type = that_q->element_type();
+
+      if (const netdarray_t*that_da = dynamic_cast<const netdarray_t*>(that))
+	    elem_type = that_da->element_type();
+
+      if (elem_type == 0)
 	    return false;
 
-      return element_type()->type_compatible(that_q->element_type());
+      return element_type()->type_compatible(elem_type);
 }
