@@ -1,7 +1,7 @@
 #ifndef IVL_pform_H
 #define IVL_pform_H
 /*
- * Copyright (c) 1998-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -104,7 +104,7 @@ struct net_decl_assign_t {
 
 /* The lgate is gate instantiation information. */
 struct lgate {
-      inline lgate(int =0)
+      explicit inline lgate(int =0)
       : parms(0), parms_by_name(0), file(NULL), lineno(0)
       { }
 
@@ -147,7 +147,11 @@ extern bool pform_in_interface(void);
  */
 extern PWire* pform_get_wire_in_scope(perm_string name);
 
-extern PWire* pform_get_make_wire_in_scope(perm_string name, NetNet::Type net_type, NetNet::PortType port_type, ivl_variable_type_t vt_type);
+extern PWire* pform_get_make_wire_in_scope(const struct vlltype&li,
+                                           perm_string name,
+                                           NetNet::Type net_type,
+                                           NetNet::PortType port_type,
+                                           ivl_variable_type_t vt_type);
 
 /*
  * The parser uses startmodule and endmodule together to build up a
@@ -165,8 +169,8 @@ extern void pform_startmodule(const struct vlltype&loc, const char*name,
 			      bool program_block, bool is_interface,
 			      LexicalScope::lifetime_t lifetime,
 			      list<named_pexpr_t>*attr);
-extern void pform_check_timeunit_prec();
 extern void pform_module_set_ports(vector<Module::port_t*>*);
+extern void pform_set_scope_timescale(const struct vlltype&loc);
 
 /* These functions are used when we have a complete port definition, either
    in an ansi style or non-ansi style declaration. In this case, we have
@@ -242,7 +246,10 @@ extern void pform_add_modport_port(const struct vlltype&loc,
  * This creates an identifier aware of names that may have been
  * imported from other packages.
  */
-extern PEIdent* pform_new_ident(const pform_name_t&name);
+extern PEIdent* pform_new_ident(const struct vlltype&loc, const pform_name_t&name);
+
+extern PTrigger* pform_new_trigger(const struct vlltype&loc, PPackage*pkg,
+				   const pform_name_t&name);
 
 /*
  * Enter/exit name scopes. The push_scope function pushes the scope
@@ -270,7 +277,8 @@ extern PTask*pform_push_task_scope(const struct vlltype&loc, char*name,
 extern PFunction*pform_push_function_scope(const struct vlltype&loc, const char*name,
 					   LexicalScope::lifetime_t lifetime);
 
-extern PBlock*pform_push_block_scope(char*name, PBlock::BL_TYPE tt);
+extern PBlock*pform_push_block_scope(const struct vlltype&loc, char*name,
+				     PBlock::BL_TYPE tt);
 
 extern void pform_put_behavior_in_scope(AProcess*proc);
 
@@ -284,6 +292,7 @@ extern verinum* pform_verinum_with_size(verinum*s, verinum*val,
 extern void pform_genvars(const struct vlltype&li, list<perm_string>*names);
 
 extern void pform_start_generate_for(const struct vlltype&li,
+				     bool local_index,
 				     char*ident1,
 				     PExpr*init,
 				     PExpr*test,
@@ -295,7 +304,7 @@ extern void pform_start_generate_case(const struct vlltype&lp, PExpr*test);
 extern void pform_start_generate_nblock(const struct vlltype&lp, char*name);
 extern void pform_generate_case_item(const struct vlltype&lp, list<PExpr*>*test);
 extern void pform_generate_block_name(char*name);
-extern void pform_endgenerate();
+extern void pform_endgenerate(bool end_conditional);
 
 /*
  * This function returns the lexically containing generate scheme, if
@@ -307,9 +316,10 @@ extern PGenerate* pform_parent_generate(void);
 extern void pform_set_typedef(perm_string name, data_type_t*data_type,
 			      std::list<pform_range_t>*unp_ranges);
 
+extern void pform_set_type_referenced(const struct vlltype&loc, const char*name);
+
 /*
- * This function makes a PECallFunction of the named function. Decide
- * if this function is in the scope or is imported from a package.
+ * This function makes a PECallFunction of the named function.
  */
 extern PECallFunction* pform_make_call_function(const struct vlltype&loc,
 						const pform_name_t&name,
@@ -386,11 +396,11 @@ extern void pform_set_reg_idx(perm_string name,
 
 extern void pform_set_data_type(const struct vlltype&li, data_type_t*, list<perm_string>*names, NetNet::Type net_type, list<named_pexpr_t>*attr);
 
-extern void pform_set_struct_type(struct_type_t*struct_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
+extern void pform_set_struct_type(const struct vlltype&li, struct_type_t*struct_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
 
-extern void pform_set_string_type(const string_type_t*string_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
+extern void pform_set_string_type(const struct vlltype&li, const string_type_t*string_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
 
-extern void pform_set_class_type(class_type_t*class_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*addr);
+extern void pform_set_class_type(const struct vlltype&li, class_type_t*class_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*addr);
 
 
   /* pform_set_attrib and pform_set_type_attrib exist to support the
@@ -424,6 +434,13 @@ extern void pform_set_specparam(const struct vlltype&loc,
 				 PExpr*expr);
 extern void pform_set_defparam(const pform_name_t&name, PExpr*expr);
 
+extern void pform_set_param_from_type(const struct vlltype&loc,
+                                      const data_type_t *data_type,
+                                      const char *name,
+                                      list<pform_range_t> *&param_range,
+                                      bool &param_signed,
+                                      ivl_variable_type_t &param_type);
+
 /*
  * Functions related to specify blocks.
  */
@@ -471,7 +488,8 @@ extern void pform_makegates(const struct vlltype&loc,
 extern void pform_make_modgates(const struct vlltype&loc,
 				perm_string type,
 				struct parmvalue_t*overrides,
-				svector<lgate>*gates);
+				svector<lgate>*gates,
+				list<named_pexpr_t>*attr);
 
 /* Make a continuous assignment node, with optional bit- or part- select. */
 extern void pform_make_pgassign_list(list<PExpr*>*alist,
@@ -502,6 +520,13 @@ extern std::vector<pform_tf_port_t>*pform_make_task_ports(const struct vlltype&l
 extern PAssign* pform_compressed_assign_from_inc_dec(const struct vlltype&loc,
 						     PExpr*exp);
 
+/*
+ * The parser uses this function to convert a genvar increment/decrement
+ * expression to the equivalent binary add/subtract expression.
+ */
+extern PExpr* pform_genvar_inc_dec(const struct vlltype&loc, const char*name,
+                                   bool inc_flag);
+
 /*
  * These are functions that the outside-the-parser code uses the do
  * interesting things to the Verilog. The parse function reads and
@@ -559,8 +584,12 @@ extern void parm_to_defparam_list(const string&param);
  */
 extern bool get_time_unit(const char*cp, int &unit);
 extern int  pform_get_timeunit();
-extern void pform_set_timeunit(const char*txt, bool in_module, bool only_check);
-extern void pform_set_timeprecision(const char*txt, bool in_module,
-                                    bool only_check);
+extern void pform_set_timeunit(const char*txt, bool initial_decl);
+extern void pform_set_timeprec(const char*txt, bool initial_decl);
+/*
+ * Flags to determine whether this is an initial declaration.
+ */
+extern bool allow_timeunit_decl;
+extern bool allow_timeprec_decl;
 
 #endif /* IVL_pform_H */
