@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2013 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2017 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -497,6 +497,7 @@ void dll_target::proc_case(const NetCase*net)
       }
       assert(stmt_cur_->type_ != IVL_ST_NONE);
 
+      stmt_cur_->u_.case_.quality = net->case_quality();
       assert(expr_ == 0);
       assert(net->expr());
       net->expr()->expr_scan(this);
@@ -847,12 +848,15 @@ bool dll_target::proc_wait(const NetEvWait*net)
 
 	/* This is a wait fork statement. */
       if ((net->nevents() == 1) && (net->event(0) == 0)) {
+	    stmt_cur_->u_.wait_.needs_t0_trigger = 0;
 	    stmt_cur_->u_.wait_.event = 0;
 	    stmt_cur_->type_ = IVL_ST_WAIT;
 	    stmt_cur_->u_.wait_.stmt_->type_ = IVL_ST_NOOP;
 	    return true;
       }
 
+      stmt_cur_->u_.wait_.needs_t0_trigger = net->has_t0_trigger();
+
 	// This event processing code is also in the NB assign above.
       if (net->nevents() > 1) {
 	    stmt_cur_->u_.wait_.events = (ivl_event_t*)
