@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2020 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2013 / Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -27,6 +27,7 @@
 # include  "compiler.h"
 # include  "t-dll.h"
 # include  "netclass.h"
+# include  "netqueue.h"
 # include  "netmisc.h"
 # include  "discipline.h"
 # include  <cstdlib>
@@ -103,6 +104,13 @@ inline const char*dlerror(void)
 { return strerror( errno ); }
 #endif
 
+ivl_scope_s::ivl_scope_s()
+{
+      func_type = IVL_VT_NO_TYPE;
+      func_signed = false;
+      func_width = 0;
+}
+
 /*
  * The custom new operator for the ivl_nexus_s type allows us to
  * allocate nexus objects in blocks. There are generally lots of them
@@ -249,17 +257,6 @@ ivl_scope_t dll_target::find_scope(ivl_design_s &des, const NetScope*cur)
 	    return 0;
       }
 
-      if (cur->type() == NetScope::CLASS) {
-	    ivl_scope_t tmp = des.classes[cur];
-	    return tmp;
-      }
-
-      if (cur->type()==NetScope::TASK || cur->type()==NetScope::FUNC) {
-	    map<const NetScope*,ivl_scope_t>::const_iterator idx = des.root_tasks.find(cur);
-	    if (idx != des.root_tasks.end())
-		  return idx->second;
-      }
-
       for (unsigned idx = 0; idx < des.roots.size(); idx += 1) {
 	    assert(des.roots[idx]);
 	    ivl_scope_t scope = find_scope_from_root(des.roots[idx], cur);
@@ -274,20 +271,6 @@ ivl_scope_t dll_target::find_scope(ivl_design_s &des, const NetScope*cur)
 		  return scope;
       }
 
-      for (map<const NetScope*,ivl_scope_t>::iterator idx = des.classes.begin()
-		 ; idx != des.classes.end() ; ++ idx) {
-	    ivl_scope_t scope = find_scope_from_root(idx->second, cur);
-	    if (scope)
-		  return scope;
-      }
-
-      for (map<const NetScope*,ivl_scope_t>::iterator idx = des.root_tasks.begin()
-		 ; idx != des.root_tasks.end() ; ++ idx) {
-	    ivl_scope_t scope = find_scope_from_root(idx->second, cur);
-	    if (scope)
-		  return scope;
-      }
-
       return 0;
 }
 
@@ -596,6 +579,28 @@ void dll_target::make_scope_param_expr(ivl_parameter_t cur_par, NetExpr*etmp)
       expr_ = 0;
 }
 
+static void fill_in_scope_function(ivl_scope_t scope, const NetScope*net)
+{
+      scope->type_ = IVL_SCT_FUNCTION;
+      const NetFuncDef*def = net->func_def();
+      assert(def);
+
+      if (def->is_void()) {
+	      // Special case: If there is no return signal, this is
+	      // apparently a VOID function.
+	    scope->func_type = IVL_VT_VOID;
+	    scope->func_signed = 0;
+	    scope->func_width = 0;
+      } else {
+	    const NetNet*return_sig = def->return_sig();
+	    scope->func_type = return_sig->data_type();
+	    scope->func_signed = return_sig->get_signed();
+	    scope->func_width = return_sig->vector_width();
+      }
+
+      scope->tname_ = def->scope()->basename();
+}
+
 void dll_target::add_root(const NetScope *s)
 {
       ivl_scope_t root_ = new struct ivl_scope_s;
@@ -628,22 +633,6 @@ void dll_target::add_root(const NetScope *s)
 	  case NetScope::CLASS:
 	    root_->type_ = IVL_SCT_CLASS;
 	    break;
-	  case NetScope::TASK: {
-		const NetTaskDef*def = s->task_def();
-		if (def == 0) {
-		      cerr << "?:?" << ": internal error: "
-			   << "task " << root_->name_
-			   << " has no definition." << endl;
-		}
-		assert(def);
-		root_->type_ = IVL_SCT_TASK;
-		root_->tname_ = def->scope()->basename();
-		break;
-	  }
-	    break;
-	  case NetScope::FUNC:
-	    root_->type_ = IVL_SCT_FUNCTION;
-	    break;
 	  default:
 	    assert(0);
       }
@@ -667,16 +656,6 @@ void dll_target::add_root(const NetScope *s)
 	    des_.packages.push_back(root_);
 	    break;
 
-	  case NetScope::CLASS:
-	    root_->ports = 0;
-	    des_.classes[s] = root_;
-	    break;
-
-	  case NetScope::TASK:
-	  case NetScope::FUNC:
-	    des_.root_tasks[s] = root_;
-	    break;
-
 	  default:
 	    assert(0);
 	    break;
@@ -718,11 +697,7 @@ bool dll_target::start_design(const Design*des)
       }
       assert(idx == des_.disciplines.size());
 
-      list<NetScope *> scope_list = des->find_roottask_scopes();
-      for (list<NetScope*>::const_iterator cur = scope_list.begin()
-		 ; cur != scope_list.end() ; ++ cur) {
-	    add_root(*cur);
-      }
+      list<NetScope *> scope_list;
 
       scope_list = des->find_package_scopes();
       for (list<NetScope*>::const_iterator cur = scope_list.begin()
@@ -997,6 +972,12 @@ void dll_target::logic(const NetLogic*net)
 	  case NetLogic::CMOS:
 	    obj->type_ = IVL_LO_CMOS;
 	    break;
+	  case NetLogic::EQUIV:
+	    obj->type_ = IVL_LO_EQUIV;
+	    break;
+	  case NetLogic::IMPL:
+	    obj->type_ = IVL_LO_IMPL;
+	    break;
 	  case NetLogic::NAND:
 	    obj->type_ = IVL_LO_NAND;
 	    break;
@@ -1253,6 +1234,12 @@ void dll_target::net_case_cmp(const NetCaseCmp*net)
 	  case NetCaseCmp::NEQ:
 	    obj->type = IVL_LPM_CMP_NEE;
 	    break;
+	  case NetCaseCmp::WEQ:
+	    obj->type = IVL_LPM_CMP_WEQ;
+	    break;
+	  case NetCaseCmp::WNE:
+	    obj->type = IVL_LPM_CMP_WNE;
+	    break;
 	  case NetCaseCmp::XEQ:
 	      obj->type = IVL_LPM_CMP_EQX;
 	    break;
@@ -2029,6 +2016,39 @@ void dll_target::lpm_ff(const NetFF*net)
       nexus_lpm_add(obj->u_.ff.d.pin, obj, 0, IVL_DR_HiZ, IVL_DR_HiZ);
 }
 
+void dll_target::lpm_latch(const NetLatch*net)
+{
+      ivl_lpm_t obj = new struct ivl_lpm_s;
+      obj->type  = IVL_LPM_LATCH;
+      obj->name  = net->name();
+      obj->scope = find_scope(des_, net->scope());
+      assert(obj->scope);
+      FILE_NAME(obj, net);
+
+      obj->width = net->width();
+
+      scope_add_lpm(obj->scope, obj);
+
+      const Nexus*nex;
+
+      nex = net->pin_Enable().nexus();
+      assert(nex->t_cookie());
+      obj->u_.latch.e = nex->t_cookie();
+      assert(obj->u_.latch.e);
+      nexus_lpm_add(obj->u_.latch.e, obj, 0, IVL_DR_HiZ, IVL_DR_HiZ);
+
+      nex = net->pin_Q().nexus();
+      assert(nex->t_cookie());
+      obj->u_.latch.q.pin = nex->t_cookie();
+      nexus_lpm_add(obj->u_.latch.q.pin, obj, 0,
+		    IVL_DR_STRONG, IVL_DR_STRONG);
+
+      nex = net->pin_Data().nexus();
+      assert(nex->t_cookie());
+      obj->u_.latch.d.pin = nex->t_cookie();
+      nexus_lpm_add(obj->u_.latch.d.pin, obj, 0, IVL_DR_HiZ, IVL_DR_HiZ);
+}
+
 /*
  * Make the NetMult object into an IVL_LPM_MULT node.
  */
@@ -2448,16 +2468,7 @@ void dll_target::net_probe(const NetEvProbe*)
 
 void dll_target::scope(const NetScope*net)
 {
-      if (net->parent()==0 && net->type()==NetScope::CLASS) {
-
-	    if (debug_emit) {
-		  cerr << "dll_target::scope: "
-		       << "Add class " << scope_path(net)
-		       << " as a root scope." << endl;
-	    }
-	    add_root(net);
-
-      } if (net->parent() == 0) {
+      if (net->parent() == 0) {
 
 	      // Root scopes are already created...
 
@@ -2516,8 +2527,7 @@ void dll_target::scope(const NetScope*net)
 		      break;
 		}
 		case NetScope::FUNC:
-		  scop->type_ = IVL_SCT_FUNCTION;
-		  scop->tname_ = net->func_def()->scope()->basename();
+		  fill_in_scope_function(scop, net);
 		  break;
 		case NetScope::BEGIN_END:
 		  scop->type_ = IVL_SCT_BEGIN;
@@ -2532,7 +2542,8 @@ void dll_target::scope(const NetScope*net)
 		  scop->tname_ = scop->name_;
 		  break;
 		case NetScope::CLASS:
-		  assert(0);
+		  scop->type_ = IVL_SCT_CLASS;
+		  scop->tname_ = scop->name_;
 		  break;
 	    }
       }
@@ -2691,11 +2702,18 @@ void dll_target::signal(const NetNet*net)
 	      // The back-end API doesn't yet support multi-dimension
 	      // unpacked arrays, so just report the canonical dimensions.
 	    obj->array_base = 0;
-	    obj->array_words = net->unpacked_count();
+	      // For a queue we pass the maximum queue size as the array words.
+	    if (obj->net_type->base_type() == IVL_VT_QUEUE) {
+		  long max_size = net->queue_type()->max_idx()+1;
+		  ivl_assert(*net, max_size >= 0);
+		  obj->array_words = max_size;
+	    } else
+		  obj->array_words = net->unpacked_count();
 	    obj->array_addr_swapped = 0;
       }
 
-      ivl_assert(*net, obj->array_words == net->pin_count());
+      ivl_assert(*net, (obj->array_words == net->pin_count()) ||
+                       (obj->net_type->base_type() == IVL_VT_QUEUE));
       if (debug_optimizer && obj->array_words > 1000) cerr << "debug: "
 	    "t-dll creating nexus array " << obj->array_words << " long" << endl;
       if (obj->array_words > 1 && net->pins_are_virtual()) {
@@ -2789,6 +2807,7 @@ bool dll_target::signal_paths(const NetNet*net)
 		  obj->path[ptr].src = nex->t_cookie();
 		  obj->path[ptr].condit = path_condit;
 		  obj->path[ptr].conditional = src->is_condit();
+		  obj->path[ptr].parallel = src->is_parallel();
 		  obj->path[ptr].posedge = src->is_posedge();
 		  obj->path[ptr].negedge = src->is_negedge();
 		  for (unsigned pe = 0 ;  pe < 12 ;  pe += 1) {
