@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2011-2020 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,8 +44,10 @@ static expr_sign_t expr_get_binary_sign_type(ivl_expr_t expr)
       switch (ivl_expr_opcode(expr)) {
 	case 'E':
 	case 'e':
+	case 'w':
 	case 'N':
 	case 'n':
+	case 'W':
 	case '<':
 	case 'L':
 	case '>':
@@ -136,6 +138,8 @@ static expr_sign_t expr_get_unary_sign_type(ivl_expr_t expr)
 	case 'X':
 	      /* The reduction operators always act as if the argument is
 	       * unsigned. */
+	case '!':
+	      /* The logical negation operator works on either type. */
 	    break;
 	case 'r':
 	      /* For a cast to real the expression should be signed and no
@@ -417,8 +421,10 @@ static void emit_expr_binary(ivl_scope_t scope, ivl_expr_t expr, unsigned wid,
 	case 'p': oper = "**"; break;
 	case 'E': oper = "==="; break;
 	case 'e': oper = "=="; break;
+	case 'w': oper = "==?"; break;
 	case 'N': oper = "!=="; break;
 	case 'n': oper = "!="; break;
+	case 'W': oper = "!=?"; break;
 	case '<': oper = "<"; break;
 	case 'L': oper = "<="; break;
 	case '>': oper = ">"; break;
@@ -466,6 +472,14 @@ static void emit_expr_binary(ivl_scope_t scope, ivl_expr_t expr, unsigned wid,
 	    fprintf(vlog_out, " %s ", oper);
 	    emit_expr(scope, oper2, wid, 0, can_skip_unsigned, is_full_prec);
 	    break;
+	case 'w':
+	case 'W':
+	    fprintf(stderr, "%s:%u: vlog95 error: The wild equality operators "
+	                    "cannot be converted.\n",
+	                    ivl_expr_file(expr),
+	                    ivl_expr_lineno(expr));
+	    vlog_errors += 1;
+	    // fallthrough
 	case 'E':
 	case 'e':
 	case 'N':
@@ -486,6 +500,17 @@ static void emit_expr_binary(ivl_scope_t scope, ivl_expr_t expr, unsigned wid,
 	    fprintf(vlog_out, " %s ", oper);
 	    emit_expr(scope, oper2, ivl_expr_width(oper2), 0, 1, 0);
 	    break;
+	case 'q': // The arguments have already been reduced
+	    fprintf(vlog_out, "~");
+	    emit_expr(scope, oper1, ivl_expr_width(oper1), 0, 1, 0);
+	    fprintf(vlog_out, " | ");
+	    emit_expr(scope, oper2, ivl_expr_width(oper2), 0, 1, 0);
+	    break;
+	case 'Q': // The arguments have already been reduced
+	    emit_expr(scope, oper1, ivl_expr_width(oper1), 0, 1, 0);
+	    fprintf(vlog_out, " ~^ ");
+	    emit_expr(scope, oper2, ivl_expr_width(oper2), 0, 1, 0);
+	    break;
 	case 'R':
 	    if (! allow_signed) {
 		  fprintf(stderr, "%s:%u: vlog95 error: >>> operator is not "
