@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2016 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2010-2020 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,10 @@ const char *func_rtn_name = 0;
 
 static void emit_func_return(ivl_signal_t sig)
 {
+        // Handle SV void functions.
+      if (sig == 0)
+            return;
+
       if (ivl_signal_dimensions(sig) > 0) {
 	    fprintf(stderr, "%s:%u: vlog95 error: A function cannot return "
 	                    "an array.\n", ivl_signal_file(sig),
@@ -679,17 +683,17 @@ static void emit_named_block_scope(ivl_scope_t scope)
 }
 
 /*
- * In SystemVerilog a task or function can have a process to initialize
- * variables. In reality SystemVerilog requires this to be before the
- * initial/always blocks are processed, but that's not how it is currently
- * implemented in Icarus!
+ * In SystemVerilog a task, function, or block can have a process to
+ * initialize variables. SystemVerilog requires this to be before the
+ * initial/always blocks are processed, but there's no way to express
+ * this in Verilog-95.
  */
-static int find_tf_process(ivl_process_t proc, ivl_scope_t scope)
+static int find_tfb_process(ivl_process_t proc, ivl_scope_t scope)
 {
       if (scope == ivl_process_scope(proc)) {
 	    ivl_scope_t mod_scope = scope;
-	      /* A task or function can only have initial processes that
-	       * are used to set local variables. */
+	      /* A task or function or named block can only have initial
+	       * processes that are used to set local variables. */
 	    assert(ivl_process_type(proc) == IVL_PR_INITIAL);
 	      /* Find the module scope for this task/function. */
 	    while (ivl_scope_type(mod_scope) != IVL_SCT_MODULE) {
@@ -704,15 +708,16 @@ static int find_tf_process(ivl_process_t proc, ivl_scope_t scope)
 }
 
 /*
- * Emit any initial blocks for the tasks or functions in a module.
+ * Emit any initial blocks for the tasks/functions/named blocks in a module.
  */
-static int emit_tf_process(ivl_scope_t scope, ivl_scope_t parent)
+static int emit_tfb_process(ivl_scope_t scope, ivl_scope_t parent)
 {
       ivl_scope_type_t sc_type = ivl_scope_type(scope);
       (void)parent;  /* Parameter is not used. */
-      if ((sc_type == IVL_SCT_FUNCTION) || (sc_type == IVL_SCT_TASK)) {
+      if ((sc_type == IVL_SCT_FUNCTION) || (sc_type == IVL_SCT_TASK) ||
+          (sc_type == IVL_SCT_BEGIN) || (sc_type == IVL_SCT_FORK)) {
 	/* Output the initial/always blocks for this module. */
-	    ivl_design_process(design, (ivl_process_f)find_tf_process, scope);
+	    ivl_design_process(design, (ivl_process_f)find_tfb_process, scope);
       }
       return 0;
 }
@@ -793,7 +798,11 @@ static void emit_specify_paths(ivl_scope_t scope, ivl_signal_t sig)
 		  has_edge = 1;
 	    }
 	    emit_nexus_as_ca(scope, source, 0, 0);
-	    fprintf(vlog_out, " =>");
+	    if (ivl_path_is_parallel(dpath)) {
+		  fprintf(vlog_out, " =>");
+	    } else {
+		  fprintf(vlog_out, " *>");
+	    }
 	      /* The compiler does not keep the source expression for an edge
 	       * sensitive path so add a constant to get the syntax right. */
 	    if (has_edge) {
@@ -1145,9 +1154,10 @@ int emit_scope(ivl_scope_t scope, ivl_scope_t parent)
 		  emit_tran(scope, ivl_scope_switch(scope, idx));
 	    }
 
-	      /* Output any initial blocks for tasks or functions defined
-	       * in this module. Used to initialize local variables. */
-	    ivl_scope_children(scope, (ivl_scope_f*) emit_tf_process, scope);
+	      /* Output any initial blocks for tasks or functions or named
+	       * blocks defined in this module. Used to initialize local
+	       * variables. */
+	    ivl_scope_children(scope, (ivl_scope_f*) emit_tfb_process, scope);
 
 	      /* Output the initial/always blocks for this module. */
 	    ivl_design_process(design, (ivl_process_f)find_process, scope);
