@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2014 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2011-2020 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -94,12 +94,18 @@ static void emit_stmt_lval_name(ivl_scope_t scope, ivl_lval_t lval,
       emit_id(ivl_signal_basename(sig));
       if (array_idx) {
 	    int msb, lsb;
-	    assert(ivl_signal_dimensions(sig));
+	    ivl_type_t net_type = ivl_signal_net_type(sig);
 	    fprintf(vlog_out, "[");
-	      /* For an array the LSB/MSB order is not important. They are
-	       * always accessed from base counting up. */
-	    lsb = ivl_signal_array_base(sig);
-	    msb = lsb + ivl_signal_array_count(sig) - 1;
+	    if (ivl_type_base(net_type) == IVL_VT_QUEUE) {
+		  lsb = 0;
+		  msb = 1;
+	    } else {
+		  assert(ivl_signal_dimensions(sig));
+		    /* For an array the LSB/MSB order is not important.
+		     * They are always accessed from base counting up. */
+		  lsb = ivl_signal_array_base(sig);
+		  msb = lsb + ivl_signal_array_count(sig) - 1;
+	    }
 	    emit_scaled_expr(scope, array_idx, msb, lsb);
 	    fprintf(vlog_out, "]");
       }
@@ -266,7 +272,8 @@ static void emit_stmt_lval_piece(ivl_scope_t scope, ivl_lval_t lval)
 
 	/* If there are no selects then just print the name. */
       sel_expr = ivl_lval_part_off(lval);
-      if (! sel_expr && (width == ivl_signal_width(sig))) {
+      if (! sel_expr && ((width == ivl_signal_width(sig)) ||
+                         (ivl_signal_data_type(sig) == IVL_VT_QUEUE))) {
 	    emit_stmt_lval_name(scope, lval, sig);
 	    return;
       }
@@ -667,6 +674,7 @@ static unsigned is_wait(ivl_scope_t scope, ivl_statement_t stmt)
 static unsigned utask_in_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
 {
       unsigned idx, ports = ivl_scope_ports(scope);
+      unsigned first_arg = is_void_function(scope) ? 1 : 0;
       ivl_lval_t lval = ivl_stmt_lval(stmt, 0);
       ivl_signal_t lsig = ivl_lval_sig(lval);
       const char *sig_name;
@@ -682,7 +690,7 @@ static unsigned utask_in_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
       if (scope != ivl_signal_scope(lsig)) return ports;
 	/* It must be an input or inout port of the task. */
       sig_name = ivl_signal_basename(lsig);
-      for (idx = 0; idx < ports; idx += 1) {
+      for (idx = first_arg; idx < ports; idx += 1) {
 	    ivl_signal_t port = ivl_scope_port(scope, idx);
 	    ivl_signal_port_t port_type = ivl_signal_port(port);
 	    if ((port_type != IVL_SIP_INPUT) &&
@@ -699,6 +707,7 @@ static unsigned utask_in_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
 static unsigned utask_out_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
 {
       unsigned idx, ports = ivl_scope_ports(scope);
+      unsigned first_arg = is_void_function(scope) ? 1 : 0;
       ivl_expr_t rval = ivl_stmt_rval(stmt);
       ivl_signal_t rsig = 0;
       ivl_expr_type_t expr_type = ivl_expr_type(rval);
@@ -731,7 +740,7 @@ static unsigned utask_out_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
       if (ivl_signal_dimensions(rsig)) return ports;
 	/* It must be an output or inout port of the task. */
       sig_name = ivl_signal_basename(rsig);
-      for (idx = 0; idx < ports; idx += 1) {
+      for (idx = first_arg; idx < ports; idx += 1) {
 	    ivl_signal_t port = ivl_scope_port(scope, idx);
 	    ivl_signal_port_t port_type = ivl_signal_port(port);
 	    if ((port_type != IVL_SIP_OUTPUT) &&
@@ -794,6 +803,7 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
       unsigned lineno = ivl_stmt_lineno(stmt);
       unsigned start, stop, is_auto = 0;
       ivl_scope_t task_scope = 0;
+      unsigned is_void_func = 0;
       port_expr_t port_exprs;
 	/* Check to see if the block is of the basic form first.  */
       for (idx = 0; idx < count; idx += 1) {
@@ -809,7 +819,8 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
 	    if (ivl_statement_type(tmp) == IVL_ST_UTASK && !task_scope) {
 		  task_idx = idx;
 		  task_scope = ivl_stmt_call(tmp);
-		  assert(ivl_scope_type(task_scope) == IVL_SCT_TASK);
+		  is_void_func = is_void_function(task_scope);
+		  assert(ivl_scope_type(task_scope) == IVL_SCT_TASK || is_void_func);
 		  continue;
 	    }
 	      /* For an automatic task the FREE must be last. */
@@ -871,7 +882,8 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
       }
 
 	/* Verify that all the ports were defined. */
-      for (idx = 0; idx < ports; idx += 1) {
+      start = is_void_func ? 1 : 0;
+      for (idx = start; idx < ports; idx += 1) {
 	    if (port_exprs[idx].type == IVL_SIP_NONE) {
 		  free(port_exprs);
 		  return 0;
@@ -880,14 +892,18 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
 
 	/* Now that we have the arguments figured out, print the task call. */
       fprintf(vlog_out, "%*c", get_indent(), ' ');
+      if (is_void_func)
+            fprintf(vlog_out, "if (");
       emit_scope_path(scope, task_scope);
       fprintf(vlog_out, "(");
-      emit_port(scope, port_exprs[0]);
-      for (idx = 1; idx < ports; idx += 1) {
+      emit_port(scope, port_exprs[start]);
+      for (idx = start + 1; idx < ports; idx += 1) {
 	    fprintf(vlog_out, ", ");
 	    emit_port(scope, port_exprs[idx]);
       }
       free(port_exprs);
+      if (is_void_func)
+            fprintf(vlog_out, ")");
       fprintf(vlog_out, ");");
       emit_stmt_file_line(stmt);
       fprintf(vlog_out, "\n");
@@ -1419,10 +1435,16 @@ static void emit_stmt_trigger(ivl_scope_t scope, ivl_statement_t stmt)
 static void emit_stmt_utask(ivl_scope_t scope, ivl_statement_t stmt)
 {
       ivl_scope_t task_scope = ivl_stmt_call(stmt);
-      assert(ivl_scope_type(task_scope) == IVL_SCT_TASK);
-      assert(ivl_scope_ports(task_scope) == 0);
+      assert((ivl_scope_type(task_scope) == IVL_SCT_TASK
+		&& ivl_scope_ports(task_scope) == 0)
+	  || (is_void_function(task_scope)
+		&& ivl_scope_ports(task_scope) == 1));
       fprintf(vlog_out, "%*c", get_indent(), ' ');
+      if (is_void_function(task_scope))
+            fprintf(vlog_out, "if (");
       emit_scope_path(scope, task_scope);
+      if (is_void_function(task_scope))
+            fprintf(vlog_out, "(1'bx))");
       fprintf(vlog_out, ";");
       emit_stmt_file_line(stmt);
       fprintf(vlog_out, "\n");
@@ -1606,6 +1628,9 @@ void emit_process(ivl_scope_t scope, ivl_process_t proc)
             fprintf(vlog_out, "initial");
             break;
         case IVL_PR_ALWAYS:
+        case IVL_PR_ALWAYS_COMB:
+        case IVL_PR_ALWAYS_FF:
+        case IVL_PR_ALWAYS_LATCH:
             fprintf(vlog_out, "always");
             break;
         case IVL_PR_FINAL:
