@@ -1,7 +1,7 @@
 #ifndef IVL_vlog95_priv_H
 #define IVL_vlog95_priv_H
 /*
- * Copyright (C) 2010-2016 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2010-2019 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -106,7 +106,6 @@ extern void emit_nexus_as_ca(ivl_scope_t scope, ivl_nexus_t nex,
 extern void emit_nexus_port_driver_as_ca(ivl_scope_t scope, ivl_nexus_t nex);
 extern void emit_const_nexus(ivl_scope_t scope, ivl_net_const_t const_net);
 extern void emit_signal_net_const_as_ca(ivl_scope_t scope, ivl_signal_t sig);
-extern void emit_icarus_generated_udps(void);
 
 extern void add_udp_to_list(ivl_udp_t udp);
 extern void emit_udp_list(void);
@@ -118,6 +117,9 @@ extern void emit_number(const char *bits, unsigned nbits, unsigned is_signed,
                         const char *file, unsigned lineno);
 extern void emit_string(const char *string);
 
+extern void emit_icarus_generated_udps(void);
+extern void emit_icarus_generated_top_module(void);
+
 /*
  * Find the enclosing module scope.
  */
@@ -160,4 +162,10 @@ extern void free_emitted_scope_list(void);
 */
 extern void dump_nexus_information(ivl_scope_t scope, ivl_nexus_t nex);
 
+static inline unsigned is_void_function(ivl_scope_t scope)
+{
+      return ivl_scope_type(scope) == IVL_SCT_FUNCTION
+          && ivl_scope_port(scope, 0) == 0;
+}
+
 #endif /* IVL_vlog95_priv_H */
