@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2013-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -227,8 +227,8 @@ static void draw_binary_vec4_compare_real(ivl_expr_t expr)
 	    break;
 	  case 'n': /* != */
 	    fprintf(vvp_out, "    %%cmp/wr;\n");
+	    fprintf(vvp_out, "    %%flag_inv 4;\n");
 	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
-	    fprintf(vvp_out, "    %%inv;\n");
 	    break;
 	  default:
 	    assert(0);
@@ -247,8 +247,8 @@ static void draw_binary_vec4_compare_string(ivl_expr_t expr)
 	    break;
 	  case 'n': /* != */
 	    fprintf(vvp_out, "    %%cmp/str;\n");
+	    fprintf(vvp_out, "    %%flag_inv 4;\n");
 	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
-	    fprintf(vvp_out, "    %%inv;\n");
 	    break;
 	  default:
 	    assert(0);
@@ -298,9 +298,9 @@ static void draw_binary_vec4_compare_class(ivl_expr_t expr)
 		  fprintf(vvp_out, "    %%test_nul/a v%p, %d;\n", sig, word_ix);
 		  clr_word(word_ix);
 	    }
-	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 	    if (ivl_expr_opcode(expr) == 'n')
-		  fprintf(vvp_out, "    %%inv;\n");
+		  fprintf(vvp_out, "    %%flag_inv 4;\n");
+	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 	    return;
       }
 
@@ -320,9 +320,9 @@ static void draw_binary_vec4_compare_class(ivl_expr_t expr)
 	    fprintf(vvp_out, "    %%load/obj v%p_0;\n", sig);
 	    fprintf(vvp_out, "    %%test_nul/prop %u, %d;\n", pidx, idx);
 	    fprintf(vvp_out, "    %%pop/obj 1, 0;\n");
-	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 	    if (ivl_expr_opcode(expr) == 'n')
-		  fprintf(vvp_out, "    %%inv;\n");
+		  fprintf(vvp_out, "    %%flag_inv 4;\n");
+	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 
 	    if (idx != 0) clr_word(idx);
 	    return;
@@ -385,24 +385,53 @@ static void draw_binary_vec4_compare(ivl_expr_t expr)
 	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 	    break;
 	  case 'n': /* != */
-	    fprintf(vvp_out, "    %%cmp/e;\n");
+	    fprintf(vvp_out, "    %%cmp/ne;\n");
 	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
-	    fprintf(vvp_out, "    %%inv;\n");
 	    break;
 	  case 'E': /* === */
 	    fprintf(vvp_out, "    %%cmp/e;\n");
 	    fprintf(vvp_out, "    %%flag_get/vec4 6;\n");
 	    break;
 	  case 'N': /* !== */
-	    fprintf(vvp_out, "    %%cmp/e;\n");
+	    fprintf(vvp_out, "    %%cmp/ne;\n");
 	    fprintf(vvp_out, "    %%flag_get/vec4 6;\n");
-	    fprintf(vvp_out, "    %%inv;\n");
+	    break;
+	  case 'w': /* ==? */
+	    fprintf(vvp_out, "    %%cmp/we;\n");
+	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
+	    break;
+	  case 'W': /* !=? */
+	    fprintf(vvp_out, "    %%cmp/wne;\n");
+	    fprintf(vvp_out, "    %%flag_get/vec4 4;\n");
 	    break;
 	  default:
 	    assert(0);
       }
 }
 
+static void draw_binary_vec4_limpl(ivl_expr_t expr)
+{
+      fprintf(stderr, "vvp.tgt sorry: No support for logical implication (%s:%u).\n",
+                      ivl_expr_file(expr), ivl_expr_lineno(expr));
+      assert(0);
+}
+
+static void draw_binary_vec4_lequiv(ivl_expr_t expr)
+{
+      ivl_expr_t le = ivl_expr_oper1(expr);
+      ivl_expr_t re = ivl_expr_oper2(expr);
+
+	/* The arguments should have already been reduced. */
+      assert(ivl_expr_width(le) == 1);
+      draw_eval_vec4(le);
+      assert(ivl_expr_width(re) == 1);
+      draw_eval_vec4(re);
+
+      fprintf(vvp_out, "    %%xnor;\n");
+
+      assert(ivl_expr_width(expr) == 1);
+}
+
 static void draw_binary_vec4_land(ivl_expr_t expr)
 {
       ivl_expr_t le = ivl_expr_oper1(expr);
@@ -689,8 +718,10 @@ static void draw_binary_vec4(ivl_expr_t expr)
 
 	  case 'e': /* == */
 	  case 'E': /* === */
-	  case 'n': /* !== */
+	  case 'n': /* != */
 	  case 'N': /* !== */
+	  case 'w': /* ==? */
+	  case 'W': /* !=? */
 	    draw_binary_vec4_compare(expr);
 	    break;
 
@@ -711,6 +742,14 @@ static void draw_binary_vec4(ivl_expr_t expr)
 	    draw_binary_vec4_lor(expr);
 	    break;
 
+	  case 'q': /* -> (logical implication) */
+	    draw_binary_vec4_limpl(expr);
+	    break;
+
+	  case 'Q': /* <-> (logical equivalence) */
+	    draw_binary_vec4_lequiv(expr);
+	    break;
+
 	  default:
 	    fprintf(stderr, "vvp.tgt error: unsupported binary (%c)\n",
 		    ivl_expr_opcode(expr));
@@ -953,7 +992,7 @@ static void draw_select_pad_vec4(ivl_expr_t expr)
 
 /*
  * This function handles the special case of a call to the internal
- * functions $ivl_darray_method$pop_back et al. The first (and only)
+ * functions $ivl_queue_method$pop_back et al. The first (and only)
  * argument is the signal that represents a dynamic queue. Generate a
  * %qpop instruction to pop a value and push it to the vec4 stack.
  */
@@ -961,7 +1000,7 @@ static void draw_darray_pop(ivl_expr_t expr)
 {
       const char*fb;
 
-      if (strcmp(ivl_expr_name(expr), "$ivl_darray_method$pop_back")==0)
+      if (strcmp(ivl_expr_name(expr), "$ivl_queue_method$pop_back")==0)
 	    fb = "b";
       else
 	    fb = "f";
@@ -969,7 +1008,8 @@ static void draw_darray_pop(ivl_expr_t expr)
       ivl_expr_t arg = ivl_expr_parm(expr, 0);
       assert(ivl_expr_type(arg) == IVL_EX_SIGNAL);
 
-      fprintf(vvp_out, "    %%qpop/%s/v v%p_0;\n", fb, ivl_expr_signal(arg));
+      fprintf(vvp_out, "    %%qpop/%s/v v%p_0, %u;\n", fb, ivl_expr_signal(arg),
+                       ivl_expr_width(expr));
 }
 
 static void draw_sfunc_vec4(ivl_expr_t expr)
@@ -989,11 +1029,11 @@ static void draw_sfunc_vec4(ivl_expr_t expr)
 	    return;
       }
 
-      if (strcmp(ivl_expr_name(expr), "$ivl_darray_method$pop_back")==0) {
+      if (strcmp(ivl_expr_name(expr), "$ivl_queue_method$pop_back")==0) {
 	    draw_darray_pop(expr);
 	    return;
       }
-      if (strcmp(ivl_expr_name(expr),"$ivl_darray_method$pop_front")==0) {
+      if (strcmp(ivl_expr_name(expr),"$ivl_queue_method$pop_front")==0) {
 	    draw_darray_pop(expr);
 	    return;
       }
@@ -1005,6 +1045,15 @@ static void draw_signal_vec4(ivl_expr_t expr)
 {
       ivl_signal_t sig = ivl_expr_signal(expr);
 
+	/* Special Case: If the signal is the return value of the function,
+	   then use a different opcode to get the value. */
+      if (signal_is_return_value(sig)) {
+	    assert(ivl_signal_dimensions(sig) == 0);
+	    fprintf(vvp_out, "    %%retload/vec4 0; Load %s (draw_signal_vec4)\n",
+		    ivl_signal_basename(sig));
+	    return;
+      }
+
 	/* Handle the simple case, a signal expression that is a
 	   simple vector, no array dimensions. */
       if (ivl_signal_dimensions(sig) == 0) {
@@ -1238,10 +1287,24 @@ static void draw_unary_vec4(ivl_expr_t expr)
 	    local_count += 1;
 	    break;
 
-	  case 'v': /* Cast real to vec4 */
-	    assert(ivl_expr_value(sub) == IVL_VT_REAL);
-	    draw_eval_real(sub);
-	    fprintf(vvp_out, "    %%cvt/vr %u;\n", ivl_expr_width(expr));
+	  case 'v': /* Cast expression to vec4 */
+	    switch (ivl_expr_value(sub)) {
+		case IVL_VT_REAL:
+		  draw_eval_real(sub);
+		  fprintf(vvp_out, "    %%cvt/vr %u;\n", ivl_expr_width(expr));
+		  break;
+		case IVL_VT_STRING:
+		  draw_eval_string(sub);
+		  fprintf(vvp_out, "    %%cast/vec4/str %u;\n", ivl_expr_width(expr));
+		  break;
+		case IVL_VT_DARRAY:
+		  draw_eval_object(sub);
+		  fprintf(vvp_out, "    %%cast/vec4/dar %u;\n", ivl_expr_width(expr));
+		  break;
+		default:
+		  assert(0);
+		  break;
+	    }
 	    break;
 
 	  case '2': /* Cast expression to bool */
@@ -1259,6 +1322,14 @@ static void draw_unary_vec4(ivl_expr_t expr)
 		  draw_eval_real(sub);
 		  fprintf(vvp_out, "    %%cvt/vr %u;\n", ivl_expr_width(expr));
 		  break;
+		case IVL_VT_STRING:
+		  draw_eval_string(sub);
+		  fprintf(vvp_out, "    %%cast/vec4/str %u;\n", ivl_expr_width(expr));
+		  break;
+		case IVL_VT_DARRAY:
+		  draw_eval_object(sub);
+		  fprintf(vvp_out, "    %%cast/vec2/dar %u;\n", ivl_expr_width(expr));
+		  break;
 		default:
 		  assert(0);
 		  break;
