@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2011-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -111,7 +111,12 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 
 	    slice->type = SLICE_SIMPLE_VECTOR;
 	    slice->u_.simple_vector.use_word = use_word;
-	    fprintf(vvp_out, "    %%load/vec4 v%p_%lu;\n", sig, use_word);
+	    if (signal_is_return_value(sig)) {
+		  assert(use_word==0);
+		  fprintf(vvp_out, "    %%retload/vec4 0;\n");
+	    } else {
+		  fprintf(vvp_out, "    %%load/vec4 v%p_%lu;\n", sig, use_word);
+	    }
 
       } else if (ivl_signal_dimensions(sig)==0 && part_off_ex==0 && word_ix==0) {
 
@@ -120,7 +125,12 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 	    slice->type = SLICE_PART_SELECT_STATIC;
 	    slice->u_.part_select_static.part_off = part_off;
 
-	    fprintf(vvp_out, "    %%load/vec4 v%p_%lu;\n", sig, use_word);
+	    if (signal_is_return_value(sig)) {
+		  assert(use_word==0);
+		  fprintf(vvp_out, "    %%retload/vec4 0;\n");
+	    } else {
+		  fprintf(vvp_out, "    %%load/vec4 v%p_%lu;\n", sig, use_word);
+	    }
 	    fprintf(vvp_out, "    %%pushi/vec4 %lu, 0, 32;\n", part_off);
 	    fprintf(vvp_out, "    %%part/u %u;\n", wid);
 
@@ -128,6 +138,7 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 
 	    assert(use_word == 0);
 	    assert(part_off == 0);
+	    assert(!signal_is_return_value(sig)); // NOT IMPLEMENTED
 
 	    slice->type = SLICE_PART_SELECT_DYNAMIC;
 
@@ -143,6 +154,8 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix == 0) {
 
+	    assert(!signal_is_return_value(sig)); // NOT IMPLEMENTED
+
 	    slice->type = SLICE_MEMORY_WORD_STATIC;
 	    slice->u_.memory_word_static.use_word = use_word;
 	    if (use_word < ivl_signal_array_count(sig)) {
@@ -156,6 +169,7 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix != 0) {
 
+	    assert(!signal_is_return_value(sig)); // NOT IMPLEMENTED
 	    slice->type = SLICE_MEMORY_WORD_DYNAMIC;
 
 	    slice->u_.memory_word_dynamic.word_idx_reg = allocate_word();
@@ -202,6 +216,49 @@ static void get_vec_from_lval(ivl_statement_t net, struct vec_slice_info*slices)
 
 }
 
+static void put_vec_to_ret_slice(ivl_signal_t sig, struct vec_slice_info*slice,
+				 unsigned wid)
+{
+      int part_off_idx;
+
+	/* If the slice of the l-value is a BOOL variable, then cast
+	   the data to a BOOL vector so that the stores can be valid. */
+      if (ivl_signal_data_type(sig) == IVL_VT_BOOL) {
+	    fprintf(vvp_out, "    %%cast2;\n");
+      }
+
+      switch (slice->type) {
+	  default:
+	    fprintf(vvp_out, " ; XXXX slice->type=%d\n", slice->type);
+	    assert(0);
+	    break;
+
+	  case SLICE_SIMPLE_VECTOR:
+	    assert(slice->u_.simple_vector.use_word == 0);
+	    fprintf(vvp_out, "    %%ret/vec4 0, 0, %u;\n", wid);
+	    break;
+
+	  case SLICE_PART_SELECT_STATIC:
+	    part_off_idx = allocate_word();
+	    fprintf(vvp_out, "    %%ix/load %d, %lu, 0;\n",
+		    part_off_idx, slice->u_.part_select_static.part_off);
+	    fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
+	    fprintf(vvp_out, "    %%ret/vec4 0, %d, %u;\n", part_off_idx, wid);
+	    clr_word(part_off_idx);
+	    break;
+
+	  case SLICE_PART_SELECT_DYNAMIC:
+	    fprintf(vvp_out, "    %%flag_mov 4, %u;\n",
+		    slice->u_.part_select_dynamic.x_flag);
+	    fprintf(vvp_out, "    %%ret/vec4 0, %d, %u;\n",
+		    slice->u_.part_select_dynamic.word_idx_reg, wid);
+	    clr_word(slice->u_.part_select_dynamic.word_idx_reg);
+	    clr_flag(slice->u_.part_select_dynamic.x_flag);
+	    break;
+
+      }
+}
+
 static void put_vec_to_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice,
 				  unsigned wid)
 {
@@ -209,6 +266,15 @@ static void put_vec_to_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice,
       ivl_signal_t sig = ivl_lval_sig(lval);
       int part_off_idx;
 
+
+	/* Special Case: If the l-value signal is named after its scope,
+	   and the scope is a function, then this is an assign to a return
+	   value and should be handled differently. */
+      if (signal_is_return_value(sig)) {
+	    put_vec_to_ret_slice(sig, slice, wid);
+	    return;
+      }
+
 	/* If the slice of the l-value is a BOOL variable, then cast
 	   the data to a BOOL vector so that the stores can be valid. */
       if (ivl_signal_data_type(sig) == IVL_VT_BOOL) {
@@ -375,7 +441,10 @@ static void store_vec4_to_lval(ivl_statement_t net)
 		  draw_eval_expr_into_integer(part_off_ex, offset_index);
 		    /* Note that flag4 is set by the eval above. */
 		  assert(lsig);
-		  if (ivl_signal_type(lsig)==IVL_SIT_UWIRE) {
+		  if (signal_is_return_value(lsig)) {
+			fprintf(vvp_out, "    %%ret/vec4 0, %d, %u; Assign to %s (store_vec4_to_lval)\n",
+				offset_index, lwid, ivl_signal_basename(lsig));
+		  } else if (ivl_signal_type(lsig)==IVL_SIT_UWIRE) {
 			fprintf(vvp_out, "    %%force/vec4/off v%p_0, %d;\n",
 				lsig, offset_index);
 		  } else {
@@ -390,6 +459,7 @@ static void store_vec4_to_lval(ivl_statement_t net)
 		       member. We will use a property assign
 		       function. */
 		  assert(!lsig);
+
 		  ivl_type_t sub_type = draw_lval_expr(nest);
 		  assert(ivl_type_base(sub_type) == IVL_VT_CLASS);
 		  fprintf(vvp_out, "    %%store/prop/v %d, %u;\n",
@@ -400,7 +470,13 @@ static void store_vec4_to_lval(ivl_statement_t net)
 		    /* No offset expression, so use simpler store function. */
 		  assert(lsig);
 		  assert(lwid == ivl_signal_width(lsig));
-		  fprintf(vvp_out, "    %%store/vec4 v%p_0, 0, %u;\n", lsig, lwid);
+		  if (signal_is_return_value(lsig)) {
+			fprintf(vvp_out, "    %%ret/vec4 0, 0, %u;  Assign to %s (store_vec4_to_lval)\n",
+				lwid, ivl_signal_basename(lsig));
+		  } else {
+			fprintf(vvp_out, "    %%store/vec4 v%p_0, 0, %u;\n",
+				lsig, lwid);
+		  }
 	    }
       }
 }
@@ -417,6 +493,7 @@ static int show_stmt_assign_vector(ivl_statement_t net)
 	   of the l-value. We need these values as part of the r-value
 	   calculation. */
       if (ivl_stmt_opcode(net) != 0) {
+	    fprintf(vvp_out, "    ; show_stmt_assign_vector: Get l-value for compressed %c= operand\n", ivl_stmt_opcode(net));
             slices = calloc(ivl_stmt_lvals(net), sizeof(struct vec_slice_info));
 	    get_vec_from_lval(net, slices);
       }
@@ -438,37 +515,6 @@ static int show_stmt_assign_vector(ivl_statement_t net)
 		 value pushed. */
 	    fprintf(vvp_out, "    %%cvt/vr %u;\n", wid);
 
-      } else if (ivl_expr_value(rval) == IVL_VT_STRING) {
-	    /* Special case: string to vector casting */
-	    ivl_lval_t lval = ivl_stmt_lval(net, 0);
-	    fprintf(vvp_out, "    %%vpi_call %u %u \"$ivl_string_method$to_vec\", v%p_0, v%p_0 {0 0 0};\n",
-		ivl_file_table_index(ivl_stmt_file(net)), ivl_stmt_lineno(net),
-		ivl_expr_signal(rval), ivl_lval_sig(lval));
-	    if (slices) free(slices);
-            return 0;
-
-      } else if (ivl_expr_value(rval) == IVL_VT_DARRAY) {
-	    /* Special case: dynamic array to vector casting */
-	    ivl_lval_t lval = ivl_stmt_lval(net, 0);
-            void*rval_addr = NULL;
-
-            /* Even more special case: function call returning dynamic array */
-            if(ivl_expr_type(rval) == IVL_EX_UFUNC) {
-                rval_addr = ivl_scope_port(ivl_expr_def(rval), 0);
-                draw_ufunc_object(rval);
-                /* We do not need to store the result, it is going to be
-                   converted to vector quite soon. */
-                fprintf(vvp_out, "    %%pop/obj 1, 0; drop the result\n");
-            } else {
-                rval_addr = ivl_expr_signal(rval);
-            }
-
-	    fprintf(vvp_out, "    %%vpi_call %u %u \"$ivl_darray_method$to_vec\", v%p_0, v%p_0 {0 0 0};\n",
-		ivl_file_table_index(ivl_stmt_file(net)), ivl_stmt_lineno(net),
-		rval_addr, ivl_lval_sig(lval));
-	    if (slices) free(slices);
-            return 0;
-
       } else {
 	    unsigned wid = ivl_stmt_lwidth(net);
 	    draw_eval_vec4(rval);
@@ -602,10 +648,17 @@ static void get_real_from_lval(ivl_lval_t lval, struct real_lval_info*slice)
 
 	    slice->type = REAL_SIMPLE_WORD;
 	    slice->u_.simple_word.use_word = use_word;
-	    fprintf(vvp_out, "    %%load/real v%p_%lu;\n", sig, use_word);
+	    if (signal_is_return_value(sig)) {
+		  assert(use_word==0);
+		  fprintf(vvp_out, "    %%retload/real 0;\n");
+	    } else {
+		  fprintf(vvp_out, "    %%load/real v%p_%lu;\n", sig, use_word);
+	    }
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix == 0) {
 
+	    assert(!signal_is_return_value(sig)); // NOT IMPLEMENTED
+
 	    slice->type = REAL_MEMORY_WORD_STATIC;
 	    slice->u_.memory_word_static.use_word = use_word;
 	    if (use_word < ivl_signal_array_count(sig)) {
@@ -618,6 +671,7 @@ static void get_real_from_lval(ivl_lval_t lval, struct real_lval_info*slice)
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix != 0) {
 
+	    assert(!signal_is_return_value(sig)); // NOT IMPLEMENTED
 	    slice->type = REAL_MEMORY_WORD_DYNAMIC;
 
 	    slice->u_.memory_word_dynamic.word_idx_reg = allocate_word();
@@ -636,6 +690,15 @@ static void put_real_to_lval(ivl_lval_t lval, struct real_lval_info*slice)
 {
       ivl_signal_t sig = ivl_lval_sig(lval);
 
+	/* Special Case: If the l-value signal is named after its scope,
+	   and the scope is a function, then this is an assign to a return
+	   value and should be handled differently. */
+      if (signal_is_return_value(sig)) {
+	    assert(slice->u_.simple_word.use_word == 0);
+	    fprintf(vvp_out, "    %%ret/real 0;\n");
+	    return;
+      }
+
       switch (slice->type) {
 	  default:
 	    fprintf(vvp_out, " ; XXXX slice->type=%d\n", slice->type);
@@ -676,6 +739,18 @@ static void store_real_to_lval(ivl_lval_t lval)
       var = ivl_lval_sig(lval);
       assert(var != 0);
 
+	/* Special Case: If the l-value signal is named after its scope,
+	   and the scope is a function, then this is an assign to a return
+	   value and should be handled differently. */
+      ivl_scope_t sig_scope = ivl_signal_scope(var);
+      if ((ivl_scope_type(sig_scope) == IVL_SCT_FUNCTION)
+	  && (strcmp(ivl_signal_basename(var), ivl_scope_basename(sig_scope)) == 0)) {
+	    assert(ivl_signal_dimensions(var) == 0);
+	    fprintf(vvp_out, "    %%ret/real 0; Assign to %s\n",
+		    ivl_signal_basename(var));
+	    return;
+      }
+
       if (ivl_signal_dimensions(var) == 0) {
 	    fprintf(vvp_out, "    %%store/real v%p_0;\n", var);
 	    return;
@@ -788,6 +863,19 @@ static int show_stmt_assign_sig_string(ivl_statement_t net)
       assert(ivl_stmt_lvals(net) == 1);
       assert(ivl_stmt_opcode(net) == 0);
 
+	/* Special case: If the l-value signal (string) is named after
+	   its scope, and the scope is a function, then this is an
+	   assign to a return value and should be handled
+	   differently. */
+      if (signal_is_return_value(var)) {
+	    assert(ivl_signal_dimensions(var) == 0);
+	    assert(part == 0 && aidx == 0);
+	    draw_eval_string(rval);
+	    fprintf(vvp_out, "    %%ret/str 0; Assign to %s\n",
+		    ivl_signal_basename(var));
+	    return 0;
+      }
+
 	/* Simplest case: no mux. Evaluate the r-value as a string and
 	   store the result into the variable. Note that the
 	   %store/str opcode pops the string result. */
@@ -848,12 +936,14 @@ static int show_stmt_assign_darray_pattern(ivl_statement_t net)
       ivl_lval_t lval = ivl_stmt_lval(net, 0);
       ivl_expr_t rval = ivl_stmt_rval(net);
 
-      ivl_signal_t var= ivl_lval_sig(lval);
+      ivl_signal_t var = ivl_lval_sig(lval);
       ivl_type_t var_type= ivl_signal_net_type(var);
       assert(ivl_type_base(var_type) == IVL_VT_DARRAY);
 
       ivl_type_t element_type = ivl_type_element(var_type);
       unsigned idx;
+      unsigned size_reg = allocate_word();
+
 #if 0
       unsigned element_width = 1;
       if (ivl_type_base(element_type) == IVL_VT_BOOL)
@@ -861,6 +951,16 @@ static int show_stmt_assign_darray_pattern(ivl_statement_t net)
       else if (ivl_type_base(element_type) == IVL_VT_LOGIC)
 	    element_width = width_of_packed_type(element_type);
 #endif
+
+// FIXME: At the moment we reallocate the array space.
+//        This probably should be a resize to avoid values glitching
+	/* Allocate at least enough space for the array patter. */
+      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", size_reg, ivl_expr_parms(rval));
+	/* This can not have have a X/Z value so clear flag 4. */
+      fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
+      darray_new(element_type, size_reg);
+      fprintf(vvp_out, "    %%store/obj v%p_0;\n", var);
+
       assert(ivl_expr_type(rval) == IVL_EX_ARRAY_PATTERN);
       for (idx = 0 ; idx < ivl_expr_parms(rval) ; idx += 1) {
 	    switch (ivl_type_base(element_type)) {
@@ -910,33 +1010,26 @@ static int show_stmt_assign_sig_darray(ivl_statement_t net)
       assert(ivl_stmt_opcode(net) == 0);
       assert(part == 0);
 
-      if (mux && (ivl_type_base(element_type)==IVL_VT_REAL)) {
+      if (mux && (ivl_type_base(element_type) == IVL_VT_REAL)) {
 	    draw_eval_real(rval);
-
-	      /* The %set/dar expects the array index to be in index
+	      /* The %store/dar/r expects the array index to be in index
 		 register 3. Calculate the index in place. */
 	    draw_eval_expr_into_integer(mux, 3);
-
 	    fprintf(vvp_out, "    %%store/dar/r v%p_0;\n", var);
 
-      } else if (mux && ivl_type_base(element_type)==IVL_VT_STRING) {
-
-	      /* Evaluate the rval into the top of the string stack. */
+      } else if (mux && ivl_type_base(element_type) == IVL_VT_STRING) {
 	    draw_eval_string(rval);
-
-	      /* The %store/dar/s expects the array index to me in index
+	      /* The %store/dar/str expects the array index to me in index
 		 register 3. Calculate the index in place. */
 	    draw_eval_expr_into_integer(mux, 3);
-
 	    fprintf(vvp_out, "    %%store/dar/str v%p_0;\n", var);
 
       } else if (mux) {
 	    draw_eval_vec4(rval);
-
+	    resize_vec4_wid(rval, ivl_stmt_lwidth(net));
 	      /* The %store/dar/vec4 expects the array index to be in index
 		 register 3. Calculate the index in place. */
 	    draw_eval_expr_into_integer(mux, 3);
-
 	    fprintf(vvp_out, "    %%store/dar/vec4 v%p_0;\n", var);
 
       } else if (ivl_expr_type(rval) == IVL_EX_ARRAY_PATTERN) {
@@ -956,27 +1049,147 @@ static int show_stmt_assign_sig_darray(ivl_statement_t net)
       return errors;
 }
 
+/*
+ * This function handles the special case that we assign an array
+ * pattern to a queue. Handle this by assigning each element.
+ * The array pattern will have a fixed size.
+ */
+static int show_stmt_assign_queue_pattern(ivl_signal_t var, ivl_expr_t rval,
+                                          ivl_type_t element_type, int max_idx)
+{
+      int errors = 0;
+      unsigned idx;
+      unsigned max_size;
+      unsigned max_elems;
+      assert(ivl_expr_type(rval) == IVL_EX_ARRAY_PATTERN);
+      max_size = ivl_signal_array_count(var);
+      max_elems = ivl_expr_parms(rval);
+      if ((max_size != 0) && (max_elems > max_size)) {
+	    fprintf(stderr, "%s:%u: Warning: Array pattern assignment has more elements "
+	                    "(%u) than bounded queue '%s' supports (%u).\n"
+	                    "         Only using first %u elements.\n",
+	                    ivl_expr_file(rval), ivl_expr_lineno(rval),
+	                    max_elems, ivl_signal_basename(var), max_size, max_size);
+	    max_elems = max_size;
+      }
+      for (idx = 0 ; idx < max_elems ; idx += 1) {
+	    switch (ivl_type_base(element_type)) {
+		case IVL_VT_BOOL:
+		case IVL_VT_LOGIC:
+		  draw_eval_vec4(ivl_expr_parm(rval,idx));
+		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%store/qdar/v v%p_0, %d, %u;\n", var, max_idx,
+		                   width_of_packed_type(element_type));
+		  break;
+
+		case IVL_VT_REAL:
+		  draw_eval_real(ivl_expr_parm(rval,idx));
+		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%store/qdar/r v%p_0, %d;\n", var, max_idx);
+		  break;
+
+		case IVL_VT_STRING:
+		  draw_eval_string(ivl_expr_parm(rval,idx));
+		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%store/qdar/str v%p_0, %d;\n", var, max_idx);
+		  break;
+
+		default:
+		  fprintf(vvp_out, "; ERROR: show_stmt_assign_queue_pattern: "
+		                   "type_base=%d not implemented\n", ivl_type_base(element_type));
+		  errors += 1;
+		  break;
+	    }
+      }
+
+      if ((max_size == 0) || (max_elems < max_size)) {
+	    int del_idx = allocate_word();
+	    assert(del_idx >= 0);
+	      /* Save the first queue element to delete. */
+	    fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", del_idx, max_elems);
+	    fprintf(vvp_out, "    %%delete/tail v%p_0, %u;\n", var, del_idx);
+	    clr_word(del_idx);
+      }
+
+      return errors;
+}
+
 static int show_stmt_assign_sig_queue(ivl_statement_t net)
 {
       int errors = 0;
       ivl_lval_t lval = ivl_stmt_lval(net, 0);
       ivl_expr_t rval = ivl_stmt_rval(net);
+      ivl_expr_t part = ivl_lval_part_off(lval);
       ivl_signal_t var= ivl_lval_sig(lval);
       ivl_type_t var_type= ivl_signal_net_type(var);
+      ivl_type_t element_type = ivl_type_element(var_type);
+
+      ivl_expr_t mux  = ivl_lval_idx(lval);
+
+      assert(ivl_stmt_lvals(net) == 1);
+      assert(ivl_stmt_opcode(net) == 0);
+      assert(part == 0);
+
       assert(ivl_type_base(var_type) == IVL_VT_QUEUE);
 
-      switch (ivl_expr_type(rval)) {
-	  case IVL_EX_NULL:
+      int idx = allocate_word();
+      assert(idx >= 0);
+        /* Save the queue maximum index value to an integer register. */
+      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
+
+      if (ivl_expr_type(rval) == IVL_EX_NULL) {
 	    errors += draw_eval_object(rval);
-	    break;
-	  default:
-	    fprintf(stderr, "XXXX: I don't know how to handle expr_type=%d here\n", ivl_expr_type(rval));
-	    fprintf(vvp_out, " ; XXXX expr_type=%d\n", ivl_expr_type(rval));
-	    errors += 1;
-	    break;
+	    fprintf(vvp_out, "    %%store/obj v%p_0;\n", var);
+
+      } else if  (mux && (ivl_type_base(element_type) == IVL_VT_REAL)) {
+	    draw_eval_real(rval);
+	      /* The %store/qdar expects the array index to be in
+		 index register 3. */
+	    draw_eval_expr_into_integer(mux, 3);
+	    fprintf(vvp_out, "    %%store/qdar/r v%p_0, %d;\n", var, idx);
+
+      } else if (mux && ivl_type_base(element_type) == IVL_VT_STRING) {
+	    draw_eval_string(rval);
+	      /* The %store/qdar expects the array index to be in
+		 index register 3. */
+	    draw_eval_expr_into_integer(mux, 3);
+	    fprintf(vvp_out, "    %%store/qdar/str v%p_0, %d;\n", var, idx);
+
+      } else if (mux) { // What is left must be some form of vector
+	    assert(ivl_type_base(element_type) == IVL_VT_BOOL ||
+                   ivl_type_base(element_type) == IVL_VT_LOGIC);
+	    draw_eval_vec4(rval);
+	    resize_vec4_wid(rval, ivl_stmt_lwidth(net));
+	      /* The %store/qdar expects the array index to be in
+		 index register 3. */
+	    draw_eval_expr_into_integer(mux, 3);
+	    fprintf(vvp_out, "    %%store/qdar/v v%p_0, %d, %u;\n", var, idx,
+	                     width_of_packed_type(element_type));
+
+      } else if (ivl_expr_type(rval) == IVL_EX_ARRAY_PATTERN) {
+	      /* There is no l-value mux, but the r-value is an array
+		 pattern. This is a special case of an assignment to
+		 the l-value. */
+	    errors += show_stmt_assign_queue_pattern(var, rval, element_type, idx);
+
+      } else {
+	      /* There is no l-value mux, so this must be an
+		 assignment to the array as a whole. Evaluate the
+		 "object", and store the evaluated result. */
+	    errors += draw_eval_object(rval);
+	    if (ivl_type_base(element_type) == IVL_VT_REAL)
+		  fprintf(vvp_out, "    %%store/qobj/r v%p_0, %d;\n", var, idx);
+	    else if (ivl_type_base(element_type) == IVL_VT_STRING)
+		  fprintf(vvp_out, "    %%store/qobj/str v%p_0, %d;\n", var, idx);
+	    else {
+		  assert(ivl_type_base(element_type) == IVL_VT_BOOL ||
+		         ivl_type_base(element_type) == IVL_VT_LOGIC);
+		  fprintf(vvp_out, "    %%store/qobj/v v%p_0, %d, %u;\n",
+		                   var, idx, width_of_packed_type(element_type));
+	    }
       }
+      clr_word(idx);
 
-      fprintf(vvp_out, "    %%store/obj v%p_0;\n", var);
       return errors;
 }
 
@@ -995,8 +1208,9 @@ static int show_stmt_assign_sig_cobject(ivl_statement_t net)
 	    ivl_type_t prop_type = ivl_type_prop_type(sig_type, prop_idx);
 
 	    if (ivl_type_base(prop_type) == IVL_VT_BOOL) {
-		  assert(ivl_type_packed_dimensions(prop_type) == 1);
-		  assert(ivl_type_packed_msb(prop_type,0) >= ivl_type_packed_lsb(prop_type, 0));
+		  assert(ivl_type_packed_dimensions(prop_type) == 0 ||
+		         (ivl_type_packed_dimensions(prop_type) == 1 &&
+		          ivl_type_packed_msb(prop_type,0) >= ivl_type_packed_lsb(prop_type, 0)));
 
 		  draw_eval_vec4(rval);
 		  if (ivl_expr_value(rval)!=IVL_VT_BOOL)
@@ -1008,8 +1222,9 @@ static int show_stmt_assign_sig_cobject(ivl_statement_t net)
 		  fprintf(vvp_out, "    %%pop/obj 1, 0;\n");
 
 	    } else if (ivl_type_base(prop_type) == IVL_VT_LOGIC) {
-		  assert(ivl_type_packed_dimensions(prop_type) == 1);
-		  assert(ivl_type_packed_msb(prop_type,0) >= ivl_type_packed_lsb(prop_type, 0));
+		  assert(ivl_type_packed_dimensions(prop_type) == 0 ||
+		         (ivl_type_packed_dimensions(prop_type) == 1 &&
+		          ivl_type_packed_msb(prop_type,0) >= ivl_type_packed_lsb(prop_type, 0)));
 
 		  draw_eval_vec4(rval);
 
