@@ -1,7 +1,7 @@
 #ifndef IVL_vvp_priv_H
 #define IVL_vvp_priv_H
 /*
- * Copyright (c) 2001-2014 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -120,7 +120,7 @@ extern char* process_octal_codes(const char*txt, unsigned wid);
  * Note: draw_modpath drive_label must be malloc'ed by the
  * caller. This function will free the string sometime in the future.
  */
-extern void draw_modpath(ivl_signal_t path_sig, char*drive_label);
+extern void draw_modpath(ivl_signal_t path_sig, char*drive_label, unsigned drive_index);
 extern void cleanup_modpath(void);
 
 /*
@@ -133,6 +133,7 @@ extern void draw_vpi_task_call(ivl_statement_t net);
 
 extern void draw_vpi_func_call(ivl_expr_t expr);
 extern void draw_vpi_rfunc_call(ivl_expr_t expr);
+extern void draw_vpi_sfunc_call(ivl_expr_t expr);
 
 extern void draw_class_in_scope(ivl_type_t classtype);
 
@@ -208,6 +209,10 @@ extern void draw_eval_expr_into_integer(ivl_expr_t expr, unsigned ix);
  */
 extern int draw_eval_condition(ivl_expr_t expr);
 
+/*
+ * Return true if the signal is the return value of a function.
+ */
+extern int signal_is_return_value(ivl_signal_t sig);
 
 extern int number_is_unknown(ivl_expr_t ex);
 extern int number_is_immediate(ivl_expr_t ex, unsigned lim_wid, int negative_is_ok);
@@ -281,4 +286,7 @@ extern void clr_flag(int idx);
 extern unsigned local_count;
 extern unsigned thread_count;
 
+extern void darray_new(ivl_type_t element_type, unsigned size_reg);
+
+
 #endif /* IVL_vvp_priv_H */
