@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -225,14 +225,9 @@ static void assign_to_lvector(ivl_lval_t lval,
 
       const unsigned long use_word = 0;
 
-      if (ivl_signal_type(sig) == IVL_SIT_UWIRE) {
-	    fprintf(stderr, "%s:%u: tgt-vvp sorry: V10 does not support "
-		    "mixed continuous and non-blocking assignments to "
-		    "different parts of the same vector (%s).\n",
-		    ivl_signal_file(sig), ivl_signal_lineno(sig),
-		    ivl_signal_basename(sig));
-	    vvp_errors += 1;
-      }
+      const char*assign_op = "%assign";
+      if (ivl_signal_type(sig) == IVL_SIT_UWIRE)
+	    assign_op = "%force";
 
 	// Detect the case that this is actually a non-blocking assign
 	// to an array word. In that case, run off somewhere else to
@@ -272,8 +267,8 @@ static void assign_to_lvector(ivl_lval_t lval,
 		       to know to skip the assign. */
 		  draw_eval_expr_into_integer(part_off_ex, offset_index);
 		    /* If the index expression has XZ bits, skip the assign. */
-		  fprintf(vvp_out, "    %%assign/vec4/off/d v%p_%lu, %d, %d;\n",
-			  sig, use_word, offset_index, delay_index);
+		  fprintf(vvp_out, "    %s/vec4/off/d v%p_%lu, %d, %d;\n",
+			  assign_op, sig, use_word, offset_index, delay_index);
 
 		  clr_word(offset_index);
 		  clr_word(delay_index);
@@ -282,8 +277,8 @@ static void assign_to_lvector(ivl_lval_t lval,
 		  int offset_index = allocate_word();
 		    /* Event control delay... */
 		  draw_eval_expr_into_integer(part_off_ex, offset_index);
-		  fprintf(vvp_out, "    %%assign/vec4/off/e v%p_%lu, %d;\n",
-			  sig, use_word, offset_index);
+		  fprintf(vvp_out, "    %s/vec4/off/e v%p_%lu, %d;\n",
+			  assign_op, sig, use_word, offset_index);
 		  fprintf(vvp_out, "    %%evctl/c;\n");
 
 		  clr_word(offset_index);
@@ -301,8 +296,8 @@ static void assign_to_lvector(ivl_lval_t lval,
 		       to know to skip the assign. */
 		  draw_eval_expr_into_integer(part_off_ex, offset_index);
 		    /* If the index expression has XZ bits, skip the assign. */
-		  fprintf(vvp_out, "    %%assign/vec4/off/d v%p_%lu, %d, %d;\n",
-			  sig, use_word, offset_index, delay_index);
+		  fprintf(vvp_out, "    %s/vec4/off/d v%p_%lu, %d, %d;\n",
+			  assign_op, sig, use_word, offset_index, delay_index);
 		  clr_word(offset_index);
 		  clr_word(delay_index);
 	    }
@@ -315,8 +310,8 @@ static void assign_to_lvector(ivl_lval_t lval,
 		  fprintf(vvp_out, "    %%ix/load %d, %lu, 0;\n",
 			  offset_index, part_off);
 		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
-		  fprintf(vvp_out, "    %%assign/vec4/off/e v%p_%lu, %d;\n",
-			  sig, use_word, offset_index);
+		  fprintf(vvp_out, "    %s/vec4/off/e v%p_%lu, %d;\n",
+			  assign_op, sig, use_word, offset_index);
 		  fprintf(vvp_out, "    %%evctl/c;\n");
 		  clr_word(offset_index);
 
@@ -334,8 +329,8 @@ static void assign_to_lvector(ivl_lval_t lval,
 				delay_index, low_d, hig_d);
 			fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  }
-		  fprintf(vvp_out, "    %%assign/vec4/off/d v%p_%lu, %d, %d;\n",
-			  sig, use_word, offset_index, delay_index);
+		  fprintf(vvp_out, "    %s/vec4/off/d v%p_%lu, %d, %d;\n",
+			  assign_op, sig, use_word, offset_index, delay_index);
 		  clr_word(offset_index);
 		  clr_word(delay_index);
 	    }
@@ -344,13 +339,13 @@ static void assign_to_lvector(ivl_lval_t lval,
 	      /* Calculated delay... */
 	    int delay_index = allocate_word();
 	    draw_eval_expr_into_integer(dexp, delay_index);
-	    fprintf(vvp_out, "    %%assign/vec4/d v%p_%lu, %d;\n",
-		    sig, use_word, delay_index);
+	    fprintf(vvp_out, "    %s/vec4/d v%p_%lu, %d;\n",
+		    assign_op, sig, use_word, delay_index);
 	    clr_word(delay_index);
       } else if (nevents != 0) {
 	      /* Event control delay... */
-	    fprintf(vvp_out, "    %%assign/vec4/e v%p_%lu;\n",
-		    sig, use_word);
+	    fprintf(vvp_out, "    %s/vec4/e v%p_%lu;\n",
+		    assign_op, sig, use_word);
 	    fprintf(vvp_out, "    %%evctl/c;\n");
 
       } else {
@@ -362,12 +357,12 @@ static void assign_to_lvector(ivl_lval_t lval,
 		  int delay_index = allocate_word();
 		  fprintf(vvp_out, "    %%ix/load %d, %lu, %lu;\n",
 		          delay_index, low_d, hig_d);
-		  fprintf(vvp_out, "    %%assign/vec4/d v%p_%lu, %d;\n",
-		          sig, use_word, delay_index);
+		  fprintf(vvp_out, "    %s/vec4/d v%p_%lu, %d;\n",
+		          assign_op, sig, use_word, delay_index);
 		  clr_word(delay_index);
 	    } else {
-		  fprintf(vvp_out, "    %%assign/vec4 v%p_%lu, %lu;\n",
-		          sig, use_word, low_d);
+		  fprintf(vvp_out, "    %s/vec4 v%p_%lu, %lu;\n",
+		          assign_op, sig, use_word, low_d);
 	    }
       }
 }
@@ -649,6 +644,7 @@ static int show_stmt_block_named(ivl_statement_t net, ivl_scope_t scope)
 static int show_stmt_case(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
+      ivl_case_quality_t qual = ivl_stmt_case_quality(net);
       ivl_expr_t expr = ivl_stmt_cond_expr(net);
       unsigned count = ivl_stmt_case_count(net);
 
@@ -656,6 +652,12 @@ static int show_stmt_case(ivl_statement_t net, ivl_scope_t sscope)
 
       unsigned idx, default_case;
 
+      if (qual != IVL_CASE_QUALITY_BASIC && qual != IVL_CASE_QUALITY_PRIORITY) {
+	    fprintf(stderr, "%s:%u: vvp.tgt sorry: "
+		    "Case unique/unique0 qualities are ignored.\n",
+		    ivl_stmt_file(net), ivl_stmt_lineno(net));
+      }
+
       show_stmt_file_line(net, "Case statement.");
 
       local_count += count + 1;
@@ -714,6 +716,23 @@ static int show_stmt_case(ivl_statement_t net, ivl_scope_t sscope)
 	    ivl_statement_t cst = ivl_stmt_case_stmt(net, default_case);
 	    rc += show_statement(cst, sscope);
       }
+	/* Emit code to check unique and priority have handled a value
+           (when there is no default). */
+      else if (default_case == count) {
+          switch (qual) {
+          case IVL_CASE_QUALITY_UNIQUE:
+          case IVL_CASE_QUALITY_PRIORITY:
+              fprintf(vvp_out, "    %%vpi_call/w %u %u \"$warning\", "
+                      "\"value is unhandled for priority or unique case statement\""
+                      " {0 0 0};\n",
+                      ivl_file_table_index(ivl_stmt_file(net)),
+                      ivl_stmt_lineno(net));
+              break;
+
+          default:
+              break;
+          }
+      }
 
 	/* Jump to the out of the case. */
       fprintf(vvp_out, "    %%jmp T_%u.%u;\n", thread_count,
@@ -863,7 +882,7 @@ static void force_real_to_lval(ivl_statement_t net)
 	      /* We do not currently support using a word from a variable
 		 array as the L-value (SystemVerilog / Icarus extension). */
 	    if (ivl_signal_type(lsig) == IVL_SIT_REG) {
-		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s to the "
+		  fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s to the "
 			  "word of a variable array (%s[%ld]).\n",
 			  ivl_stmt_file(net), ivl_stmt_lineno(net),
 			  command_name, ivl_signal_basename(lsig),
@@ -919,7 +938,7 @@ static void force_vector_to_lval(ivl_statement_t net)
 		    /* We do not currently support using a word from a variable
 		       array as the L-value (SystemVerilog / Icarus extension). */
 		  if (ivl_signal_type(lsig) == IVL_SIT_REG) {
-			fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s to the "
+			fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s to the "
 				"word of a variable array (%s[%ld]).\n",
 				ivl_stmt_file(net), ivl_stmt_lineno(net),
 				command_name, ivl_signal_basename(lsig),
@@ -965,7 +984,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 	        ivl_expr_type(rval) == IVL_EX_REALNUM)
 		  return;
 
-	    fprintf(stderr, "%s:%u: tgt-vvp sorry: procedural continuous "
+	    fprintf(stderr, "%s:%u: vvp.tgt sorry: procedural continuous "
 		    "assignments are not yet fully supported. The RHS of "
 		    "this assignment will only be evaluated once, at the "
 		    "time the assignment statement is executed.\n",
@@ -1009,7 +1028,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 	    long use_wid = ivl_signal_width(rsig);
 	    long use_lsb, use_msb;
 	    if (ivl_signal_packed_dimensions(lsig) > 1) {
-		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s part of a "
+		  fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s part of a "
 		                  "packed array.\n",
 		                  ivl_stmt_file(net), ivl_stmt_lineno(net),
 		                  command_name);
@@ -1024,7 +1043,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 		  use_msb = use_lsb;
 		  use_msb -= use_wid - 1;
 	    }
-	    fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s signal to a ",
+	    fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s signal to a ",
 	            ivl_stmt_file(net), ivl_stmt_lineno(net), command_name);
 	    if (use_wid > 1) {
 		  fprintf(stderr, "part select (%s[%ld:%ld]).\n",
@@ -1051,7 +1070,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 		    /* Normalize the array access. */
 		  long real_word = use_lword;
 		  real_word += ivl_signal_array_base(lsig);
-		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s to the "
+		  fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s to the "
 		          "word of a variable array (%s[%ld]).\n",
 		          ivl_stmt_file(net), ivl_stmt_lineno(net),
 		          command_name, ivl_signal_basename(lsig), real_word);
@@ -1062,7 +1081,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
       if ((rword_idx = ivl_expr_oper1(rval)) != 0) {
 	    assert(ivl_signal_dimensions(rsig) != 0);
 	    if (!number_is_immediate(rword_idx, IMM_WID, 0)) {
-		  fprintf(stderr, "%s:%u: tgt-vvp sorry: procedural continuous "
+		  fprintf(stderr, "%s:%u: vvp.tgt sorry: procedural continuous "
 			  "assignments are not yet fully supported. The RHS of "
 			  "this assignment will only be evaluated once, at the "
 			  "time the assignment statement is executed.\n",
@@ -1081,7 +1100,7 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 		    /* Normalize the array access. */
 		  long real_word = use_rword;
 		  real_word += ivl_signal_array_base(rsig);
-		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s from the "
+		  fprintf(stderr, "%s:%u: vvp.tgt sorry: cannot %s from the "
 		          "word of a variable array (%s[%ld]).\n",
 		          ivl_expr_file(rval), ivl_expr_lineno(rval),
 		          command_name, ivl_signal_basename(rsig), real_word);
@@ -1630,16 +1649,25 @@ static int show_stmt_utask(ivl_statement_t net)
 
       show_stmt_file_line(net, "User task call.");
 
-      fprintf(vvp_out, "    %%fork TD_%s",
-	      vvp_mangle_id(ivl_scope_name(task)));
-      fprintf(vvp_out, ", S_%p;\n", task);
-      fprintf(vvp_out, "    %%join;\n");
+      if (ivl_scope_type(task) == IVL_SCT_FUNCTION) {
+	      // A function called as a task is (presumably) a void function.
+	      // Use the %callf/void instruction to call it.
+	    fprintf(vvp_out, "    %%callf/void TD_%s",
+		    vvp_mangle_id(ivl_scope_name(task)));
+	    fprintf(vvp_out, ", S_%p;\n", task);
+      } else {
+	    fprintf(vvp_out, "    %%fork TD_%s",
+		    vvp_mangle_id(ivl_scope_name(task)));
+	    fprintf(vvp_out, ", S_%p;\n", task);
+	    fprintf(vvp_out, "    %%join;\n");
+      }
 
       return 0;
 }
 
 static int show_stmt_wait(ivl_statement_t net, ivl_scope_t sscope)
 {
+      static unsigned int cascade_counter = 0;
 	/* Look to see if this is a SystemVerilog wait fork. */
       if ((ivl_stmt_nevent(net) == 1) && (ivl_stmt_events(net, 0) == 0)) {
 	    assert(ivl_statement_type(ivl_stmt_sub_stmt(net)) == IVL_ST_NOOP);
@@ -1650,13 +1678,22 @@ static int show_stmt_wait(ivl_statement_t net, ivl_scope_t sscope)
 
       show_stmt_file_line(net, "Event wait (@) statement.");
 
-      if (ivl_stmt_nevent(net) == 1) {
+      if (ivl_stmt_nevent(net) == 0) {
+	    assert(ivl_stmt_needs_t0_trigger(net) == 1);
+	    fprintf(vvp_out, "    %%wait E_0x0;\n");
+      } else if (ivl_stmt_nevent(net) == 1) {
 	    ivl_event_t ev = ivl_stmt_events(net, 0);
-	    fprintf(vvp_out, "    %%wait E_%p;\n", ev);
+	    if (ivl_stmt_needs_t0_trigger(net)) {
+		  fprintf(vvp_out, "Ewait_%u .event/or E_%p, E_0x0;\n",
+		                   cascade_counter, ev);
+		  fprintf(vvp_out, "    %%wait Ewait_%u;\n", cascade_counter);
+		  cascade_counter += 1;
+	    } else {
+		  fprintf(vvp_out, "    %%wait E_%p;\n", ev);
+	    }
 
       } else {
 	    unsigned idx;
-	    static unsigned int cascade_counter = 0;
 	    ivl_event_t ev = ivl_stmt_events(net, 0);
 	    fprintf(vvp_out, "Ewait_%u .event/or E_%p", cascade_counter, ev);
 
@@ -1664,6 +1701,7 @@ static int show_stmt_wait(ivl_statement_t net, ivl_scope_t sscope)
 		  ev = ivl_stmt_events(net, idx);
 		  fprintf(vvp_out, ", E_%p", ev);
 	    }
+	    assert(ivl_stmt_needs_t0_trigger(net) == 0);
 	    fprintf(vvp_out, ";\n    %%wait Ewait_%u;\n", cascade_counter);
 	    cascade_counter += 1;
       }
@@ -1710,14 +1748,68 @@ static int show_delete_method(ivl_statement_t net)
       show_stmt_file_line(net, "Delete object");
 
       unsigned parm_count = ivl_stmt_parm_count(net);
-      if (parm_count < 1)
+      if ((parm_count < 1) || (parm_count > 2))
 	    return 1;
 
       ivl_expr_t parm = ivl_stmt_parm(net, 0);
       assert(ivl_expr_type(parm) == IVL_EX_SIGNAL);
       ivl_signal_t var = ivl_expr_signal(parm);
 
-      fprintf(vvp_out, "    %%delete/obj v%p_0;\n", var);
+	/* If this is a queue then it can have an element to delete. */
+      if (parm_count == 2) {
+	    if (ivl_type_base(ivl_signal_net_type(var)) != IVL_VT_QUEUE)
+		  return 1;
+	    draw_eval_expr_into_integer(ivl_stmt_parm(net, 1), 3);
+	    fprintf(vvp_out, "    %%delete/elem v%p_0;\n", var);
+      } else {
+	    fprintf(vvp_out, "    %%delete/obj v%p_0;\n", var);
+      }
+      return 0;
+}
+
+static int show_insert_method(ivl_statement_t net)
+{
+      show_stmt_file_line(net, "queue: insert");
+
+      unsigned parm_count = ivl_stmt_parm_count(net);
+      if (parm_count != 3)
+	    return 1;
+
+      ivl_expr_t parm0 = ivl_stmt_parm(net,0);
+      assert(ivl_expr_type(parm0) == IVL_EX_SIGNAL);
+      ivl_signal_t var = ivl_expr_signal(parm0);
+      ivl_type_t var_type = ivl_signal_net_type(var);
+      assert(ivl_type_base(var_type) == IVL_VT_QUEUE);
+
+      int idx = allocate_word();
+      assert(idx >= 0);
+	/* Save the queue maximum index value to an integer register. */
+      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
+
+      ivl_type_t element_type = ivl_type_element(var_type);
+
+      ivl_expr_t parm1 = ivl_stmt_parm(net,1);
+	/* The %qinsert expects the array index to be in index register 3. */
+      draw_eval_expr_into_integer(parm1, 3);
+      ivl_expr_t parm2 = ivl_stmt_parm(net,2);
+      switch (ivl_type_base(element_type)) {
+	  case IVL_VT_REAL:
+	    draw_eval_real(parm2);
+	    fprintf(vvp_out, "    %%qinsert/real v%p_0, %u;\n",
+	            var, idx);
+	    break;
+	  case IVL_VT_STRING:
+	    draw_eval_string(parm2);
+	    fprintf(vvp_out, "    %%qinsert/str v%p_0, %u;\n",
+	            var, idx);
+	    break;
+	  default:
+	    draw_eval_vec4(parm2);
+	    fprintf(vvp_out, "    %%qinsert/v v%p_0, %u, %u;\n",
+	            var, idx,
+	            width_of_packed_type(element_type));
+	    break;
+      }
       return 0;
 }
 
@@ -1740,7 +1832,12 @@ static int show_push_frontback_method(ivl_statement_t net, bool is_front)
       assert(ivl_expr_type(parm0) == IVL_EX_SIGNAL);
       ivl_signal_t var = ivl_expr_signal(parm0);
       ivl_type_t var_type = ivl_signal_net_type(var);
-      assert(ivl_type_base(var_type)== IVL_VT_QUEUE);
+      assert(ivl_type_base(var_type) == IVL_VT_QUEUE);
+
+      int idx = allocate_word();
+      assert(idx >= 0);
+	/* Save the queue maximum index value to an integer register. */
+      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
 
       ivl_type_t element_type = ivl_type_element(var_type);
 
@@ -1748,18 +1845,22 @@ static int show_push_frontback_method(ivl_statement_t net, bool is_front)
       switch (ivl_type_base(element_type)) {
 	  case IVL_VT_REAL:
 	    draw_eval_real(parm1);
-	    fprintf(vvp_out, "    %%store/%s/r v%p_0;\n", type_code, var);
+	    fprintf(vvp_out, "    %%store/%s/r v%p_0, %u;\n",
+	            type_code, var, idx);
 	    break;
 	  case IVL_VT_STRING:
 	    draw_eval_string(parm1);
-	    fprintf(vvp_out, "    %%store/%s/str v%p_0;\n", type_code, var);
+	    fprintf(vvp_out, "    %%store/%s/str v%p_0, %u;\n",
+	            type_code, var, idx);
 	    break;
 	  default:
 	    draw_eval_vec4(parm1);
-	    fprintf(vvp_out, "    %%store/%s/v v%p_0, %u;\n",
-		    type_code, var, width_of_packed_type(element_type));
+	    fprintf(vvp_out, "    %%store/%s/v v%p_0, %u, %u;\n",
+	            type_code, var, idx,
+	            width_of_packed_type(element_type));
 	    break;
       }
+      clr_word(idx);
 
       return 0;
 }
@@ -1771,6 +1872,9 @@ static int show_system_task_call(ivl_statement_t net)
       if (strcmp(stmt_name,"$ivl_darray_method$delete") == 0)
 	    return show_delete_method(net);
 
+      if (strcmp(stmt_name,"$ivl_queue_method$insert") == 0)
+	    return show_insert_method(net);
+
       if (strcmp(stmt_name,"$ivl_queue_method$push_front") == 0)
 	    return show_push_frontback_method(net, true);
 
@@ -1993,6 +2097,7 @@ static unsigned utask_in_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
       sig_name = ivl_signal_basename(lsig);
       for (idx = 0; idx < ports; idx += 1) {
 	    ivl_signal_t port = ivl_scope_port(scope, idx);
+	    if (!port) continue;
 	    ivl_signal_port_t port_type = ivl_signal_port(port);
 	    if ((port_type != IVL_SIP_INPUT) &&
 	        (port_type != IVL_SIP_INOUT)) continue;
@@ -2032,6 +2137,7 @@ static unsigned utask_out_port_idx(ivl_scope_t scope, ivl_statement_t stmt)
       sig_name = ivl_signal_basename(rsig);
       for (idx = 0; idx < ports; idx += 1) {
 	    ivl_signal_t port = ivl_scope_port(scope, idx);
+	    if (!port) continue;
 	    ivl_signal_port_t port_type = ivl_signal_port(port);
 	    if ((port_type != IVL_SIP_OUTPUT) &&
 	        (port_type != IVL_SIP_INOUT)) continue;
@@ -2071,6 +2177,7 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
                                         ivl_statement_t stmt)
 {
       unsigned idx, ports, task_idx = 0;
+      unsigned is_void_func;
       unsigned count = ivl_stmt_block_count(stmt);
       unsigned lineno = ivl_stmt_lineno(stmt);
       ivl_scope_t task_scope = 0;
@@ -2085,7 +2192,8 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
 	    if (ivl_statement_type(tmp) == IVL_ST_UTASK && !task_scope) {
 		  task_idx = idx;
 		  task_scope = ivl_stmt_call(tmp);
-		  assert(ivl_scope_type(task_scope) == IVL_SCT_TASK);
+		  assert((ivl_scope_type(task_scope) == IVL_SCT_TASK) ||
+		         (ivl_scope_type(task_scope) == IVL_SCT_FUNCTION));
 		  continue;
 	    }
 	    return 0;
@@ -2136,7 +2244,11 @@ static unsigned is_utask_call_with_args(ivl_scope_t scope,
       }
 
 	/* Verify that all the ports were defined. */
-      for (idx = 0; idx < ports; idx += 1) {
+      is_void_func = (ivl_scope_type(task_scope) == IVL_SCT_FUNCTION);
+      if (is_void_func) {
+	    assert(port_exprs[0].type == IVL_SIP_NONE);
+      }
+      for (idx = is_void_func; idx < ports; idx += 1) {
 	    if (port_exprs[idx].type == IVL_SIP_NONE) {
 		  free(port_exprs);
 		  return 0;
@@ -2355,6 +2467,9 @@ int draw_process(ivl_process_t net, void*x)
 	    break;
 
 	  case IVL_PR_ALWAYS:
+	  case IVL_PR_ALWAYS_COMB:
+	  case IVL_PR_ALWAYS_FF:
+	  case IVL_PR_ALWAYS_LATCH:
 	    fprintf(vvp_out, "    %%jmp T_%u;\n", thread_count);
 	    break;
       }
@@ -2365,6 +2480,9 @@ int draw_process(ivl_process_t net, void*x)
 
 	  case IVL_PR_INITIAL:
 	  case IVL_PR_ALWAYS:
+	  case IVL_PR_ALWAYS_COMB:
+	  case IVL_PR_ALWAYS_FF:
+	  case IVL_PR_ALWAYS_LATCH:
 	    if (init_flag) {
 		  fprintf(vvp_out, "    .thread T_%u, $init;\n", thread_count);
 	    } else if (push_flag) {
