@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -290,6 +290,16 @@ static unsigned is_netlist_signal(ivl_net_logic_t net, ivl_nexus_t nex)
       return rtn;
 }
 
+int signal_is_return_value(ivl_signal_t sig)
+{
+      ivl_scope_t sig_scope = ivl_signal_scope(sig);
+      if (ivl_scope_type(sig_scope) != IVL_SCT_FUNCTION)
+	    return 0;
+      if (strcmp(ivl_signal_basename(sig), ivl_scope_basename(sig_scope)) == 0)
+	    return 1;
+      return 0;
+}
+
 /*
  * This tests a bufz device against an output receiver, and determines
  * if the device can be skipped. If this function returns false, then a
@@ -449,6 +459,33 @@ static void draw_reg_in_scope(ivl_signal_t sig)
 	    break;
       }
 
+	/* Special Case: If this variable is the return value of a function,
+	   then it need to exist as an actual variable. */
+      if ((ivl_signal_data_type(sig)==IVL_VT_REAL)
+	  && signal_is_return_value(sig)) {
+	    fprintf(vvp_out, "; Variable %s is REAL return value of scope S_%p\n",
+		    ivl_signal_basename(sig), ivl_signal_scope(sig));
+	    return;
+      }
+      if ((ivl_signal_data_type(sig)==IVL_VT_STRING)
+	  && signal_is_return_value(sig)) {
+	    fprintf(vvp_out, "; Variable %s is string return value of scope S_%p\n",
+		    ivl_signal_basename(sig), ivl_signal_scope(sig));
+	    return;
+      }
+      if ((ivl_signal_data_type(sig)==IVL_VT_LOGIC)
+	  && signal_is_return_value(sig)) {
+	    fprintf(vvp_out, "; Variable %s is vec4 return value of scope S_%p\n",
+		    ivl_signal_basename(sig), ivl_signal_scope(sig));
+	    return;
+      }
+      if ((ivl_signal_data_type(sig)==IVL_VT_BOOL)
+	  && signal_is_return_value(sig)) {
+	    fprintf(vvp_out, "; Variable %s is bool return value of scope S_%p\n",
+		    ivl_signal_basename(sig), ivl_signal_scope(sig));
+	    return;
+      }
+
       const char *datatype_flag = ivl_signal_integer(sig) ? "/i" :
 			       ivl_signal_signed(sig)? "/s" : "";
       const char *local_flag = local_flag_str(sig);
@@ -503,13 +540,15 @@ static void draw_reg_in_scope(ivl_signal_t sig)
 		    swapped ? first: last, swapped ? last : first, msb, lsb);
 
       } else if (ivl_signal_data_type(sig) == IVL_VT_DARRAY) {
-	    fprintf(vvp_out, "v%p_0 .var/darray \"%s\";%s\n", sig,
+	    fprintf(vvp_out, "v%p_0 .var/darray \"%s\", %u;%s\n", sig,
 		    vvp_mangle_name(ivl_signal_basename(sig)),
+		    ivl_signal_width(sig),
 		    ivl_signal_local(sig)? " Local signal" : "");
 
       } else if (ivl_signal_data_type(sig) == IVL_VT_QUEUE) {
-	    fprintf(vvp_out, "v%p_0 .var/queue \"%s\";%s\n", sig,
+	    fprintf(vvp_out, "v%p_0 .var/queue \"%s\", %u;%s\n", sig,
 		    vvp_mangle_name(ivl_signal_basename(sig)),
+		    ivl_signal_width(sig),
 		    ivl_signal_local(sig)? " Local signal" : "");
 
       } else if (ivl_signal_data_type(sig) == IVL_VT_STRING) {
@@ -870,6 +909,32 @@ static void draw_udp_in_scope(ivl_net_logic_t lptr)
       if (need_delay_flag) draw_logic_delay(lptr);
 }
 
+static void draw_equiv_impl_in_scope(ivl_net_logic_t lptr)
+{
+      unsigned ninp;
+      const char *lval;
+      const char *rval;
+      const char*ltype = "?";
+
+      assert(width_of_nexus(ivl_logic_pin(lptr, 0)) == 1);
+
+      assert(ivl_logic_pins(lptr) > 0);
+      ninp = ivl_logic_pins(lptr) - 1;
+      assert(ninp == 2);
+
+      lval = draw_net_input(ivl_logic_pin(lptr, 1));
+      rval = draw_net_input(ivl_logic_pin(lptr, 2));
+
+      if (ivl_logic_type(lptr) == IVL_LO_EQUIV) {
+	    ltype = "EQUIV";
+      } else {
+	    assert(ivl_logic_type(lptr) == IVL_LO_IMPL);
+	    ltype = "IMPL";
+      }
+
+      fprintf(vvp_out, "L_%p .functor %s 1, %s, %s, C4<0>, C4<0>;\n", lptr, ltype, lval, rval);
+}
+
 static void draw_logic_in_scope(ivl_net_logic_t lptr)
 {
       unsigned pdx;
@@ -947,6 +1012,11 @@ static void draw_logic_in_scope(ivl_net_logic_t lptr)
 	    ltype = "BUFIF1";
 	    break;
 
+	  case IVL_LO_EQUIV:
+	  case IVL_LO_IMPL:
+	    draw_equiv_impl_in_scope(lptr);
+	    return;
+
 	  case IVL_LO_NAND:
 	    ltype = "NAND";
 	    lcasc = "AND";
@@ -1043,7 +1113,7 @@ static void draw_logic_in_scope(ivl_net_logic_t lptr)
 		  }
 		  for (pdx = inst; pdx < (unsigned)ninp && pdx < inst+4 ; pdx += 1) {
 			if (level) {
-			      fprintf(vvp_out, ", L_%p/%d/%d",
+			      fprintf(vvp_out, ", L_%p/%d/%u",
 				      lptr, level - 1, pdx*4);
 			} else {
 			      fprintf(vvp_out, ", %s", input_strings[pdx]);
@@ -1485,6 +1555,16 @@ static void draw_lpm_cmp(ivl_lpm_t net)
 	    type = "nee";
 	    signed_string = "";
 	    break;
+	  case IVL_LPM_CMP_WEQ:
+	    assert(dtc != IVL_VT_REAL); /* Should never get here! */
+	    type = "weq";
+	    signed_string = "";
+	    break;
+	  case IVL_LPM_CMP_WNE:
+	    assert(dtc != IVL_VT_REAL); /* Should never get here! */
+	    type = "wne";
+	    signed_string = "";
+	    break;
 	  default:
 	    assert(0);
       }
@@ -1701,29 +1781,29 @@ static void draw_lpm_ff(ivl_lpm_t net)
       nex = ivl_lpm_data(net,0);
       assert(nex);
       fprintf(vvp_out, "%s", draw_net_input(nex));
-      assert(width_of_nexus(nex) == width);;
+      assert(width_of_nexus(nex) == width);
 
       nex = ivl_lpm_clk(net);
       assert(nex);
-      assert(width_of_nexus(nex) == 1);;
+      assert(width_of_nexus(nex) == 1);
       fprintf(vvp_out, ", %s", draw_net_input(nex));
 
       nex = ivl_lpm_enable(net);
       if (nex) {
-	    assert(width_of_nexus(nex) == 1);;
+	    assert(width_of_nexus(nex) == 1);
 	    fprintf(vvp_out, ", %s", draw_net_input(nex));
       } else {
 	    fprintf(vvp_out, ", C4<1>");
       }
 
       if ( (nex = ivl_lpm_async_clr(net)) ) {
-	    assert(width_of_nexus(nex) == 1);;
+	    assert(width_of_nexus(nex) == 1);
 	    fprintf(vvp_out, ", %s", draw_net_input(nex));
       }
 
       if ( (nex = ivl_lpm_async_set(net)) ) {
 	    ivl_expr_t val = ivl_lpm_aset_value(net);
-	    assert(width_of_nexus(nex) == 1);;
+	    assert(width_of_nexus(nex) == 1);
 	    fprintf(vvp_out, ", %s", draw_net_input(nex));
 	    if (val) {
 		  unsigned nbits = ivl_expr_width(val);
@@ -1740,6 +1820,31 @@ static void draw_lpm_ff(ivl_lpm_t net)
       fprintf(vvp_out, ";\n");
 }
 
+/*
+ * Emit a LATCH primitive. This uses the following syntax:
+ *
+ * .latch <width> <data>, <enable>;
+ */
+static void draw_lpm_latch(ivl_lpm_t net)
+{
+      ivl_nexus_t nex;
+
+      unsigned width = ivl_lpm_width(net);
+      fprintf(vvp_out, "L_%p .latch %u ", net, width);
+
+      nex = ivl_lpm_data(net,0);
+      assert(nex);
+      fprintf(vvp_out, "%s", draw_net_input(nex));
+      assert(width_of_nexus(nex) == width);
+
+      nex = ivl_lpm_enable(net);
+      assert(nex);
+      assert(width_of_nexus(nex) == 1);
+      fprintf(vvp_out, ", %s", draw_net_input(nex));
+
+      fprintf(vvp_out, ";\n");
+}
+
 static void draw_lpm_shiftl(ivl_lpm_t net)
 {
       unsigned width = ivl_lpm_width(net);
@@ -1879,6 +1984,19 @@ static void draw_lpm_ufunc(ivl_lpm_t net)
       ivl_variable_type_t dt = data_type_of_nexus(ivl_lpm_q(net));
       const char*dly = draw_lpm_output_delay(net, dt);
 
+      const char*type_string = "";
+      switch (dt) {
+	  case IVL_VT_REAL:
+		type_string = "/real";
+		break;
+	  case IVL_VT_BOOL:
+	  case IVL_VT_LOGIC:
+		type_string = "/vec4";
+		break;
+	  default:
+		break;
+      }
+
 	/* Get all the input labels that I will use for net signals that
 	   connect to the inputs of the function. */
       ninp = ivl_lpm_size(net);
@@ -1891,7 +2009,7 @@ static void draw_lpm_ufunc(ivl_lpm_t net)
                     vvp_mangle_id(ivl_scope_name(def)),
                     ivl_lpm_width(net), ivl_lpm_trigger(net));
       else
-            fprintf(vvp_out, "L_%p%s .ufunc TD_%s, %u", net, dly,
+            fprintf(vvp_out, "L_%p%s .ufunc%s TD_%s, %u", net, dly, type_string,
                     vvp_mangle_id(ivl_scope_name(def)),
                     ivl_lpm_width(net));
 
@@ -1919,7 +2037,7 @@ static void draw_lpm_ufunc(ivl_lpm_t net)
       }
 
       fprintf(vvp_out, ")");
-
+#if 0
 	/* Now print the reference to the signal from which the
 	   result is collected. */
       { ivl_signal_t psig = ivl_scope_port(def, 0);
@@ -1928,7 +2046,7 @@ static void draw_lpm_ufunc(ivl_lpm_t net)
 
 	fprintf(vvp_out, " v%p_0", psig);
       }
-
+#endif
         /* Finally, print the scope identifier. */
       fprintf(vvp_out, " S_%p;\n", def);
 }
@@ -2057,6 +2175,10 @@ static void draw_lpm_in_scope(ivl_lpm_t net)
 	    draw_lpm_ff(net);
 	    return;
 
+	  case IVL_LPM_LATCH:
+	    draw_lpm_latch(net);
+	    return;
+
 	  case IVL_LPM_CMP_EEQ:
 	  case IVL_LPM_CMP_EQ:
 	  case IVL_LPM_CMP_EQX:
@@ -2065,6 +2187,8 @@ static void draw_lpm_in_scope(ivl_lpm_t net)
 	  case IVL_LPM_CMP_GT:
 	  case IVL_LPM_CMP_NE:
 	  case IVL_LPM_CMP_NEE:
+	  case IVL_LPM_CMP_WEQ:
+	  case IVL_LPM_CMP_WNE:
 	    draw_lpm_cmp(net);
 	    return;
 
@@ -2142,6 +2266,9 @@ int draw_scope(ivl_scope_t net, ivl_scope_t parent)
       const char *type;
 
       const char*prefix = ivl_scope_is_auto(net) ? "auto" : "";
+      char suffix[32];
+
+      suffix[0] = 0;
 
       switch (ivl_scope_type(net)) {
       case IVL_SCT_MODULE:   type = "module";   break;
@@ -2155,8 +2282,40 @@ int draw_scope(ivl_scope_t net, ivl_scope_t parent)
       default:               type = "?";        assert(0);
       }
 
-      fprintf(vvp_out, "S_%p .scope %s%s, \"%s\" \"%s\" %u %u",
-	      net, prefix, type,
+      if (ivl_scope_type(net)==IVL_SCT_FUNCTION) {
+	    switch (ivl_scope_func_type(net)) {
+		case IVL_VT_LOGIC:
+		  snprintf(suffix, sizeof suffix, ".vec4.%c%u",
+			   ivl_scope_func_signed(net)? 'u' : 's',
+			   ivl_scope_func_width(net));
+		  break;
+		case IVL_VT_BOOL:
+		  snprintf(suffix, sizeof suffix, ".vec2.%c%u",
+			   ivl_scope_func_signed(net)? 'u' : 's',
+			   ivl_scope_func_width(net));
+		  break;
+		case IVL_VT_REAL:
+		  snprintf(suffix, sizeof suffix, ".real");
+		  break;
+		case IVL_VT_STRING:
+		  snprintf(suffix, sizeof suffix, ".str");
+		  break;
+		case IVL_VT_CLASS:
+		case IVL_VT_DARRAY:
+		case IVL_VT_QUEUE:
+		  snprintf(suffix, sizeof suffix, ".obj");
+		  break;
+		case IVL_VT_VOID:
+		  snprintf(suffix, sizeof suffix, ".void");
+		  break;
+		default:
+		  assert(0);
+		  break;
+	    }
+      }
+
+      fprintf(vvp_out, "S_%p .scope %s%s%s, \"%s\" \"%s\" %u %u",
+	      net, prefix, type, suffix,
 	      vvp_mangle_name(ivl_scope_basename(net)),
               vvp_mangle_name(ivl_scope_tname(net)),
 	      ivl_file_table_index(ivl_scope_file(net)),
@@ -2183,7 +2342,7 @@ int draw_scope(ivl_scope_t net, ivl_scope_t parent)
             unsigned width = ivl_scope_mod_module_port_width(net,idx);
             if( name == 0 )
                 name = "";
-            fprintf( vvp_out, "    .port_info %u %s %u \"%s\"\n",
+            fprintf( vvp_out, "    .port_info %u %s %u \"%s\";\n",
                     idx, vvp_port_info_type_str(ptype), width,
 		    vvp_mangle_name(name) );
         }
