@@ -29,7 +29,8 @@ int VType::elaborate(Entity*, ScopeBase*) const
 int VTypeArray::elaborate(Entity*ent, ScopeBase*scope) const
 {
       int errors = 0;
-      etype_->elaborate(ent, scope);
+
+      errors += etype_->elaborate(ent, scope);
 
       for (vector<range_t>::const_iterator cur = ranges_.begin()
 		 ; cur != ranges_.end() ; ++ cur) {
@@ -43,3 +44,14 @@ int VTypeArray::elaborate(Entity*ent, ScopeBase*scope) const
 
       return errors;
 }
+
+int VTypeRangeExpr::elaborate(Entity*ent, ScopeBase*scope) const
+{
+      int errors = 0;
+
+      errors += base_->elaborate(ent, scope);
+      errors += start_->elaborate_expr(ent, scope, 0);
+      errors += end_->elaborate_expr(ent, scope, 0);
+
+      return errors;
+}
