@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -27,8 +27,6 @@
 # include  <math.h>
 # include  "ivl_alloc.h"
 
-#define IS_MCD(mcd)     !((mcd)>>31&1)
-
 // Flag to enable better compatibility with other simulators
 static unsigned compatible_flag = 0;
 
@@ -169,10 +167,11 @@ static int get_default_format(const char *name)
     int default_format;
 
     switch(name[ strlen(name)-1 ]){
-	/*  writE/strobE or monitoR or displaY/fdisplaY or sformaT */
+	/*  writE/strobE or monitoR or displaY/fdisplaY or sformaT/sformatF */
     case 'e':
     case 'r':
     case 't':
+    case 'f':
     case 'y': default_format = vpiDecStrVal; break;
     case 'h': default_format = vpiHexStrVal; break;
     case 'o': default_format = vpiOctStrVal; break;
@@ -229,7 +228,7 @@ static void get_time(char *rtn, const char *value, int prec,
       shift -= 1;
     }
     if (prec > 0) strcat(rtn, ".");
-    while(prec > 0) {
+    while (prec > 0) {
       strcat(rtn, "0");
       prec -= 1;
     }
@@ -620,10 +619,6 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
     case 't':
     case 'T':
       *idx += 1;
-      if (plus != 0) {
-        vpi_printf("WARNING: %s:%d: invalid format %s%s.\n",
-                   info->filename, info->lineno, info->name, fmtb);
-      }
       if (*idx >= info->nitems) {
         vpi_printf("WARNING: %s:%d: missing argument for %s%s.\n",
                    info->filename, info->lineno, info->name, fmtb);
@@ -645,8 +640,35 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
           vpi_printf("WARNING: %s:%d: incompatible value for %s%s.\n",
                      info->filename, info->lineno, info->name, fmtb);
         } else {
-          char *tbuf;
+          char *tbuf, *prev_suff = 0;
           PLI_INT32 time_units = vpi_get(vpiTimeUnit, info->scope);
+
+          if (plus != 0) {
+              /* Icarus-specific extension to print out time units.
+               * It is needed by vhdlpp to correctly implement time'image(). */
+              PLI_INT32 time_prec = vpi_get(vpiTimePrecision, info->scope);
+
+              /* We are about to override the suffix string set with $timeformat(),
+               * therefore we need to restore it after the call. */
+              prev_suff = timeformat_info.suff;
+
+              if (time_units < -12)
+                  timeformat_info.suff = strdup(" fs");
+              else if (time_units < -9)
+                  timeformat_info.suff = strdup(" ps");
+              else if (time_units < -6)
+                  timeformat_info.suff = strdup(" ns");
+              else if (time_units < -3)
+                  timeformat_info.suff = strdup(" us");
+              else if (time_units < 0)
+                  timeformat_info.suff = strdup(" ms");
+              else
+                  timeformat_info.suff = strdup(" s");
+
+              /* Adjust shift for get_time(), so the number indeed matches the unit */
+              time_units += (3 + (time_units % 3)) % 3 + (time_prec - time_units);
+          }
+
           unsigned swidth, free_flag = 0;
           unsigned suff_len = strlen(timeformat_info.suff);
           char *cp;
@@ -667,6 +689,13 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
           cp = tbuf;
           swidth = strlen(tbuf);
 
+          if (plus != 0) {
+              /* Restore the previous suffix string, overridden by
+               * a vhdlpp-specific $sformatf() call. */
+              free(timeformat_info.suff);
+              timeformat_info.suff = prev_suff;
+          }
+
           if (ld_zero == 1) {
             /* No leading zeros are created by this conversion so just make
              * the width 0 for this case. */
@@ -955,10 +984,10 @@ static char *get_display(unsigned int *rtnsz, const struct strobe_cb_info *info)
           value.format = vpiRealVal;
           vpi_get_value(item, &value);
 #if !defined(__GNUC__)
-		  if(compatible_flag)
+		  if (compatible_flag)
 			  sprintf(buf, "%g", value.value.real);
 		  else {
-			  if(value.value.real == 0.0 || value.value.real == -0.0)
+			  if (value.value.real == 0.0 || value.value.real == -0.0)
 				  sprintf(buf, "%.05f", value.value.real);
 			  else
 				  sprintf(buf, "%#g", value.value.real);
@@ -1205,7 +1234,7 @@ static PLI_INT32 sys_common_compiletf(ICARUS_VPI_CONST PLI_BYTE8*name, int no_au
       callh = vpi_handle(vpiSysTfCall, 0);
       argv = vpi_iterate(vpiArgument, callh);
 
-      if(name[1] == 'f') {
+      if (name[1] == 'f') {
 	      /* Check that there is a fd/mcd and that it is numeric. */
 	    if (argv == 0) {
 		  vpi_printf("ERROR: %s:%d: ", vpi_get_str(vpiFile, callh),
@@ -1238,7 +1267,8 @@ static PLI_INT32 sys_display_compiletf(ICARUS_VPI_CONST PLI_BYTE8*name)
       return sys_common_compiletf(name, 0, 0);
 }
 
-/* This implements the $display/$fdisplay and the $write/$fwrite based tasks. */
+/* This implements the $sformatf, $display/$fdisplay
+ * and the $write/$fwrite based tasks. */
 static PLI_INT32 sys_display_calltf(ICARUS_VPI_CONST PLI_BYTE8 *name)
 {
       vpiHandle callh, argv, scope;
@@ -1246,37 +1276,24 @@ static PLI_INT32 sys_display_calltf(ICARUS_VPI_CONST PLI_BYTE8 *name)
       char* result;
       unsigned int size;
       PLI_UINT32 fd_mcd;
+      s_vpi_value val;
 
       callh = vpi_handle(vpiSysTfCall, 0);
       argv = vpi_iterate(vpiArgument, callh);
 
 	/* Get the file/MC descriptor and verify it is valid. */
-      if(name[1] == 'f') {
-	      errno = 0;
-	      vpiHandle arg = vpi_scan(argv);
-	      s_vpi_value val;
-	      val.format = vpiIntVal;
-	      vpi_get_value(arg, &val);
-	      fd_mcd = val.value.integer;
-
-		/* If the MCD is zero we have nothing to do so just return. */
-	      if (fd_mcd == 0)  {
-		    vpi_free_object(argv);
-		    return 0;
-	      }
-
-	      if ((! IS_MCD(fd_mcd) && vpi_get_file(fd_mcd) == NULL) ||
-	          ( IS_MCD(fd_mcd) && my_mcd_printf(fd_mcd, "") == EOF)) {
-		    vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
-		               (int)vpi_get(vpiLineNo, callh));
-		    vpi_printf("invalid file descriptor/MCD (0x%x) given "
-		               "to %s.\n", (unsigned int)fd_mcd, name);
-		    errno = EBADF;
-		    vpi_free_object(argv);
-		    return 0;
-	      }
+      if (name[1] == 'f') {
+	    vpiHandle fd = vpi_scan(argv);
+	    if (get_fd_mcd_from_arg(&fd_mcd, fd, callh, name)) {
+		  vpi_free_object(argv);
+		  return 0;
+	    }
+      } else if (strncmp(name, "$sformatf", 9) == 0) {
+	      /* return as a string */
+	    fd_mcd = 0;
       } else {
-	      fd_mcd = 1;
+	      /* stdout */
+	    fd_mcd = 1;
       }
 
       scope = vpi_handle(vpiScope, callh);
@@ -1293,13 +1310,21 @@ static PLI_INT32 sys_display_calltf(ICARUS_VPI_CONST PLI_BYTE8 *name)
 	/* Because %u and %z may put embedded NULL characters into the
 	 * returned string strlen() may not match the real size! */
       result = get_display(&size, &info);
-      my_mcd_rawwrite(fd_mcd, result, size);
-      if ((strncmp(name,"$display",8) == 0) ||
-          (strncmp(name,"$fdisplay",9) == 0)) my_mcd_rawwrite(fd_mcd, "\n", 1);
 
+      if (fd_mcd > 0) {
+	     my_mcd_rawwrite(fd_mcd, result, size);
+	     if ((strncmp(name,"$display",8) == 0) ||
+	         (strncmp(name,"$fdisplay",9) == 0)) my_mcd_rawwrite(fd_mcd, "\n", 1);
+      } else {
+	       /* Return as a string ($sformatf) */
+	     val.format = vpiStringVal;
+	     val.value.str = result;
+	     vpi_put_value(callh, &val, 0, vpiNoDelay);
+      }
+
+      free(result);
       free(info.filename);
       free(info.items);
-      free(result);
       return 0;
 }
 
@@ -1317,8 +1342,7 @@ static PLI_INT32 strobe_cb(p_cb_data cb)
 	/* We really need to cancel any $fstrobe() calls for a file when it
 	 * is closed, but for now we will just skip processing the result.
 	 * Which has the same basic effect. */
-      if ((! IS_MCD(info->fd_mcd) && vpi_get_file(info->fd_mcd) != NULL) ||
-          ( IS_MCD(info->fd_mcd) && my_mcd_printf(info->fd_mcd, "") != EOF)) {
+      if (is_valid_fd_mcd(info->fd_mcd)) {
 	    char* result;
 	    unsigned int size;
 	      /* Because %u and %z may put embedded NULL characters into the
@@ -1355,32 +1379,15 @@ static PLI_INT32 sys_strobe_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       argv = vpi_iterate(vpiArgument, callh);
 
 	/* Get the file/MC descriptor and verify it is valid. */
-      if(name[1] == 'f') {
-	      errno = 0;
-	      vpiHandle arg = vpi_scan(argv);
-	      s_vpi_value val;
-	      val.format = vpiIntVal;
-	      vpi_get_value(arg, &val);
-	      fd_mcd = val.value.integer;
+      if (name[1] == 'f') {
+	    vpiHandle fd = vpi_scan(argv);
+	    if (get_fd_mcd_from_arg(&fd_mcd, fd, callh, name)) {
+		  vpi_free_object(argv);
+                  return 0;
+	    }
 
-		/* If the MCD is zero we have nothing to do so just return. */
-	      if (fd_mcd == 0)  {
-		    vpi_free_object(argv);
-		    return 0;
-	      }
-
-	      if ((! IS_MCD(fd_mcd) && vpi_get_file(fd_mcd) == NULL) ||
-	          ( IS_MCD(fd_mcd) && my_mcd_printf(fd_mcd, "") == EOF))  {
-		    vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
-		               (int)vpi_get(vpiLineNo, callh));
-		    vpi_printf("invalid file descriptor/MCD (0x%x) given "
-		               "to %s.\n", (unsigned int)fd_mcd, name);
-		    errno = EBADF;
-		    vpi_free_object(argv);
-		    return 0;
-	      }
       } else {
-	      fd_mcd = 1;
+	    fd_mcd = 1;
       }
 
       scope = vpi_handle(vpiScope, callh);
@@ -1408,6 +1415,7 @@ static PLI_INT32 sys_strobe_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       cb.value = 0;
       cb.user_data = (char*)info;
       vpi_register_cb(&cb);
+
       return 0;
 }
 
@@ -1665,7 +1673,7 @@ static PLI_INT32 sys_sformat_compiletf(ICARUS_VPI_CONST PLI_BYTE8 *name)
   if (argv == 0) {
     vpi_printf("ERROR:%s:%d: ", vpi_get_str(vpiFile, callh),
                (int)vpi_get(vpiLineNo, callh));
-    vpi_printf("%s requires at least two argument.\n", name);
+    vpi_printf("%s requires at least two arguments.\n", name);
     vpi_control(vpiFinish, 1);
     return 0;
   }
@@ -1686,7 +1694,7 @@ static PLI_INT32 sys_sformat_compiletf(ICARUS_VPI_CONST PLI_BYTE8 *name)
   if (arg == 0) {
     vpi_printf("ERROR:%s:%d: ", vpi_get_str(vpiFile, callh),
                (int)vpi_get(vpiLineNo, callh));
-    vpi_printf("%s requires at least two argument.\n", name);
+    vpi_printf("%s requires at least two arguments.\n", name);
     vpi_control(vpiFinish, 1);
     return 0;
   }
@@ -1754,6 +1762,46 @@ static PLI_INT32 sys_sformat_calltf(ICARUS_VPI_CONST PLI_BYTE8 *name)
   return 0;
 }
 
+static PLI_INT32 sys_sformatf_compiletf(ICARUS_VPI_CONST PLI_BYTE8 *name)
+{
+  vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+  vpiHandle argv = vpi_iterate(vpiArgument, callh);
+  vpiHandle arg;
+  PLI_INT32 type;
+
+  /* Check that there are arguments. */
+  if (argv == 0) {
+    vpi_printf("ERROR:%s:%d: ", vpi_get_str(vpiFile, callh),
+               (int)vpi_get(vpiLineNo, callh));
+    vpi_printf("%s requires at least one argument.\n", name);
+    vpi_control(vpiFinish, 1);
+    return 0;
+  }
+
+  /* The first argument must be a string, a register or a SV string. */
+  arg = vpi_scan(argv);
+  if (arg == 0) {
+    vpi_printf("ERROR:%s:%d: ", vpi_get_str(vpiFile, callh),
+               (int)vpi_get(vpiLineNo, callh));
+    vpi_printf("%s requires at least one argument.\n", name);
+    vpi_control(vpiFinish, 1);
+    return 0;
+  }
+  type = vpi_get(vpiType, arg);
+  if (((type != vpiConstant && type != vpiParameter) ||
+      vpi_get(vpiConstType, arg) != vpiStringConst) &&
+      type != vpiReg && type != vpiStringVar) {
+    vpi_printf("ERROR:%s:%d: ", vpi_get_str(vpiFile, callh),
+               (int)vpi_get(vpiLineNo, callh));
+    vpi_printf("%s's first argument must be a string or a register.\n", name);
+    vpi_control(vpiFinish, 1);
+    return 0;
+  }
+
+  if (sys_check_args(callh, argv, name, 0, 0)) vpi_control(vpiFinish, 1);
+  return 0;
+}
+
 static PLI_INT32 sys_end_of_compile(p_cb_data cb_data)
 {
       (void)cb_data; /* Parameter is not used. */
@@ -2077,6 +2125,9 @@ static PLI_INT32 sys_severity_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       free(dstr);
 
       if (strncmp(name,"$fatal",6) == 0) {
+	      /* Set the exit code from vvp as an error code. */
+	    vpip_set_return_value(1);
+	      /* Now tell the simulator to finish. */
             vpi_control(vpiFinish, finish_number.value.integer);
       }
 
@@ -2431,6 +2482,16 @@ void sys_display_register(void)
       res = vpi_register_systf(&tf_data);
       vpip_make_systf_system_defined(res);
 
+      tf_data.type      = vpiSysFunc;
+      tf_data.sysfunctype = vpiStringFunc;
+      tf_data.tfname    = "$sformatf";
+      tf_data.calltf    = sys_display_calltf;
+      tf_data.compiletf = sys_sformatf_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$sformatf";
+      res = vpi_register_systf(&tf_data);
+      vpip_make_systf_system_defined(res);
+
       /*============================ timeformat */
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$timeformat";
