@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2003-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -26,7 +26,6 @@
 # include  <stdlib.h>
 # include  "ivl_alloc.h"
 
-#define IS_MCD(mcd)     !((mcd)>>31&1)
 
 /*
  * Implement the $fopen system function.
@@ -86,11 +85,11 @@ static PLI_INT32 sys_fopen_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 
 	/* Get the mode handle if it exists. */
       if (mode) {
-            char *esc_md;
-            val.format = vpiStringVal;
-            vpi_get_value(mode, &val);
+	    char *esc_md;
+	    val.format = vpiStringVal;
+	    vpi_get_value(mode, &val);
 	      /* Verify that we have a string and that it is not NULL. */
-            if (val.format != vpiStringVal || !*(val.value.str)) {
+	    if (val.format != vpiStringVal || !*(val.value.str)) {
 		  vpi_printf("WARNING: %s:%d: ",
 		             vpi_get_str(vpiFile, callh),
 		             (int)vpi_get(vpiLineNo, callh));
@@ -146,7 +145,7 @@ static PLI_INT32 sys_fopen_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 		  }
 	    }
 
-            mode_string = strdup(val.value.str);
+	    mode_string = strdup(val.value.str);
 
 	    vpi_free_object(argv);
       }
@@ -212,32 +211,40 @@ static PLI_INT32 sys_fclose_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
       vpiHandle argv = vpi_iterate(vpiArgument, callh);
       vpiHandle fd = vpi_scan(argv);
-      s_vpi_value val;
       PLI_UINT32 fd_mcd;
-      errno = 0;
 
       vpi_free_object(argv);
 
 	/* Get the file/MC descriptor and verify that it is valid. */
-      val.format = vpiIntVal;
-      vpi_get_value(fd, &val);
-      fd_mcd = val.value.integer;
-
-      if ((! IS_MCD(fd_mcd) && vpi_get_file(fd_mcd) == NULL) ||
-          ( IS_MCD(fd_mcd) && vpi_mcd_printf(fd_mcd, "%s", "") == EOF) ||
-          (! fd_mcd)) {
-	    vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
-	               (int)vpi_get(vpiLineNo, callh));
-	    vpi_printf("invalid file descriptor/MCD (0x%x) given to %s.\n",
-	               (unsigned int)fd_mcd, name);
-	    errno = EBADF;
-	    return 0;
-      }
+      if (get_fd_mcd_from_arg(&fd_mcd, fd, callh, name)) return 0;
 
 	/* We need to cancel any active $fstrobe()'s for this FD/MCD.
 	 * For now we check in the strobe callback and skip the output
 	 * generation when needed. */
-      vpi_mcd_close(fd_mcd);
+      fd_mcd = vpi_mcd_close(fd_mcd);
+
+      if (fd_mcd) {
+	    char *fd_mcd_name;
+	    switch(fd_mcd) {
+		case 0x00000001: // MCD STDOUT
+		case 0x80000001: // FD STDOUT
+		  fd_mcd_name = "STDOUT ";
+		  break;
+		case 0x80000000: // FD STDIN
+		  fd_mcd_name = "STDIN ";
+		  break;
+		case 0x80000002: // FD STDERR
+		  fd_mcd_name = "STDERR ";
+		  break;
+		default:
+		  fd_mcd_name = "";
+	    }
+            vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
+                       (int)vpi_get(vpiLineNo, callh));
+            vpi_printf("could not close %s %s(0x%x) in %s().\n",
+	               IS_MCD(fd_mcd) ? "MCD" : "file descriptor",
+                       fd_mcd_name, (unsigned int)fd_mcd, name);
+      }
 
       return 0;
 }
@@ -249,10 +256,8 @@ static PLI_INT32 sys_fflush_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 {
       vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
       vpiHandle argv = vpi_iterate(vpiArgument, callh);
-      vpiHandle arg;
-      s_vpi_value val;
+      vpiHandle fd;
       PLI_UINT32 fd_mcd;
-      errno = 0;
 
 	/* If we have no argument then flush all the streams. */
       if (argv == 0) {
@@ -261,32 +266,14 @@ static PLI_INT32 sys_fflush_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
 	/* Get the file/MC descriptor and verify that it is valid. */
-      arg = vpi_scan(argv);
+      fd = vpi_scan(argv);
       vpi_free_object(argv);
-      val.format = vpiIntVal;
-      vpi_get_value(arg, &val);
-      fd_mcd = val.value.integer;
-
-	/* If the MCD is zero we have nothing to do so just return. */
-      if (fd_mcd == 0) return 0;
-
-      if ((! IS_MCD(fd_mcd) && vpi_get_file(fd_mcd) == NULL) ||
-          ( IS_MCD(fd_mcd) && vpi_mcd_printf(fd_mcd, "%s", "") == EOF) ||
-          (! fd_mcd)) {
-	    vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
-	               (int)vpi_get(vpiLineNo, callh));
-	    vpi_printf("invalid file descriptor/MCD (0x%x) given to %s.\n",
-	               (unsigned int)fd_mcd, name);
-	    errno = EBADF;
-	    return 0;
-      }
+      if (get_fd_mcd_from_arg(&fd_mcd, fd, callh, name)) return 0;
 
       if (IS_MCD(fd_mcd)) {
 	    vpi_mcd_flush(fd_mcd);
       } else {
-	      /* If we have a valid file descriptor flush the file. */
-	    FILE *fp = vpi_get_file(fd_mcd);
-	    if (fp) fflush(fp);
+	    fflush(vpi_get_file(fd_mcd));
       }
 
       return 0;
@@ -652,7 +639,7 @@ static PLI_INT32 sys_fread_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 			val.format = vpiIntVal;
 			vpi_get_value(arg, &val);
 			count = val.value.integer;
-			if (count > max-start) {
+			if (count > max-start+1) {
 			      vpi_printf("WARNING: %s:%d: ",
 			                 vpi_get_str(vpiFile, callh),
 			                 (int)vpi_get(vpiLineNo, callh));
@@ -670,11 +657,11 @@ static PLI_INT32 sys_fread_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 		  start = min;
 		  count = max - min + 1;
 	    }
-            width = vpi_get(vpiSize, vpi_handle_by_index(mem_reg, start));
+	    width = vpi_get(vpiSize, vpi_handle_by_index(mem_reg, start));
       } else {
 	    start = 0;
 	    count = 1;
-            width = vpi_get(vpiSize, mem_reg);
+	    width = vpi_get(vpiSize, mem_reg);
 	    vpi_free_object(argv);
       }
 
@@ -883,12 +870,11 @@ static PLI_INT32 sys_fseek_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 #if defined(__GNUC__)
       val.value.integer = fseek(fp, offset, oper);
 #else
-      if(oper < 0) {
-           val.value.integer = EOF;
-           errno = EINVAL;
-      }
-      else
-           val.value.integer = fseek(fp, offset, oper);
+      if (oper < 0) {
+	    val.value.integer = EOF;
+	    errno = EINVAL;
+      } else
+	    val.value.integer = fseek(fp, offset, oper);
 #endif
       vpi_put_value(callh, &val, 0 , vpiNoDelay);
 
