@@ -38,6 +38,8 @@
 static char *dump_path = NULL;
 static struct lt_trace *dump_file = NULL;
 
+static struct t_vpi_time zero_delay = { vpiSimTime, 0, 0, 0.0 };
+
 struct vcd_info {
       vpiHandle item;
       vpiHandle cb;
@@ -261,6 +263,7 @@ static PLI_INT32 variable_cb_1(p_cb_data cause)
 
       if (!vcd_dmp_list) {
           cb = *cause;
+	  cb.time = &zero_delay;
           cb.reason = cbReadOnlySynch;
           cb.cb_rtn = variable_cb_2;
           vpi_register_cb(&cb);
@@ -320,7 +323,6 @@ static PLI_INT32 finish_cb(p_cb_data cause)
 __inline__ static int install_dumpvars_callback(void)
 {
       struct t_cb_data cb;
-      static struct t_vpi_time now;
 
       if (dumpvars_status == 1) return 0;
 
@@ -331,8 +333,7 @@ __inline__ static int install_dumpvars_callback(void)
 	    return 1;
       }
 
-      now.type = vpiSimTime;
-      cb.time = &now;
+      cb.time = &zero_delay;
       cb.reason = cbReadOnlySynch;
       cb.cb_rtn = dumpvars_cb;
       cb.user_data = 0x0;
@@ -489,7 +490,7 @@ static PLI_INT32 sys_dumpfile_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
 	    return 0;
       }
 
-      path = get_filename(callh, name, vpi_scan(argv));
+      path = get_filename_with_suffix(callh, name, vpi_scan(argv), "lxt");
       vpi_free_object(argv);
       if (! path) return 0;
 
