@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2003-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -20,6 +20,7 @@
 #include "sys_priv.h"
 #include <assert.h>
 #include <ctype.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include "ivl_alloc.h"
@@ -103,6 +104,26 @@ char *get_filename(vpiHandle callh, const char *name, vpiHandle file)
       return strdup(val.value.str);
 }
 
+char* get_filename_with_suffix(vpiHandle callh, const char *name, vpiHandle file, const char*suff)
+{
+      char*path = get_filename(callh, name, file);
+      if (path == 0) return 0;
+
+	/* If the name already has a suffix, then don't replace it or
+	   add another suffix. Just return this path. */
+      char*tailp = strrchr(path, '.');
+      if (tailp != 0) return path;
+
+	/* The name doesn't have a suffix, so append the passed in
+	   suffix to the file name. */
+      char*new_path = malloc(strlen(path) + strlen(suff) + 2);
+      strcpy(new_path, path);
+      strcat(new_path, ".");
+      strcat(new_path, suff);
+      free(path);
+      return new_path;
+}
+
 void check_for_extra_args(vpiHandle argv, vpiHandle callh, const char *name,
                           const char *arg_str, unsigned opt)
 {
@@ -228,6 +249,56 @@ unsigned is_string_obj(vpiHandle obj)
 }
 
 
+/*
+ * Check if the file descriptor or MCD is valid.
+ * For a MCD check that every bit set is a valid file and
+ * for a FD make sure it exists.
+ */
+unsigned is_valid_fd_mcd(PLI_UINT32 fd_mcd)
+{
+      assert(fd_mcd); // Should already be handled!
+
+      if (IS_MCD(fd_mcd)){
+	    if (vpi_mcd_printf(fd_mcd, "%s", "") == EOF) return 0;
+      } else {
+	    if (vpi_get_file(fd_mcd) == NULL) return 0;
+      }
+
+      return 1;
+}
+
+/*
+ * Get a FD/MCD from the given argument and check if it is valid. Return the
+ * FD/MCD in the fd_mcd argument and return 0 if it is valid and 1 if it is
+ * invalid. We do not print a warning mesage if the FD/MCD is zero.
+ */
+unsigned get_fd_mcd_from_arg(PLI_UINT32 *fd_mcd, vpiHandle arg,
+                             vpiHandle callh, const char *name)
+{
+      s_vpi_value val;
+
+      val.format = vpiIntVal;
+      vpi_get_value(arg, &val);
+      *fd_mcd = val.value.integer;
+
+      if (*fd_mcd == 0) return 1;
+
+      errno = 0;
+      if (! is_valid_fd_mcd(*fd_mcd)) {
+            vpi_printf("WARNING: %s:%d: ", vpi_get_str(vpiFile, callh),
+                       (int)vpi_get(vpiLineNo, callh));
+            vpi_printf("invalid %s (0x%x) given to %s().\n",
+                       IS_MCD(*fd_mcd) ? "MCD" : "file descriptor",
+                       (unsigned int)*fd_mcd,
+                       name);
+            errno = EBADF;
+            return 1;
+      }
+
+      return 0;
+}
+
+
 /*
  * Find the enclosing module. If there is no enclosing module (which can be
  * the case in SystemVerilog), return the highest enclosing scope.
@@ -399,3 +470,23 @@ PLI_INT32 sys_one_string_arg_compiletf(ICARUS_VPI_CONST PLI_BYTE8 *name)
 
       return 0;
 }
+
+/* Return an integer value to the caller. */
+void put_integer_value(vpiHandle callh, PLI_INT32 result)
+{
+      s_vpi_value val;
+
+      val.format = vpiIntVal;
+      val.value.integer = result;
+      vpi_put_value(callh, &val, 0, vpiNoDelay);
+}
+
+/* Return a scalar value to the caller. */
+void put_scalar_value(vpiHandle callh, PLI_INT32 result)
+{
+      s_vpi_value val;
+
+      val.format = vpiScalarVal;
+      val.value.scalar = result;
+      vpi_put_value(callh, &val, 0, vpiNoDelay);
+}
