@@ -95,10 +95,11 @@ struct __vpiArrayVthrA : public __vpiHandle {
 
 
 struct __vpiArrayVthrAPV : public __vpiHandle {
-      int get_type_code(void) const { return vpiMemoryWord; }
+      int get_type_code(void) const { return vpiPartSelect; }
       int vpi_get(int code);
       char* vpi_get_str(int code);
       void vpi_get_value(p_vpi_value val);
+      vpiHandle vpi_handle(int code);
 
       struct __vpiArray*array;
       unsigned word_sel;
@@ -275,7 +276,7 @@ int __vpiArray::vpi_get(int code)
 	    return get_size();
 
 	  case vpiAutomatic:
-	    return (int) scope->is_automatic;
+	    return scope->is_automatic()? 1 : 0;
 
 	  default:
 	    return 0;
@@ -360,8 +361,23 @@ int __vpiArrayWord::as_word_t::vpi_get(int code)
             assert(val.format == vpiIntVal);
             return val.value.integer;
 
+	  case vpiIndex:
+	    {
+		  int base_offset = 0;
+		  struct __vpiArray*base = dynamic_cast<__vpiArray*> (parent);
+		  if (base) {
+			val.format = vpiIntVal;
+			base->first_addr.vpi_get_value(&val);
+			base_offset += val.value.integer;
+		  }
+		  val.format = vpiIntVal;
+		  obj->as_index.vpi_get_value(&val);
+		  assert(val.format == vpiIntVal);
+		  return val.value.integer + base_offset;
+	    }
+
 	  case vpiAutomatic:
-	    return (int) parent->get_scope()->is_automatic;
+	    return parent->get_scope()->is_automatic()? 1 : 0;
 
 #if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
 	  case _vpiFromThr:
@@ -393,7 +409,7 @@ int __vpiArrayVthrA::vpi_get(int code)
 	    return (int)get_address() + array->first_addr.get_value();
 
 	  case vpiAutomatic:
-	    return (int) array->get_scope()->is_automatic;
+	    return array->get_scope()->is_automatic() ? 1 : 0;
 
 #if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
 	  case _vpiFromThr:
@@ -495,16 +511,13 @@ int __vpiArrayVthrAPV::vpi_get(int code)
 	    return part_wid;
 
 	  case vpiLeftRange:
-	    return array->msb.get_value();
+	    return part_bit + part_wid - 1;
 
 	  case vpiRightRange:
-	    return array->lsb.get_value();
-
-	  case vpiIndex:
-	    return (int)word_sel;
+	    return part_bit;
 
 	  case vpiAutomatic:
-	    return (int) array->get_scope()->is_automatic;
+	    return array->get_scope()->is_automatic() ? 1 : 0;
 
 #if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
 	  case _vpiFromThr:
@@ -545,6 +558,28 @@ void __vpiArrayVthrAPV::vpi_get_value(p_vpi_value vp)
       }
 }
 
+vpiHandle __vpiArrayVthrAPV::vpi_handle(int code)
+{
+      switch (code) {
+            // Not currently implemented. We would need to create a VPI
+            // object for the memory word.
+	  case vpiParent:
+	    return 0;
+
+            // Not part of the Verilog standard. We use this internally.
+	  case vpiArray:
+	    return array;
+
+	  case vpiScope:
+	    return array->get_scope();
+
+	  case vpiModule:
+	    return vpip_module(array->get_scope());
+      }
+
+      return 0;
+}
+
 void __vpiArray::set_word(unsigned address, unsigned part_off, const vvp_vector4_t&val)
 {
       if (address >= get_size())
@@ -855,7 +890,7 @@ void compile_var_array(char*label, char*name, int last, int first,
 
 	/* Make the words. */
       arr->vals_width = labs(msb-lsb) + 1;
-      if (vpip_peek_current_scope()->is_automatic) {
+      if (vpip_peek_current_scope()->is_automatic()) {
             arr->vals4 = new vvp_vector4array_aa(arr->vals_width,
 						 arr->get_size());
       } else {
@@ -902,8 +937,7 @@ void compile_var2_array(char*label, char*name, int last, int first,
       } else if (lsb == 0 && msb == 63 && !signed_flag) {
 	    arr->vals = new vvp_darray_atom<uint64_t>(arr->get_size());
       } else {
-	      // For now, only support the atom sizes.
-	    assert(0);
+	    arr->vals = new vvp_darray_vec2(arr->get_size(), arr->vals_width);
       }
       count_var_arrays += 1;
       count_var_array_words += arr->get_size();
@@ -1100,7 +1134,7 @@ class vvp_fun_arrayport_aa  : public vvp_fun_arrayport, public automatic_hooks_s
     private:
       void check_word_change_(unsigned long addr, vvp_context_t context);
 
-      struct __vpiScope*context_scope_;
+      __vpiScope*context_scope_;
       unsigned context_idx_;
 };
 
@@ -1201,7 +1235,7 @@ void vvp_fun_arrayport_aa::check_word_change_(unsigned long addr,
 
 void vvp_fun_arrayport_aa::check_word_change(unsigned long addr)
 {
-      if (arr_->get_scope()->is_automatic) {
+      if (arr_->get_scope()->is_automatic()) {
             assert(vthread_get_wt_context());
             check_word_change_(addr, vthread_get_wt_context());
       } else {
@@ -1218,7 +1252,7 @@ static void array_attach_port(vvp_array_t array, vvp_fun_arrayport*fun)
       assert(fun->next_ == 0);
       fun->next_ = array->ports_;
       array->ports_ = fun;
-      if (!array->get_scope()->is_automatic) {
+      if (!array->get_scope()->is_automatic()) {
               /* propagate initial values for variable arrays */
             if (array->vals4) {
                   vvp_vector4_t tmp(array->vals_width, BIT4_X);
@@ -1355,12 +1389,12 @@ bool array_port_resolv_list_t::resolve(bool mes)
 
       vvp_fun_arrayport*fun;
       if (use_addr)
-            if (vpip_peek_current_scope()->is_automatic)
+            if (vpip_peek_current_scope()->is_automatic())
                   fun = new vvp_fun_arrayport_aa(mem, ptr, addr);
             else
                   fun = new vvp_fun_arrayport_sa(mem, ptr, addr);
       else
-            if (vpip_peek_current_scope()->is_automatic)
+            if (vpip_peek_current_scope()->is_automatic())
                   fun = new vvp_fun_arrayport_aa(mem, ptr);
             else
                   fun = new vvp_fun_arrayport_sa(mem, ptr);
