@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -65,6 +65,7 @@ enum operand_e {
       OA_BIT2,
 	/* The operand is a pointer to code space */
       OA_CODE_PTR,
+      OA_CODE_PTR2,
 	/* The operand is a variable or net pointer */
       OA_FUNC_PTR,
 	/* The operand is a second functor pointer */
@@ -107,31 +108,39 @@ static const struct opcode_table_s opcode_table[] = {
       { "%blend",    of_BLEND,   0,  {OA_NONE,  OA_NONE,     OA_NONE} },
       { "%blend/wr", of_BLEND_WR,0,  {OA_NONE,  OA_NONE,     OA_NONE} },
       { "%breakpoint", of_BREAKPOINT, 0,  {OA_NONE, OA_NONE, OA_NONE} },
+      { "%callf/obj",       of_CALLF_OBJ,       2,{OA_CODE_PTR2,OA_VPI_PTR, OA_NONE} },
+      { "%callf/real",      of_CALLF_REAL,      2,{OA_CODE_PTR2,OA_VPI_PTR, OA_NONE} },
+      { "%callf/str",       of_CALLF_STR,       2,{OA_CODE_PTR2,OA_VPI_PTR, OA_NONE} },
+      { "%callf/vec4",      of_CALLF_VEC4,      2,{OA_CODE_PTR2,OA_VPI_PTR, OA_NONE} },
+      { "%callf/void",      of_CALLF_VOID,      2,{OA_CODE_PTR2,OA_VPI_PTR, OA_NONE} },
       { "%cassign/link",    of_CASSIGN_LINK,    2,{OA_FUNC_PTR,OA_FUNC_PTR2,OA_NONE} },
       { "%cassign/vec4",    of_CASSIGN_VEC4,    1,{OA_FUNC_PTR,OA_NONE,     OA_NONE} },
       { "%cassign/vec4/off",of_CASSIGN_VEC4_OFF,2,{OA_FUNC_PTR,OA_BIT1,     OA_NONE} },
       { "%cassign/wr",  of_CASSIGN_WR,  1,{OA_FUNC_PTR,OA_NONE,     OA_NONE} },
-      { "%cast2",  of_CAST2,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/e",  of_CMPE,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/ne", of_CMPNE,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/s",  of_CMPS,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/str",of_CMPSTR, 0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/u",  of_CMPU,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/wr", of_CMPWR,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/ws", of_CMPWS,  2,  {OA_BIT1,     OA_BIT2,     OA_NONE} },
-      { "%cmp/wu", of_CMPWU,  2,  {OA_BIT1,     OA_BIT2,     OA_NONE} },
-      { "%cmp/x",  of_CMPX,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmp/z",  of_CMPZ,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
-      { "%cmpi/e", of_CMPIE,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
-      { "%cmpi/ne",of_CMPINE, 3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
-      { "%cmpi/s", of_CMPIS,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
-      { "%cmpi/u", of_CMPIU,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
+      { "%cast/vec2/dar", of_CAST_VEC2_DAR, 1,  {OA_NUMBER,   OA_NONE,     OA_NONE} },
+      { "%cast/vec4/dar", of_CAST_VEC4_DAR, 1,  {OA_NUMBER,   OA_NONE,     OA_NONE} },
+      { "%cast/vec4/str", of_CAST_VEC4_STR, 1,  {OA_NUMBER,   OA_NONE,     OA_NONE} },
+      { "%cast2",   of_CAST2,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/e",   of_CMPE,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/ne",  of_CMPNE,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/s",   of_CMPS,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/str", of_CMPSTR, 0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/u",   of_CMPU,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/we",  of_CMPWE,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/wne", of_CMPWNE, 0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/wr",  of_CMPWR,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/ws",  of_CMPWS,  2,  {OA_BIT1,     OA_BIT2,     OA_NONE} },
+      { "%cmp/wu",  of_CMPWU,  2,  {OA_BIT1,     OA_BIT2,     OA_NONE} },
+      { "%cmp/x",   of_CMPX,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmp/z",   of_CMPZ,   0,  {OA_NONE,     OA_NONE,     OA_NONE} },
+      { "%cmpi/e",  of_CMPIE,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
+      { "%cmpi/ne", of_CMPINE, 3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
+      { "%cmpi/s",  of_CMPIS,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
+      { "%cmpi/u",  of_CMPIU,  3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
       { "%concat/str",  of_CONCAT_STR,  0,{OA_NONE,  OA_NONE,  OA_NONE} },
       { "%concat/vec4", of_CONCAT_VEC4, 0,{OA_NONE,  OA_NONE,  OA_NONE} },
       { "%concati/str", of_CONCATI_STR, 1,{OA_STRING,OA_NONE,  OA_NONE} },
       { "%concati/vec4",of_CONCATI_VEC4,3,{OA_BIT1,  OA_BIT2,  OA_NUMBER} },
-      { "%cvt/rs", of_CVT_RS, 1,  {OA_BIT1,     OA_NONE,     OA_NONE} },
-      { "%cvt/ru", of_CVT_RU, 1,  {OA_BIT1,     OA_NONE,     OA_NONE} },
       { "%cvt/rv",   of_CVT_RV,  0, {OA_NONE,   OA_NONE,     OA_NONE} },
       { "%cvt/rv/s", of_CVT_RV_S,0, {OA_NONE,   OA_NONE,     OA_NONE} },
       { "%cvt/sr", of_CVT_SR, 1,  {OA_BIT1,     OA_NONE,     OA_NONE} },
@@ -142,7 +151,10 @@ static const struct opcode_table_s opcode_table[] = {
       { "%debug/thr",  of_DEBUG_THR,  1,{OA_STRING,   OA_NONE,     OA_NONE} },
       { "%delay",  of_DELAY,  2,  {OA_BIT1,     OA_BIT2,     OA_NONE} },
       { "%delayx", of_DELAYX, 1,  {OA_NUMBER,   OA_NONE,     OA_NONE} },
+      { "%delete/elem",of_DELETE_ELEM,1,{OA_FUNC_PTR,OA_NONE,OA_NONE} },
       { "%delete/obj",of_DELETE_OBJ,1,{OA_FUNC_PTR,OA_NONE,  OA_NONE} },
+      { "%delete/tail",of_DELETE_TAIL,2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
+      { "%disable",  of_DISABLE, 1, {OA_VPI_PTR,OA_NONE,     OA_NONE} },
       { "%disable/fork",of_DISABLE_FORK,0,{OA_NONE,OA_NONE,  OA_NONE} },
       { "%div",      of_DIV,     0, {OA_NONE,   OA_NONE,     OA_NONE} },
       { "%div/s",    of_DIV_S,   0, {OA_NONE,   OA_NONE,     OA_NONE} },
@@ -164,7 +176,9 @@ static const struct opcode_table_s opcode_table[] = {
       { "%force/link",    of_FORCE_LINK,2,{OA_FUNC_PTR, OA_FUNC_PTR2, OA_NONE} },
       { "%force/vec4",    of_FORCE_VEC4,    1,{OA_FUNC_PTR, OA_NONE,      OA_NONE} },
       { "%force/vec4/off",of_FORCE_VEC4_OFF,2,{OA_FUNC_PTR, OA_BIT1,      OA_NONE} },
+      { "%force/vec4/off/d",of_FORCE_VEC4_OFF_D,3,{OA_FUNC_PTR, OA_BIT1,  OA_BIT2} },
       { "%force/wr",      of_FORCE_WR,      1,{OA_FUNC_PTR, OA_NONE,      OA_NONE} },
+      { "%fork",   of_FORK,   2,  {OA_CODE_PTR2,OA_VPI_PTR,  OA_NONE} },
       { "%free",   of_FREE,   1,  {OA_VPI_PTR,  OA_NONE,     OA_NONE} },
       { "%inv",    of_INV,    0,  {OA_NONE,     OA_NONE,     OA_NONE} },
       { "%ix/add", of_IX_ADD, 3,  {OA_NUMBER,   OA_BIT1,     OA_BIT2} },
@@ -235,14 +249,25 @@ static const struct opcode_table_s opcode_table[] = {
       { "%pushi/vec4",of_PUSHI_VEC4,3,{OA_BIT1,   OA_BIT2,   OA_NUMBER} },
       { "%pushv/str", of_PUSHV_STR, 0,{OA_NONE,   OA_NONE,   OA_NONE} },
       { "%putc/str/vec4",of_PUTC_STR_VEC4,2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
-      { "%qpop/b/str",of_QPOP_B_STR,1,{OA_FUNC_PTR,OA_NONE,  OA_NONE} },
-      { "%qpop/b/v",  of_QPOP_B_V,  1,{OA_FUNC_PTR,OA_NONE,  OA_BIT2} },
-      { "%qpop/f/str",of_QPOP_F_STR,1,{OA_FUNC_PTR,OA_NONE,  OA_NONE} },
-      { "%qpop/f/v",  of_QPOP_F_V,  1,{OA_FUNC_PTR,OA_NONE,  OA_BIT2} },
+      { "%qinsert/real",of_QINSERT_REAL,2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
+      { "%qinsert/str", of_QINSERT_STR, 2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
+      { "%qinsert/v",   of_QINSERT_V,   3,{OA_FUNC_PTR,OA_BIT1,OA_BIT2} },
+      { "%qpop/b/real",of_QPOP_B_REAL,1,{OA_FUNC_PTR,OA_NONE,OA_NONE} },
+      { "%qpop/b/str", of_QPOP_B_STR, 1,{OA_FUNC_PTR,OA_NONE,OA_NONE} },
+      { "%qpop/b/v",   of_QPOP_B_V,   2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
+      { "%qpop/f/real",of_QPOP_F_REAL,1,{OA_FUNC_PTR,OA_NONE,OA_NONE} },
+      { "%qpop/f/str", of_QPOP_F_STR, 1,{OA_FUNC_PTR,OA_NONE,OA_NONE} },
+      { "%qpop/f/v",   of_QPOP_F_V,   2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
       { "%release/net",of_RELEASE_NET,3,{OA_FUNC_PTR,OA_BIT1,OA_BIT2} },
       { "%release/reg",of_RELEASE_REG,3,{OA_FUNC_PTR,OA_BIT1,OA_BIT2} },
       { "%release/wr", of_RELEASE_WR, 2,{OA_FUNC_PTR,OA_BIT1,OA_NONE} },
       { "%replicate", of_REPLICATE,   1,{OA_NUMBER,  OA_NONE,OA_NONE} },
+      { "%ret/real",  of_RET_REAL,    1,{OA_NUMBER,  OA_NONE,OA_NONE} },
+      { "%ret/str",   of_RET_STR,     1,{OA_NUMBER,  OA_NONE,OA_NONE} },
+      { "%ret/vec4",  of_RET_VEC4,    3,{OA_NUMBER,  OA_BIT1,OA_BIT2} },
+      { "%retload/real",of_RETLOAD_REAL,1,{OA_NUMBER,  OA_NONE,OA_NONE} },
+      { "%retload/str", of_RETLOAD_STR, 1,{OA_NUMBER,  OA_NONE,OA_NONE} },
+      { "%retload/vec4",of_RETLOAD_VEC4,1,{OA_NUMBER,  OA_NONE,OA_NONE} },
       { "%scopy",  of_SCOPY,  0,  {OA_NONE,     OA_NONE,     OA_NONE} },
       { "%set/dar/obj/real",of_SET_DAR_OBJ_REAL,1,{OA_NUMBER,OA_NONE,OA_NONE} },
       { "%set/dar/obj/str", of_SET_DAR_OBJ_STR, 1,{OA_NUMBER,OA_NONE,OA_NONE} },
@@ -260,12 +285,18 @@ static const struct opcode_table_s opcode_table[] = {
       { "%store/prop/r",  of_STORE_PROP_R,  1, {OA_NUMBER,  OA_NONE, OA_NONE} },
       { "%store/prop/str",of_STORE_PROP_STR,1, {OA_NUMBER,  OA_NONE, OA_NONE} },
       { "%store/prop/v",  of_STORE_PROP_V,  2, {OA_NUMBER,  OA_BIT1, OA_NONE} },
-      { "%store/qb/r",   of_STORE_QB_R,    1, {OA_FUNC_PTR, OA_NONE, OA_NONE} },
-      { "%store/qb/str", of_STORE_QB_STR,  1, {OA_FUNC_PTR, OA_NONE, OA_NONE} },
-      { "%store/qb/v",   of_STORE_QB_V,    2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
-      { "%store/qf/r",   of_STORE_QF_R,    1, {OA_FUNC_PTR, OA_NONE, OA_NONE} },
-      { "%store/qf/str", of_STORE_QF_STR,  1, {OA_FUNC_PTR, OA_NONE, OA_NONE} },
-      { "%store/qf/v",   of_STORE_QF_V,    2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qb/r",   of_STORE_QB_R,    2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qb/str", of_STORE_QB_STR,  2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qb/v",   of_STORE_QB_V,    3, {OA_FUNC_PTR, OA_BIT1, OA_BIT2} },
+      { "%store/qdar/r",  of_STORE_QDAR_R,  2,{OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qdar/str",of_STORE_QDAR_STR,2,{OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qdar/v",  of_STORE_QDAR_V,  3,{OA_FUNC_PTR, OA_BIT1, OA_BIT2} },
+      { "%store/qf/r",   of_STORE_QF_R,    2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qf/str", of_STORE_QF_STR,  2, {OA_FUNC_PTR, OA_BIT1, OA_NONE} },
+      { "%store/qf/v",   of_STORE_QF_V,    3, {OA_FUNC_PTR, OA_BIT1, OA_BIT2} },
+      { "%store/qobj/r",  of_STORE_QOBJ_R,  2, {OA_FUNC_PTR,OA_BIT1, OA_NONE} },
+      { "%store/qobj/str",of_STORE_QOBJ_STR,2, {OA_FUNC_PTR,OA_BIT1, OA_NONE} },
+      { "%store/qobj/v",  of_STORE_QOBJ_V,  3, {OA_FUNC_PTR,OA_BIT1, OA_BIT2} },
       { "%store/real",    of_STORE_REAL,    1, {OA_FUNC_PTR,OA_NONE, OA_NONE} },
       { "%store/reala",   of_STORE_REALA,   2, {OA_ARR_PTR, OA_BIT1, OA_NONE} },
       { "%store/str",     of_STORE_STR,     1, {OA_FUNC_PTR,OA_NONE, OA_NONE} },
@@ -360,6 +391,8 @@ vpiHandle vvp_lookup_handle(const char*label)
 
 vvp_net_t* vvp_net_lookup(const char*label)
 {
+      static bool t0_trigger_generated = false;
+
         /* First, look to see if the symbol is a vpi object of some
 	   sort. If it is, then get the vvp_ipoint_t pointer out of
 	   the vpiHandle. */
@@ -407,6 +440,23 @@ vvp_net_t* vvp_net_lookup(const char*label)
 	/* Failing that, look for a general functor. */
       vvp_net_t*tmp = lookup_functor_symbol(label);
 
+	// This is a special label used to create a T0 trigger for the
+	// always_comb/latch processes.
+     if (! t0_trigger_generated && (strcmp(label, "E_0x0") == 0)) {
+	      // This should never happen, but if it does then the E_0x0
+	      // event generation may need to be explicitly generated in
+	      // the compiler output instead of implicitly in this code.
+	    assert(! vpip_peek_current_scope()->is_automatic());
+	    t0_trigger_generated = true;
+	      // Create a local event with no name for the T0 trigger
+	    compile_named_event(strdup(label), strcpy(new char [1],""), true);
+	    tmp = vvp_net_lookup(label);
+	    assert(tmp);
+	      // Create a trigger for the T0 event.
+	    vvp_net_ptr_t ptr (tmp, 0);
+	    schedule_t0_trigger(ptr);
+     }
+
       return tmp;
 }
 
@@ -455,7 +505,7 @@ void resolv_submit(resolv_list_s*cur)
  */
 struct vvp_net_resolv_list_s: public resolv_list_s {
 
-      vvp_net_resolv_list_s(char*l) : resolv_list_s(l) { }
+      explicit vvp_net_resolv_list_s(char*l) : resolv_list_s(l) { }
 	// port to be driven by the located node.
       vvp_net_ptr_t port;
       virtual bool resolve(bool mes);
@@ -629,10 +679,12 @@ void compile_vpi_lookup(vpiHandle *handle, char*label)
  */
 
 struct code_label_resolv_list_s: public resolv_list_s {
-      code_label_resolv_list_s(char*lab) : resolv_list_s(lab) {
+      explicit code_label_resolv_list_s(char*lab, bool cptr2) : resolv_list_s(lab) {
 	    code = NULL;
+	    cptr2_flag = cptr2;
       }
       struct vvp_code_s *code;
+      bool cptr2_flag;
       virtual bool resolve(bool mes);
 };
 
@@ -640,7 +692,7 @@ bool code_label_resolv_list_s::resolve(bool mes)
 {
       symbol_value_t val = sym_get_value(sym_codespace, label());
       if (val.ptr) {
-	    if (code->opcode == of_FORK)
+	    if (cptr2_flag)
 		  code->cptr2 = reinterpret_cast<vvp_code_t>(val.ptr);
 	    else
 		  code->cptr = reinterpret_cast<vvp_code_t>(val.ptr);
@@ -653,10 +705,10 @@ bool code_label_resolv_list_s::resolve(bool mes)
       return false;
 }
 
-void code_label_lookup(struct vvp_code_s *code, char *label)
+void code_label_lookup(struct vvp_code_s *code, char *label, bool cptr2)
 {
       struct code_label_resolv_list_s *res
-	    = new struct code_label_resolv_list_s(label);
+	    = new struct code_label_resolv_list_s(label, cptr2);
 
       res->code  = code;
 
@@ -664,7 +716,7 @@ void code_label_lookup(struct vvp_code_s *code, char *label)
 }
 
 struct code_array_resolv_list_s: public resolv_list_s {
-      code_array_resolv_list_s(char*lab) : resolv_list_s(lab) {
+      explicit code_array_resolv_list_s(char*lab) : resolv_list_s(lab) {
 	    code = NULL;
       }
       struct vvp_code_s *code;
@@ -1387,6 +1439,38 @@ void compile_cmp_gt_r(char*label, unsigned argc, struct symb_s*argv)
       make_arith(arith, label, argc, argv);
 }
 
+void compile_cmp_weq(char*label, long wid,
+		     unsigned argc, struct symb_s*argv)
+{
+      assert( wid > 0 );
+
+      if (argc != 2) {
+	    fprintf(stderr, "%s .cmp/weq has wrong number of symbols\n",label);
+	    compile_errors += 1;
+	    return;
+      }
+
+      vvp_arith_ *arith = new vvp_cmp_weq(wid);
+
+      make_arith(arith, label, argc, argv);
+}
+
+void compile_cmp_wne(char*label, long wid,
+		     unsigned argc, struct symb_s*argv)
+{
+      assert( wid > 0 );
+
+      if (argc != 2) {
+	    fprintf(stderr, "%s .cmp/wne has wrong number of symbols\n",label);
+	    compile_errors += 1;
+	    return;
+      }
+
+      vvp_arith_ *arith = new vvp_cmp_wne(wid);
+
+      make_arith(arith, label, argc, argv);
+}
+
 
 void compile_delay(char*label, unsigned width,
                    vvp_delay_t*delay, struct symb_s arg)
@@ -1459,8 +1543,8 @@ struct __vpiModPath* compile_modpath(char*label, unsigned width,
 
 static struct __vpiModPathSrc*make_modpath_src(struct __vpiModPath*path,
 					       char edge,
-					       struct symb_s src,
-					       struct numbv_s vals,
+					       const struct symb_s&src,
+					       struct numbv_s&vals,
 					       bool ifnone)
 {
       vvp_fun_modpath*dst = path->modpath;
@@ -1522,10 +1606,10 @@ static struct __vpiModPathSrc*make_modpath_src(struct __vpiModPath*path,
 }
 
 void compile_modpath_src(struct __vpiModPath*dst, char edge,
-			 struct symb_s src,
-			 struct numbv_s vals,
-			 struct symb_s condit_src,
-			 struct symb_s path_term_in)
+			 const struct symb_s&src,
+			 struct numbv_s&vals,
+			 const struct symb_s&condit_src,
+			 const struct symb_s&path_term_in)
 {
       struct __vpiModPathSrc*obj =
 	    make_modpath_src(dst, edge, src, vals, false);
@@ -1534,10 +1618,10 @@ void compile_modpath_src(struct __vpiModPath*dst, char edge,
 }
 
 void compile_modpath_src(struct __vpiModPath*dst, char edge,
-			 struct symb_s src,
-			 struct numbv_s vals,
+			 const struct symb_s&src,
+			 struct numbv_s&vals,
 			 int condit_src,
-			 struct symb_s path_term_in,
+			 const struct symb_s&path_term_in,
 			 bool ifnone)
 {
       assert(condit_src == 0);
@@ -1726,13 +1810,15 @@ void compile_code(char*label, char*mnem, comp_operands_t opa)
 		  break;
 
 		case OA_CODE_PTR:
+		case OA_CODE_PTR2:
 		  if (opa->argv[idx].ltype != L_SYMB) {
 			yyerror("operand format");
 			break;
 		  }
 
 		  assert(opa->argv[idx].symb.idx == 0);
-		  code_label_lookup(code, opa->argv[idx].symb.text);
+		  code_label_lookup(code, opa->argv[idx].symb.text,
+				    op->argt[idx]==OA_CODE_PTR2);
 		  break;
 
 		case OA_FUNC_PTR:
@@ -1807,42 +1893,6 @@ void compile_codelabel(char*label)
       free(label);
 }
 
-
-void compile_disable(char*label, struct symb_s symb)
-{
-      if (label)
-	    compile_codelabel(label);
-
-	/* Fill in the basics of the %disable in the instruction. */
-      vvp_code_t code = codespace_allocate();
-      code->opcode = of_DISABLE;
-
-      compile_vpi_lookup(&code->handle, symb.text);
-}
-
-/*
- * The %fork instruction is a little different from other instructions
- * in that it has an extended field that holds the information needed
- * to create the new thread. This includes the target PC and scope.
- * I get these from the parser in the form of symbols.
- */
-void compile_fork(char*label, struct symb_s dest, struct symb_s scope)
-{
-      if (label)
-	    compile_codelabel(label);
-
-
-	/* Fill in the basics of the %fork in the instruction. */
-      vvp_code_t code = codespace_allocate();
-      code->opcode = of_FORK;
-
-	/* Figure out the target PC. */
-      code_label_lookup(code, dest.text);
-
-	/* Figure out the target SCOPE. */
-      compile_vpi_lookup(&code->handle, scope.text);
-}
-
 void compile_file_line(char*label, long file_idx, long lineno,
                        char*description)
 {
