@@ -320,7 +320,9 @@ void sync_cb::run_run()
       if (cur->cb_data.cb_rtn != 0) {
 	    assert(vpi_mode_flag == VPI_MODE_NONE);
 	    vpi_mode_flag = sync_flag? VPI_MODE_ROSYNC : VPI_MODE_RWSYNC;
+	    vpip_cur_task = dynamic_cast<__vpiSysTaskCall*>(cur->cb_data.obj);
 	    (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    vpip_cur_task = 0;
 	    vpi_mode_flag = VPI_MODE_NONE;
       }
 
@@ -336,21 +338,14 @@ static sync_callback* make_sync(p_cb_data data, bool readonly_flag)
       cb->handle = obj;
       obj->cb_sync = cb;
 
+      vvp_time64_t tv = 0;
       switch (obj->cb_time.type) {
 	  case vpiSuppressTime:
-	    schedule_generic(cb, 0, true, readonly_flag);
 	    break;
 
 	  case vpiSimTime:
-	      { vvp_time64_t tv = vpip_timestruct_to_time(&obj->cb_time);
-		vvp_time64_t tn = schedule_simtime();
-		if (tv < tn) {
-		      schedule_generic(cb, 0, true, readonly_flag);
-		} else {
-		      schedule_generic(cb, tv - tn, true, readonly_flag);
-		}
-		break;
-	      }
+	    tv = vpip_timestruct_to_time(&obj->cb_time);
+	    break;
 
 	  default:
 	    fprintf(stderr, "Unsupported time type %d.\n",
@@ -358,10 +353,11 @@ static sync_callback* make_sync(p_cb_data data, bool readonly_flag)
 	    assert(0);
 	    break;
       }
+      schedule_generic(cb, tv, true, readonly_flag);
       return obj;
 }
 
-static struct __vpiCallback* make_afterdelay(p_cb_data data, bool simtime_flag)
+static struct __vpiCallback* make_afterdelay(p_cb_data data)
 {
       sync_callback*obj = new sync_callback(data);
       struct sync_cb*cb = new sync_cb;
@@ -369,7 +365,7 @@ static struct __vpiCallback* make_afterdelay(p_cb_data data, bool simtime_flag)
       cb->handle = obj;
       obj->cb_sync = cb;
 
-      vvp_time64_t tv;
+      vvp_time64_t tv = 0;
       switch (obj->cb_time.type) {
 	  case vpiSimTime:
 	    tv = vpip_timestruct_to_time(&obj->cb_time);
@@ -379,25 +375,81 @@ static struct __vpiCallback* make_afterdelay(p_cb_data data, bool simtime_flag)
 	    fprintf(stderr, "Unsupported time type %d.\n",
 	            (int)obj->cb_time.type);
 	    assert(0);
-	    tv = 0;
 	    break;
       }
 
-      if (simtime_flag) {
-	    vvp_time64_t cur = schedule_simtime();
-	    if (cur > tv) {
-		  tv = 0;
-		  assert(0);
-	    } else if (cur == tv) {
-		  tv = 0;
-	    } else {
-		  tv -= cur;
-	    }
-	    schedule_at_start_of_simtime(cb, tv);
-      } else {
-	    schedule_generic(cb, tv, false);
+      schedule_generic(cb, tv, false);
+
+      return obj;
+}
+
+static struct __vpiCallback* make_at_start_of_sim_time(p_cb_data data)
+{
+      sync_callback*obj = new sync_callback(data);
+      struct sync_cb*cb = new sync_cb;
+      cb->sync_flag = false;
+      cb->handle = obj;
+      obj->cb_sync = cb;
+
+      vvp_time64_t tv = 0;
+      switch (obj->cb_time.type) {
+	  case vpiSimTime:
+	    tv = vpip_timestruct_to_time(&obj->cb_time);
+	    break;
+
+	  default:
+	    fprintf(stderr, "Unsupported time type %d.\n",
+	            (int)obj->cb_time.type);
+	    assert(0);
+	    break;
       }
 
+      vvp_time64_t cur = schedule_simtime();
+      if (cur > tv) {
+	    tv = 0;
+	    assert(0);
+      } else if (cur == tv) {
+	    tv = 0;
+      } else {
+	    tv -= cur;
+      }
+      schedule_at_start_of_simtime(cb, tv);
+
+      return obj;
+}
+
+static struct __vpiCallback* make_at_end_of_sim_time(p_cb_data data)
+{
+      sync_callback*obj = new sync_callback(data);
+      struct sync_cb*cb = new sync_cb;
+      cb->sync_flag = false;
+      cb->handle = obj;
+      obj->cb_sync = cb;
+
+      vvp_time64_t tv = 0;
+      switch (obj->cb_time.type) {
+	  case vpiSimTime:
+	    tv = vpip_timestruct_to_time(&obj->cb_time);
+	    break;
+
+	  default:
+	    fprintf(stderr, "Unsupported time type %d.\n",
+	            (int)obj->cb_time.type);
+	    assert(0);
+	    break;
+      }
+
+      vvp_time64_t cur = schedule_simtime();
+      if (cur > tv) {
+	    tv = 0;
+	    assert(0);
+      } else if (cur == tv) {
+	    tv = 0;
+      } else {
+	    tv -= cur;
+      }
+      schedule_at_end_of_simtime(cb, tv);
+
       return obj;
 }
 
@@ -578,11 +630,15 @@ vpiHandle vpi_register_cb(p_cb_data data)
 	    break;
 
 	  case cbAtStartOfSimTime:
-	    obj = make_afterdelay(data, true);
+	    obj = make_at_start_of_sim_time(data);
+	    break;
+
+	  case cbAtEndOfSimTime:
+	    obj = make_at_end_of_sim_time(data);
 	    break;
 
 	  case cbAfterDelay:
-	    obj = make_afterdelay(data, false);
+	    obj = make_afterdelay(data);
 	    break;
 
 	  case cbEndOfCompile:
@@ -629,8 +685,7 @@ void callback_execute(struct __vpiCallback*cur)
 	  case vpiScaledRealTime: {
 	    cur->cb_data.time->real =
 	         vpip_time_to_scaled_real(schedule_simtime(),
-	             (struct __vpiScope *) vpi_handle(vpiScope,
-	                                              cur->cb_data.obj));
+	             (__vpiScope *) vpi_handle(vpiScope, cur->cb_data.obj));
 	    break;
 	  }
 	  case vpiSuppressTime:
