@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -108,23 +108,23 @@ unsigned vpip_size(__vpiSignal *sig)
       return abs(sig->msb.get_value() - sig->lsb.get_value()) + 1;
 }
 
-struct __vpiScope* vpip_scope(__vpiSignal*sig)
+__vpiScope* vpip_scope(__vpiSignal*sig)
 {
       if (sig->is_netarray)
-	    return  (struct __vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+	    return  (__vpiScope*) vpi_handle(vpiScope, sig->within.parent);
       else
 	    return sig->within.scope;
 }
 
-struct __vpiScope* vpip_scope(__vpiRealVar*sig)
+__vpiScope* vpip_scope(__vpiRealVar*sig)
 {
       if (sig->is_netarray)
-	    return  (struct __vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+	    return  (__vpiScope*) vpi_handle(vpiScope, sig->within.parent);
       else
 	    return sig->within.scope;
 }
 
-vpiHandle vpip_module(struct __vpiScope*scope)
+vpiHandle vpip_module(__vpiScope*scope)
 {
       while(scope && scope->get_type_code() != vpiModule) {
 	    scope = scope->scope;
@@ -355,6 +355,8 @@ static const char* vpi_type_values(PLI_INT32 code)
 	    return "vpiNet";
 	  case vpiNetArray:
 	    return "vpiNetArray";
+	  case vpiNetBit:
+	    return "vpiNetBit";
 	  case vpiParameter:
 	    return "vpiParameter";
 	  case vpiPartSelect:
@@ -363,6 +365,8 @@ static const char* vpi_type_values(PLI_INT32 code)
 	    return "vpiRealVar";
 	  case vpiReg:
 	    return "vpiReg";
+	  case vpiRegBit:
+	    return "vpiRegBit";
 	  case vpiShortIntVar:
 	    return "vpiShortIntVar";
 	  case vpiStringVar:
@@ -462,7 +466,7 @@ char* vpi_get_str(PLI_INT32 property, vpiHandle ref)
 int vpip_time_units_from_handle(vpiHandle obj)
 {
       struct __vpiSysTaskCall*task;
-      struct __vpiScope*scope;
+      __vpiScope*scope;
       struct __vpiSignal*signal;
       __vpiNamedEvent*event;
 
@@ -500,7 +504,7 @@ int vpip_time_units_from_handle(vpiHandle obj)
 int vpip_time_precision_from_handle(vpiHandle obj)
 {
       struct __vpiSysTaskCall*task;
-      struct __vpiScope*scope;
+      __vpiScope*scope;
       struct __vpiSignal*signal;
 
       if (obj == 0)
@@ -1280,14 +1284,14 @@ vpiHandle vpi_handle_by_index(vpiHandle ref, PLI_INT32 idx)
 static vpiHandle find_name(const char *name, vpiHandle handle)
 {
       vpiHandle rtn = 0;
-      struct __vpiScope*ref = dynamic_cast<__vpiScope*>(handle);
+      __vpiScope*ref = dynamic_cast<__vpiScope*>(handle);
 
       /* check module names */
       if (!strcmp(name, vpi_get_str(vpiName, handle)))
 	    rtn = handle;
 
       /* brute force search for the name in all objects in this scope */
-      for (unsigned i = 0 ;  i < ref->nintern ;  i += 1) {
+      for (unsigned i = 0 ;  i < ref->intern.size() ;  i += 1) {
 	      /* The standard says that since a port does not have a full
 	       * name it cannot be found by name. Because of this we need
 	       * to skip ports here so the correct handle can be located. */
@@ -1317,6 +1321,30 @@ static vpiHandle find_name(const char *name, vpiHandle handle)
       return rtn;
 }
 
+// Find the end of the escaped identifier or simple identifier
+static char * find_rest(char *name)
+{
+      char *rest;
+      if (*name == '\\') {
+	    rest = strchr(name, ' ');
+	    if (rest) {
+		  *rest++ = 0; // The space is not part of the string
+		    // There should be a '.' after the escaped ID if there is
+		    // anything more. If it is missing add it to avoid a crash
+		  if ((*rest != '.') && (*rest != 0)) {
+			*--rest = '.';
+			fprintf(stderr, "ERROR: Malformed scope string: \"%s\"", name);
+		  }
+		  if (*rest == 0) {
+			rest = 0;
+		  }
+	    }
+      } else {
+	    rest = strchr(name, '.');
+      }
+      return rest;
+}
+
 static vpiHandle find_scope(const char *name, vpiHandle handle, int depth)
 {
 
@@ -1327,7 +1355,8 @@ static vpiHandle find_scope(const char *name, vpiHandle handle, int depth)
       vector<char> name_buf (strlen(name)+1);
       strcpy(&name_buf[0], name);
       char*nm_first = &name_buf[0];
-      char*nm_rest = strchr(nm_first, '.');
+      char*nm_rest = find_rest(nm_first);
+      if (*nm_first == '\\') ++nm_first;
       if (nm_rest) {
 	    *nm_rest++ = 0;
       }
@@ -1354,6 +1383,19 @@ static vpiHandle find_scope(const char *name, vpiHandle handle, int depth)
       return rtn;
 }
 
+// Find the end of the first escaped identifier or simple identifier
+static char * find_next(char *name)
+{
+      char *next;
+      if (*name == '\\') {
+	    next = strchr(name, ' ');
+	    if (next && *++next == 0) next = 0;
+      } else {
+	    next = strchr(name, '.');
+      }
+      return next;
+}
+
 vpiHandle vpi_handle_by_name(const char *name, vpiHandle scope)
 {
       vpiHandle hand;
@@ -1369,12 +1411,41 @@ vpiHandle vpi_handle_by_name(const char *name, vpiHandle scope)
       vector<char> name_buf (strlen(name)+1);
       strcpy(&name_buf[0], name);
       char*nm_path = &name_buf[0];
-      char*nm_base = strrchr(nm_path, '.');
-      if (nm_base) {
-	    *nm_base++ = 0;
-      } else {
+      char*nm_base;
+
+	// Check to see if we have an escaped identifier and if so search
+	// the long way because a '.' could be in the escaped name.
+      if (strchr(nm_path, '\\')) {
+	    char *next;
 	    nm_base = nm_path;
-	    nm_path = 0;
+	    while ((next = find_next(nm_base))) {
+		  nm_base = ++next;
+	    }
+	    if (nm_path == nm_base) {
+		  nm_path = 0;
+	    } else {
+		  *(nm_base-1) = 0;
+	    }
+
+	    if (*nm_base == '\\') {
+		    // Skip the \ at the beginning
+		  ++nm_base;
+
+		    // Drop the space at the end if it exists
+		  if ((next = strchr(nm_base, ' '))) {
+			*next = 0;
+		  }
+	    }
+
+	// If there is no escaped identifier then just look for the last '.'
+      } else {
+	    nm_base = strrchr(nm_path, '.');
+	    if (nm_base) {
+		  *nm_base++ = 0;
+	    } else {
+		  nm_base = nm_path;
+		  nm_path = 0;
+	    }
       }
 
       /* If scope provided, look in corresponding module; otherwise
@@ -1634,3 +1705,46 @@ extern "C" void vpip_count_drivers(vpiHandle ref, unsigned idx,
       assert(rfp);
       rfp->node->count_drivers(idx, counts);
 }
+
+#if defined(__MINGW32__) || defined (__CYGWIN__)
+vpip_routines_s vpi_routines = {
+    .register_cb                = vpi_register_cb,
+    .remove_cb                  = vpi_remove_cb,
+    .register_systf             = vpi_register_systf,
+    .get_systf_info             = vpi_get_systf_info,
+    .handle_by_name             = vpi_handle_by_name,
+    .handle_by_index            = vpi_handle_by_index,
+    .handle                     = vpi_handle,
+    .iterate                    = vpi_iterate,
+    .scan                       = vpi_scan,
+    .get                        = vpi_get,
+    .get_str                    = vpi_get_str,
+    .get_delays                 = vpi_get_delays,
+    .put_delays                 = vpi_put_delays,
+    .get_value                  = vpi_get_value,
+    .put_value                  = vpi_put_value,
+    .get_time                   = vpi_get_time,
+    .get_userdata               = vpi_get_userdata,
+    .put_userdata               = vpi_put_userdata,
+    .mcd_open                   = vpi_mcd_open,
+    .mcd_close                  = vpi_mcd_close,
+    .mcd_flush                  = vpi_mcd_flush,
+    .mcd_name                   = vpi_mcd_name,
+    .mcd_vprintf                = vpi_mcd_vprintf,
+    .flush                      = vpi_flush,
+    .vprintf                    = vpi_vprintf,
+    .chk_error                  = vpi_chk_error,
+    .compare_objects            = vpi_compare_objects,
+    .free_object                = vpi_free_object,
+    .get_vlog_info              = vpi_get_vlog_info,
+    .vcontrol                   = vpi_sim_vcontrol,
+    .fopen                      = vpi_fopen,
+    .get_file                   = vpi_get_file,
+    .calc_clog2                 = vpip_calc_clog2,
+    .count_drivers              = vpip_count_drivers,
+    .format_strength            = vpip_format_strength,
+    .make_systf_system_defined  = vpip_make_systf_system_defined,
+    .mcd_rawwrite               = vpip_mcd_rawwrite,
+    .set_return_value           = vpip_set_return_value,
+};
+#endif
