@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -25,27 +25,26 @@
 #ifdef CHECK_WITH_VALGRIND
 # include  "vvp_cleanup.h"
 #endif
+# include  <vector>
 # include  <cstring>
 # include  <cstdlib>
 # include  <cassert>
 # include  "ivl_alloc.h"
 
+using namespace std;
 
-static vpiHandle *vpip_root_table_ptr = 0;
-static unsigned   vpip_root_table_cnt = 0;
+static vector<vpiHandle> vpip_root_table;
 
 vpiHandle vpip_make_root_iterator(void)
 {
-      assert(vpip_root_table_ptr);
-      assert(vpip_root_table_cnt);
-      return vpip_make_iterator(vpip_root_table_cnt,
-				vpip_root_table_ptr, false);
+      return vpip_make_iterator(vpip_root_table.size(),
+				&vpip_root_table[0], false);
 }
 
 void vpip_make_root_iterator(__vpiHandle**&table, unsigned&ntable)
 {
-      table = vpip_root_table_ptr;
-      ntable = vpip_root_table_cnt;
+      table = &vpip_root_table[0];
+      ntable = vpip_root_table.size();
 }
 
 #ifdef CHECK_WITH_VALGRIND
@@ -55,11 +54,11 @@ void port_delete(__vpiHandle*handle);
 static class_type **class_list = 0;
 static unsigned class_list_count = 0;
 
-static void delete_sub_scopes(struct __vpiScope *scope)
+static void delete_sub_scopes(__vpiScope *scope)
 {
-      for (unsigned idx = 0; idx < scope->nintern; idx += 1) {
+      for (unsigned idx = 0; idx < scope->intern.size(); idx += 1) {
 	    vpiHandle item = (scope->intern)[idx];
-	    struct __vpiScope*lscope = static_cast<__vpiScope*>(item);
+	    __vpiScope*lscope = static_cast<__vpiScope*>(item);
 	    switch(item->get_type_code()) {
 		case vpiFunction:
 		case vpiTask:
@@ -68,6 +67,7 @@ static void delete_sub_scopes(struct __vpiScope *scope)
 		case vpiGenScope:
 		case vpiNamedBegin:
 		case vpiNamedFork:
+		case vpiClassTypespec:
 		  delete_sub_scopes(lscope);
 		  vthreads_delete(lscope);
 		  delete item;
@@ -133,7 +133,7 @@ static void delete_sub_scopes(struct __vpiScope *scope)
 		  break;
 	    }
       }
-      free(scope->intern);
+      scope->intern.clear();
 
 	/* Save any class definitions to clean up later. */
       map<std::string, class_type*>::iterator citer;
@@ -148,16 +148,14 @@ static void delete_sub_scopes(struct __vpiScope *scope)
 
 void root_table_delete(void)
 {
-      for (unsigned idx = 0; idx < vpip_root_table_cnt; idx += 1) {
-	    struct __vpiScope *scope = static_cast<__vpiScope *>
-	          (vpip_root_table_ptr[idx]);
+      for (unsigned idx = 0; idx < vpip_root_table.size(); idx += 1) {
+	    __vpiScope *scope = static_cast<__vpiScope *>
+	          (vpip_root_table[idx]);
 	    vthreads_delete(scope);
 	    delete_sub_scopes(scope);
 	    delete scope;
       }
-      free(vpip_root_table_ptr);
-      vpip_root_table_ptr = 0;
-      vpip_root_table_cnt = 0;
+      vpip_root_table.clear();
 
 	/* Clean up all the class definitions. */
       for (unsigned idx = 0; idx < class_list_count; idx += 1) {
@@ -169,45 +167,14 @@ void root_table_delete(void)
 }
 #endif
 
-static int scope_get(int code, vpiHandle obj)
-{
-      struct __vpiScope*ref = dynamic_cast<__vpiScope*>(obj);
-      assert(obj);
-
-      switch (code) {
-	  case vpiCellInstance:
-	    return (int) ref->is_cell;
-
-	  case vpiDefLineNo:
-	    return ref->def_lineno;
-
-	  case vpiLineNo:
-	    return ref->lineno;
-
-	  case vpiTimeUnit:
-	    return ref->time_units;
-
-	  case vpiTimePrecision:
-	    return ref->time_precision;
-
-	  case vpiTopModule:
-	    return 0x0 == ref->scope;
-
-          case vpiAutomatic:
-	    return (int) ref->is_automatic;
-      }
-
-      return vpiUndefined;
-}
-
-static void construct_scope_fullname(struct __vpiScope*ref, char*buf)
+static void construct_scope_fullname(__vpiScope*ref, char*buf)
 {
       if (ref->scope) {
 	    construct_scope_fullname(ref->scope, buf);
 	    strcat(buf, ".");
       }
 
-      strcat(buf, ref->name);
+      strcat(buf, ref->scope_name());
 }
 
 static const char* scope_get_type(int code)
@@ -233,7 +200,7 @@ static const char* scope_get_type(int code)
 
 static char* scope_get_str(int code, vpiHandle obj)
 {
-      struct __vpiScope*ref = dynamic_cast<__vpiScope*>(obj);
+      __vpiScope*ref = dynamic_cast<__vpiScope*>(obj);
       assert(ref);
 
       char buf[4096];  // XXX is a fixed buffer size really reliable?
@@ -254,11 +221,11 @@ static char* scope_get_str(int code, vpiHandle obj)
 	    break;
 
 	  case vpiName:
-	    p = ref->name;
+	    p = ref->scope_name();
 	    break;
 
 	  case vpiDefName:
-	    p = ref->tname;
+	    p = ref->scope_def_name();
 	    break;
 
 	  case vpiType:
@@ -274,7 +241,7 @@ static char* scope_get_str(int code, vpiHandle obj)
 
 static vpiHandle scope_get_handle(int code, vpiHandle obj)
 {
-      struct __vpiScope*rfp = dynamic_cast<__vpiScope*>(obj);
+      __vpiScope*rfp = dynamic_cast<__vpiScope*>(obj);
       assert(rfp);
 
       switch (code) {
@@ -324,12 +291,12 @@ static int compare_types(int code, int type)
       return 0;
 }
 
-static vpiHandle module_iter_subset(int code, struct __vpiScope*ref)
+static vpiHandle module_iter_subset(int code, __vpiScope*ref)
 {
       unsigned mcnt = 0, ncnt = 0;
       vpiHandle*args;
 
-      for (unsigned idx = 0 ;  idx < ref->nintern ;  idx += 1)
+      for (unsigned idx = 0 ;  idx < ref->intern.size() ;  idx += 1)
 	    if (compare_types(code, ref->intern[idx]->get_type_code()))
 		  mcnt += 1;
 
@@ -337,7 +304,7 @@ static vpiHandle module_iter_subset(int code, struct __vpiScope*ref)
 	    return 0;
 
       args = (vpiHandle*)calloc(mcnt, sizeof(vpiHandle));
-      for (unsigned idx = 0 ;  idx < ref->nintern ;  idx += 1)
+      for (unsigned idx = 0 ;  idx < ref->intern.size() ;  idx += 1)
 	    if (compare_types(code, ref->intern[idx]->get_type_code()))
 		  args[ncnt++] = ref->intern[idx];
 
@@ -354,15 +321,48 @@ static vpiHandle module_iter_subset(int code, struct __vpiScope*ref)
  */
 static vpiHandle module_iter(int code, vpiHandle obj)
 {
-      struct __vpiScope*ref = dynamic_cast<__vpiScope*>(obj);
+      __vpiScope*ref = dynamic_cast<__vpiScope*>(obj);
       assert(ref);
 
       return module_iter_subset(code, ref);
 }
 
 
+__vpiScope::__vpiScope(const char*nam, const char*tnam, bool auto_flag)
+: is_automatic_(auto_flag)
+{
+      name_ = vpip_name_string(nam);
+      tname_ = vpip_name_string(tnam? tnam : "");
+}
+
 int __vpiScope::vpi_get(int code)
-{ return scope_get(code, this); }
+{
+      switch (code) {
+	  case vpiCellInstance:
+	    return is_cell? 1 : 0;
+
+	  case vpiDefLineNo:
+	    return def_lineno;
+
+	  case vpiLineNo:
+	    return lineno;
+
+	  case vpiTimeUnit:
+	    return time_units;
+
+	  case vpiTimePrecision:
+	    return time_precision;
+
+	  case vpiTopModule:
+	    return 0x0 == scope;
+
+          case vpiAutomatic:
+	    return is_automatic_? 1 : 0;
+      }
+
+      return vpiUndefined;
+}
+
 
 char*__vpiScope::vpi_get_str(int code)
 { return scope_get_str(code, this); }
@@ -374,43 +374,66 @@ vpiHandle __vpiScope::vpi_iterate(int code)
 { return module_iter(code, this); }
 
 
-struct vpiScopeModule  : public __vpiScope {
-      inline vpiScopeModule() { }
+class vpiScopeModule  : public __vpiScope {
+    public:
+      inline vpiScopeModule(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,false) { }
       int get_type_code(void) const { return vpiModule; }
 };
 
 struct vpiScopePackage  : public __vpiScope {
-      inline vpiScopePackage() { }
+      inline vpiScopePackage(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam) { }
       int get_type_code(void) const { return vpiPackage; }
 };
 
 struct vpiScopeTask  : public __vpiScope {
-      inline vpiScopeTask() { }
+      inline vpiScopeTask(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam) { }
       int get_type_code(void) const { return vpiTask; }
 };
 
-struct vpiScopeFunction  : public __vpiScope {
-      inline vpiScopeFunction() { }
-      int get_type_code(void) const { return vpiFunction; }
+struct vpiScopeTaskAuto  : public __vpiScope {
+      inline vpiScopeTaskAuto(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,true) {  }
+      int get_type_code(void) const { return vpiTask; }
 };
 
 struct vpiScopeBegin  : public __vpiScope {
-      inline vpiScopeBegin() { }
+      inline vpiScopeBegin(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,false) { }
+      int get_type_code(void) const { return vpiNamedBegin; }
+};
+
+class vpiScopeBeginAuto  : public __vpiScope {
+    public:
+      inline vpiScopeBeginAuto(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,true) {  }
       int get_type_code(void) const { return vpiNamedBegin; }
 };
 
 struct vpiScopeGenerate  : public __vpiScope {
-      inline vpiScopeGenerate() { }
+      inline vpiScopeGenerate(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam) { }
       int get_type_code(void) const { return vpiGenScope; }
 };
 
 struct vpiScopeFork  : public __vpiScope {
-      inline vpiScopeFork() { }
+      inline vpiScopeFork(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,false) { }
+      int get_type_code(void) const { return vpiNamedFork; }
+};
+
+class vpiScopeForkAuto  : public __vpiScope {
+    public:
+      inline vpiScopeForkAuto(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam,true) {  }
       int get_type_code(void) const { return vpiNamedFork; }
 };
 
 struct vpiScopeClass  : public __vpiScope {
-      inline vpiScopeClass() { }
+      inline vpiScopeClass(const char*nam, const char*tnam)
+      : __vpiScope(nam,tnam) { }
       int get_type_code(void) const { return vpiClassTypespec; }
 };
 
@@ -419,21 +442,12 @@ struct vpiScopeClass  : public __vpiScope {
  * compiled, items that have scope are placed in the current
  * scope. The ".scope" directives select the scope that is current.
  */
-static struct __vpiScope*current_scope = 0;
+static __vpiScope*current_scope = 0;
 
-void vpip_attach_to_scope(struct __vpiScope*scope, vpiHandle obj)
+void vpip_attach_to_scope(__vpiScope*scope, vpiHandle obj)
 {
       assert(scope);
-      unsigned idx = scope->nintern++;
-
-      if (scope->intern == 0)
-	    scope->intern = (vpiHandle*)
-		  malloc(sizeof(vpiHandle));
-      else
-	    scope->intern = (vpiHandle*)
-		  realloc(scope->intern, sizeof(vpiHandle)*scope->nintern);
-
-      scope->intern[idx] = obj;
+      scope->intern.push_back(obj);
 }
 
 /*
@@ -448,49 +462,88 @@ compile_scope_decl(char*label, char*type, char*name, char*tname,
                    long def_file_idx, long def_lineno, long is_cell)
 {
       count_vpi_scopes += 1;
+      char vec_type;
+      char sign_flag;
+      unsigned wid;
 
-      char*base_type;
-      bool is_automatic;
-      if (strncmp(type,"auto",4) == 0) {
-	    is_automatic = true;
-            base_type = &type[4];
-      } else {
-	    is_automatic = false;
-            base_type = &type[0];
-      }
+      __vpiScope*scope;
+      if (strcmp(type,"module") == 0) {
+	    scope = new vpiScopeModule(name, tname);
+      } else if ( sscanf(type, "function.vec%c.%c%u", &vec_type, &sign_flag, &wid) == 3 ) {
+	    int type_code;
+	    if (sign_flag=='s') {
+		  type_code = vpiSizedSignedFunc;
+	    } else if (sign_flag=='u') {
+		  type_code = vpiSizedFunc;
+	    } else if (sign_flag=='i') {
+		  type_code = vpiIntFunc;
+	    } else {
+		  assert(0);
+		  type_code = vpiSizedFunc;
+	    }
+	    vvp_bit4_t init_val = vec_type == '4' ? BIT4_X : BIT4_0;
+	    scope = new vpiScopeFunction(name, tname, false, type_code, wid, init_val);
 
-      struct __vpiScope*scope;
-      if (strcmp(base_type,"module") == 0) {
-	    scope = new vpiScopeModule;
-      } else if (strcmp(base_type,"function") == 0) {
-	    scope = new vpiScopeFunction;
-      } else if (strcmp(base_type,"task") == 0) {
-	    scope = new vpiScopeTask;
-      } else if (strcmp(base_type,"fork") == 0) {
-	    scope = new vpiScopeFork;
-      } else if (strcmp(base_type,"begin") == 0) {
-	    scope = new vpiScopeBegin;
-      } else if (strcmp(base_type,"generate") == 0) {
-	    scope = new vpiScopeGenerate;
-      } else if (strcmp(base_type,"package") == 0) {
-	    scope = new vpiScopePackage;
-      } else if (strcmp(base_type,"class") == 0) {
-	    scope = new vpiScopeClass;
+      } else if ( sscanf(type, "autofunction.vec%c.%c%u", &vec_type, &sign_flag, &wid) == 3 ) {
+	    int type_code;
+	    switch (sign_flag) {
+		case 's':
+		  type_code = vpiSizedSignedFunc;
+		  break;
+		case 'u':
+		  type_code = vpiSizedFunc;
+		  break;
+		default:
+		  assert(0);
+		  type_code = vpiSizedFunc;
+		  break;
+	    }
+	    vvp_bit4_t init_val = vec_type == '4' ? BIT4_X : BIT4_0;
+	    scope = new vpiScopeFunction(name, tname, true, type_code, wid, init_val);
+
+      } else if (strcmp(type,"function.obj") == 0) {
+	    scope = new vpiScopeFunction(name, tname, false, vpiSizedFunc, 0, BIT4_0);
+      } else if (strcmp(type,"autofunction.obj") == 0) {
+	    scope = new vpiScopeFunction(name, tname, true, vpiSizedFunc, 0, BIT4_0);
+      } else if (strcmp(type,"function.real") == 0) {
+	    scope = new vpiScopeFunction(name, tname, false, vpiRealFunc, 0, BIT4_0);
+      } else if (strcmp(type,"autofunction.real") == 0) {
+	    scope = new vpiScopeFunction(name, tname, true, vpiRealFunc, 0, BIT4_0);
+      } else if (strcmp(type,"function.str") == 0) {
+	    scope = new vpiScopeFunction(name, tname, false, vpiOtherFunc, 0, BIT4_0);
+      } else if (strcmp(type,"autofunction.str") == 0) {
+	    scope = new vpiScopeFunction(name, tname, true, vpiOtherFunc, 0, BIT4_0);
+      } else if (strcmp(type,"function.void") == 0) {
+	    scope = new vpiScopeFunction(name, tname, false, vpiOtherFunc, 0, BIT4_0);
+      } else if (strcmp(type,"autofunction.void") == 0) {
+	    scope = new vpiScopeFunction(name, tname, true, vpiOtherFunc, 0, BIT4_0);
+      } else if (strcmp(type,"task") == 0) {
+	    scope = new vpiScopeTask(name, tname);
+      } else if (strcmp(type,"autotask") == 0) {
+	    scope = new vpiScopeTaskAuto(name, tname);
+      } else if (strcmp(type,"fork") == 0) {
+	    scope = new vpiScopeFork(name, tname);
+      } else if (strcmp(type,"autofork") == 0) {
+	    scope = new vpiScopeForkAuto(name, tname);
+      } else if (strcmp(type,"begin") == 0) {
+	    scope = new vpiScopeBegin(name, tname);
+      } else if (strcmp(type,"autobegin") == 0) {
+	    scope = new vpiScopeBeginAuto(name, tname);
+      } else if (strcmp(type,"generate") == 0) {
+	    scope = new vpiScopeGenerate(name, tname);
+      } else if (strcmp(type,"package") == 0) {
+	    scope = new vpiScopePackage(name, tname);
+      } else if (strcmp(type,"class") == 0) {
+	    scope = new vpiScopeClass(name, tname);
       } else {
-	    scope = new vpiScopeModule;
+	    scope = new vpiScopeModule(name, tname);
 	    assert(0);
       }
 
-      scope->name = vpip_name_string(name);
-      if (tname) scope->tname = vpip_name_string(tname);
-      else scope->tname = vpip_name_string("");
       scope->file_idx = (unsigned) file_idx;
       scope->lineno  = (unsigned) lineno;
       scope->def_file_idx = (unsigned) def_file_idx;
       scope->def_lineno  = (unsigned) def_lineno;
-      scope->is_automatic = is_automatic;
-      scope->intern = 0;
-      scope->nintern = 0;
       scope->item = 0;
       scope->nitem = 0;
       scope->live_contexts = 0;
@@ -512,7 +565,7 @@ compile_scope_decl(char*label, char*type, char*name, char*tname,
 	    static vpiHandle obj;
 	    compile_vpi_lookup(&obj, parent);
 	    assert(obj);
-	    struct __vpiScope*sp = dynamic_cast<__vpiScope*>(obj);
+	    __vpiScope*sp = dynamic_cast<__vpiScope*>(obj);
 	    vpip_attach_to_scope(sp, scope);
 	    scope->scope = dynamic_cast<__vpiScope*>(obj);
 
@@ -523,11 +576,7 @@ compile_scope_decl(char*label, char*type, char*name, char*tname,
       } else {
 	    scope->scope = 0x0;
 
-	    unsigned cnt = vpip_root_table_cnt + 1;
-	    vpip_root_table_ptr = (vpiHandle*)
-		  realloc(vpip_root_table_ptr, cnt * sizeof(vpiHandle));
-	    vpip_root_table_ptr[vpip_root_table_cnt] = scope;
-	    vpip_root_table_cnt = cnt;
+	    vpip_root_table.push_back(scope);
 
 	      /* Root scopes inherit time_units and precision from the
 	         system precision. */
@@ -556,7 +605,7 @@ void compile_timescale(long units, long precision)
       current_scope->time_precision = precision;
 }
 
-struct __vpiScope* vpip_peek_current_scope(void)
+__vpiScope* vpip_peek_current_scope(void)
 {
       return current_scope;
 }
@@ -566,24 +615,24 @@ void vpip_attach_to_current_scope(vpiHandle obj)
       vpip_attach_to_scope(current_scope, obj);
 }
 
-struct __vpiScope* vpip_peek_context_scope(void)
+__vpiScope* vpip_peek_context_scope(void)
 {
-      struct __vpiScope*scope = current_scope;
+      __vpiScope*scope = current_scope;
 
         /* A context is allocated for each automatic task or function.
            Storage for nested scopes (named blocks) is allocated in
            the parent context. */
-      while (scope->scope && scope->scope->is_automatic)
+      while (scope->scope && scope->scope->is_automatic())
             scope = scope->scope;
 
       return scope;
 }
 
 unsigned vpip_add_item_to_context(automatic_hooks_s*item,
-                                  struct __vpiScope*scope)
+                                  __vpiScope*scope)
 {
       assert(scope);
-      assert(scope->is_automatic);
+      assert(scope->is_automatic());
 
       unsigned idx = scope->nitem++;
 
