@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -581,7 +581,7 @@ static int signal_get(int code, vpiHandle ref)
 	    return (rfp->msb.get_value() != rfp->lsb.get_value());
 
           case vpiAutomatic:
-            return (int) vpip_scope(rfp)->is_automatic;
+            return vpip_scope(rfp)->is_automatic() ? 1  : 0;
 
 #ifdef BR916_STOPGAP_FIX
           case _vpiFromThr:
@@ -671,22 +671,151 @@ static vpiHandle signal_iterate(int code, vpiHandle ref)
       return 0;
 }
 
+void __vpiSignal::make_bits()
+{
+      assert(bits == NULL);
+      bits = new struct __vpiBit[width() + 1];
+
+	// Make word[-1] point to the parent.
+      bits->parent = this;
+	// Now point to word[0].
+      bits += 1;
+
+	// Update each bit to point to the base
+      struct __vpiBit*base = bits;
+      for (unsigned idx = 0; idx < width(); idx += 1) {
+	    base[idx].bit0 = base;
+	    int real_idx;
+	    if (msb.get_value() >= lsb.get_value()) {
+		  real_idx = idx + lsb.get_value();
+	    } else {
+		  real_idx = lsb.get_value() - idx;
+	    }
+	    base[idx].index = new __vpiDecConst(real_idx);
+      }
+}
+
+vpiHandle __vpiSignal::get_index(int idx)
+{
+	/* Check to see if the index is in range. */
+      if (msb.get_value() >= lsb.get_value()) {
+	    if ((idx > msb.get_value()) || (idx < lsb.get_value())) return 0;
+      } else {
+	    if ((idx < msb.get_value()) || (idx > lsb.get_value())) return 0;
+      }
+
+	/* Normalize the index */
+      unsigned norm_idx;
+      if (msb.get_value() >= lsb.get_value()) {
+	    norm_idx = idx - lsb.get_value();
+      } else {
+	    norm_idx = lsb.get_value() - idx;
+      }
+
+      if (bits == NULL) make_bits();
+
+      return &(bits[norm_idx].as_bit);
+}
+
+void __vpiSignal::get_bit_value(struct __vpiBit*bit, p_vpi_value vp)
+{
+      unsigned index = bit->get_norm_index();
+
+      vvp_signal_value*vsig = dynamic_cast<vvp_signal_value*>(node->fil);
+      assert(vsig);
+
+      if (vp->format == vpiObjTypeVal) {
+	    vp->format = vpiIntVal;
+      }
+
+      switch (vp->format) {
+	case vpiBinStrVal:
+            format_vpiBinStrVal(vsig, index, 1, vp);
+	    break;
+
+	case vpiOctStrVal:
+            format_vpiOctStrVal(vsig, index, 1, vp);
+	    break;
+
+	case vpiDecStrVal:
+            format_vpiDecStrVal(vsig, index, 1, false, vp);
+	    break;
+
+	case vpiHexStrVal:
+            format_vpiHexStrVal(vsig, index, 1, vp);
+	    break;
+
+	case vpiStringVal:
+            format_vpiStringVal(vsig, index, 1, vp);
+	    break;
+
+	case vpiIntVal:
+            format_vpiIntVal(vsig, index, 1, false, vp);
+	    break;
+
+	case vpiRealVal:
+            format_vpiRealVal(vsig, index, 1, false, vp);
+	    break;
+
+	case vpiScalarVal:
+            format_vpiScalarVal(vsig, index, vp);
+	    break;
+
+        case vpiStrengthVal:
+            format_vpiStrengthVal(vsig, index, 1, vp);
+            break;
+
+	case vpiVectorVal:
+            format_vpiVectorVal(vsig, index, 1, vp);
+	    break;
+
+	default:
+	    fprintf(stderr, "internal error: get_value() "
+		    "type %d is not implemented for bit "
+		    "select %s[%d] in scope %s.\n",
+		    (int)vp->format, vpi_get_str(vpiName),
+		    bit->get_index(),
+	            vpip_scope(this)->scope_name());
+	    assert(0);
+      };
+}
+
+vpiHandle __vpiSignal::put_bit_value(struct __vpiBit*bit, p_vpi_value vp, int flags)
+{
+      unsigned index = bit->get_norm_index();
+      vvp_net_ptr_t dest(node, 0);
+      vvp_vector4_t val = vec4_from_vpi_value(vp, 1);
+
+      if ((flags == vpiForceFlag) || (flags == vpiReleaseFlag)) {
+	    fprintf(stderr, "Sorry: vpi_put_value() for %s does not "
+	                    "currently support force/release.\n",
+	                    bit->as_bit.vpi_get_str(vpiFullName));
+	    return NULL;
+      }
+
+      if ((get_type_code() == vpiNet) &&
+          !dynamic_cast<vvp_island_port*>(node->fun)) {
+	    node->send_vec4_pv(val, index, 1, width(),
+	                            vthread_get_wt_context());
+      } else {
+	    vvp_send_vec4_pv(dest, val, index, 1, width(),
+	                     vthread_get_wt_context());
+      }
+
+	// This is not a scheduled event so there is no event to return
+      return NULL;
+}
+
 static vpiHandle signal_index(int idx, vpiHandle ref)
 {
       struct __vpiSignal*rfp = dynamic_cast<__vpiSignal*>(ref);
       assert(rfp);
-	/* Check to see if the index is in range. */
-      if (rfp->msb.get_value() >= rfp->lsb.get_value()) {
-	    if ((idx > rfp->msb.get_value()) || (idx < rfp->lsb.get_value())) return 0;
-      } else {
-	    if ((idx < rfp->msb.get_value()) || (idx > rfp->lsb.get_value())) return 0;
-      }
-	/* Return a handle for the individual bit. */
-      cerr << "Sorry: Icarus does not currently support "
-           << "vpi_get_handle_by_index() for "
-           << vpi_get_str(vpiType, ref);
-      cerr << " objects (" << vpi_get_str(vpiName, ref) << ")." << endl;
-      return 0;
+
+	/* We can only get the bit for a net or reg. */
+      PLI_INT32 type = vpi_get(vpiType, ref);
+      if ((type != vpiNet) && (type != vpiReg)) return 0;
+
+      return rfp->get_index(idx);
 }
 
 unsigned __vpiSignal::width(void) const
@@ -770,7 +899,7 @@ static void signal_get_value(vpiHandle ref, s_vpi_value*vp)
 		    "value type %d not implemented."
 		    " Signal is %s in scope %s\n",
 		    (int)vp->format, vpi_get_str(vpiName, ref),
-		    vpip_scope(rfp)->name);
+		    vpip_scope(rfp)->scope_name());
 	    assert(0);
       }
 }
@@ -1096,6 +1225,13 @@ void signal_delete(vpiHandle item)
       assert(obj->node->fil);
       obj->node->fil->clear_all_callbacks();
       vvp_net_delete(obj->node);
+      if (obj->bits) {
+	    for (unsigned idx=0; idx<obj->width(); idx+=1) {
+		  delete obj->bits[idx].index;
+	    }
+	    obj->bits -= 1;
+	    delete [] obj->bits;
+      }
       signal_dels += 1;
       VALGRIND_MEMPOOL_FREE(reinterpret_cast<vpiSignal_plug *>(obj)->pool, obj);
 }
@@ -1131,8 +1267,8 @@ static vpiHandle fill_in_net4(struct __vpiSignal*obj, __vpiScope*scope,
 			      bool signed_flag, vvp_net_t*node)
 {
       obj->id.name = name? vpip_name_string(name) : 0;
-      obj->msb = msb;
-      obj->lsb = lsb;
+      obj->msb = __vpiDecConst(msb);
+      obj->lsb = __vpiDecConst(lsb);
       obj->signed_flag = signed_flag? 1 : 0;
       obj->is_netarray = 0;
       obj->node = node;
