@@ -1,7 +1,7 @@
 #ifndef IVL_vvp_darray_H
 #define IVL_vvp_darray_H
 /*
- * Copyright (c) 2012-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -21,7 +21,7 @@
 
 # include  "vvp_object.h"
 # include  "vvp_net.h"
-# include  <list>
+# include  <deque>
 # include  <string>
 # include  <vector>
 
@@ -46,18 +46,21 @@ class vvp_darray : public vvp_object {
       virtual void get_word(unsigned adr, vvp_object_t&value);
 
       virtual void shallow_copy(const vvp_object*obj);
+
+      virtual vvp_vector4_t get_bitstream(bool as_vec4);
 };
 
 template <class TYPE> class vvp_darray_atom : public vvp_darray {
 
     public:
-      inline vvp_darray_atom(size_t siz) : array_(siz) { }
+      explicit inline vvp_darray_atom(size_t siz) : array_(siz) { }
       ~vvp_darray_atom();
 
       size_t get_size(void) const;
       void set_word(unsigned adr, const vvp_vector4_t&value);
       void get_word(unsigned adr, vvp_vector4_t&value);
       void shallow_copy(const vvp_object*obj);
+      vvp_vector4_t get_bitstream(bool as_vec4);
 
     private:
       std::vector<TYPE> array_;
@@ -74,6 +77,7 @@ class vvp_darray_vec4 : public vvp_darray {
       void set_word(unsigned adr, const vvp_vector4_t&value);
       void get_word(unsigned adr, vvp_vector4_t&value);
       void shallow_copy(const vvp_object*obj);
+      vvp_vector4_t get_bitstream(bool as_vec4);
 
     private:
       std::vector<vvp_vector4_t> array_;
@@ -91,6 +95,7 @@ class vvp_darray_vec2 : public vvp_darray {
       void set_word(unsigned adr, const vvp_vector4_t&value);
       void get_word(unsigned adr, vvp_vector4_t&value);
       void shallow_copy(const vvp_object*obj);
+      vvp_vector4_t get_bitstream(bool as_vec4);
 
     private:
       std::vector<vvp_vector2_t> array_;
@@ -100,13 +105,14 @@ class vvp_darray_vec2 : public vvp_darray {
 class vvp_darray_real : public vvp_darray {
 
     public:
-      inline vvp_darray_real(size_t siz) : array_(siz) { }
+      explicit inline vvp_darray_real(size_t siz) : array_(siz) { }
       ~vvp_darray_real();
 
       size_t get_size(void) const;
       void set_word(unsigned adr, double value);
       void get_word(unsigned adr, double&value);
       void shallow_copy(const vvp_object*obj);
+      vvp_vector4_t get_bitstream(bool as_vec4);
 
     private:
       std::vector<double> array_;
@@ -115,7 +121,7 @@ class vvp_darray_real : public vvp_darray {
 class vvp_darray_string : public vvp_darray {
 
     public:
-      inline vvp_darray_string(size_t siz) : array_(siz) { }
+      explicit inline vvp_darray_string(size_t siz) : array_(siz) { }
       ~vvp_darray_string();
 
       size_t get_size(void) const;
@@ -130,7 +136,7 @@ class vvp_darray_string : public vvp_darray {
 class vvp_darray_object : public vvp_darray {
 
     public:
-      inline vvp_darray_object(size_t siz) : array_(siz) { }
+      explicit inline vvp_darray_object(size_t siz) : array_(siz) { }
       ~vvp_darray_object();
 
       size_t get_size(void) const;
@@ -148,17 +154,72 @@ class vvp_queue : public vvp_darray {
       inline vvp_queue(void) { }
       ~vvp_queue();
 
-      virtual void push_back(const vvp_vector4_t&value);
-      virtual void push_front(const vvp_vector4_t&value);
+      virtual size_t get_size(void) const =0;
+      virtual void copy_elems(vvp_object_t src, unsigned max_size);
 
-      virtual void push_back(double value);
-      virtual void push_front(double value);
+      virtual void set_word_max(unsigned adr, const vvp_vector4_t&value, unsigned max_size);
+      virtual void insert(unsigned idx, const vvp_vector4_t&value, unsigned max_size);
+      virtual void push_back(const vvp_vector4_t&value, unsigned max_size);
+      virtual void push_front(const vvp_vector4_t&value, unsigned max_size);
 
-      virtual void push_back(const std::string&value);
-      virtual void push_front(const std::string&value);
+      virtual void set_word_max(unsigned adr, double value, unsigned max_size);
+      virtual void insert(unsigned idx, double value, unsigned max_size);
+      virtual void push_back(double value, unsigned max_size);
+      virtual void push_front(double value, unsigned max_size);
+
+      virtual void set_word_max(unsigned adr, const std::string&value, unsigned max_size);
+      virtual void insert(unsigned idx, const std::string&value, unsigned max_size);
+      virtual void push_back(const std::string&value, unsigned max_size);
+      virtual void push_front(const std::string&value, unsigned max_size);
 
       virtual void pop_back(void) =0;
       virtual void pop_front(void)=0;
+      virtual void erase(unsigned idx)=0;
+      virtual void erase_tail(unsigned idx)=0;
+};
+
+class vvp_queue_real : public vvp_queue {
+
+    public:
+      ~vvp_queue_real();
+
+      size_t get_size(void) const { return queue.size(); };
+      void copy_elems(vvp_object_t src, unsigned max_size);
+      void set_word_max(unsigned adr, double value, unsigned max_size);
+      void set_word(unsigned adr, double value);
+      void get_word(unsigned adr, double&value);
+      void insert(unsigned idx, double value, unsigned max_size);
+      void push_back(double value, unsigned max_size);
+      void push_front(double value, unsigned max_size);
+      void pop_back(void) { queue.pop_back(); };
+      void pop_front(void) { queue.pop_front(); };
+      void erase(unsigned idx);
+      void erase_tail(unsigned idx);
+
+    private:
+      std::deque<double> queue;
+};
+
+class vvp_queue_string : public vvp_queue {
+
+    public:
+      ~vvp_queue_string();
+
+      size_t get_size(void) const { return queue.size(); };
+      void copy_elems(vvp_object_t src, unsigned max_size);
+      void set_word_max(unsigned adr, const std::string&value, unsigned max_size);
+      void set_word(unsigned adr, const std::string&value);
+      void get_word(unsigned adr, std::string&value);
+      void insert(unsigned idx, const std::string&value, unsigned max_size);
+      void push_back(const std::string&value, unsigned max_size);
+      void push_front(const std::string&value, unsigned max_size);
+      void pop_back(void) { queue.pop_back(); };
+      void pop_front(void) { queue.pop_front(); };
+      void erase(unsigned idx);
+      void erase_tail(unsigned idx);
+
+    private:
+      std::deque<std::string> queue;
 };
 
 class vvp_queue_vec4 : public vvp_queue {
@@ -166,34 +227,23 @@ class vvp_queue_vec4 : public vvp_queue {
     public:
       ~vvp_queue_vec4();
 
-      size_t get_size(void) const;
+      size_t get_size(void) const { return queue.size(); };
+      void copy_elems(vvp_object_t src, unsigned max_size);
+      void set_word_max(unsigned adr, const vvp_vector4_t&value, unsigned max_size);
       void set_word(unsigned adr, const vvp_vector4_t&value);
       void get_word(unsigned adr, vvp_vector4_t&value);
-      void push_back(const vvp_vector4_t&value);
-      void push_front(const vvp_vector4_t&value);
-      void pop_back(void);
-      void pop_front(void);
+      void insert(unsigned idx, const vvp_vector4_t&value, unsigned max_size);
+      void push_back(const vvp_vector4_t&value, unsigned max_size);
+      void push_front(const vvp_vector4_t&value, unsigned max_size);
+      void pop_back(void) { queue.pop_back(); };
+      void pop_front(void) { queue.pop_front(); };
+      void erase(unsigned idx);
+      void erase_tail(unsigned idx);
 
     private:
-      std::list<vvp_vector4_t> array_;
+      std::deque<vvp_vector4_t> queue;
 };
 
-
-class vvp_queue_string : public vvp_queue {
-
-    public:
-      ~vvp_queue_string();
-
-      size_t get_size(void) const;
-      void set_word(unsigned adr, const std::string&value);
-      void get_word(unsigned adr, std::string&value);
-      void push_back(const std::string&value);
-	//void push_front(const std::string&value);
-      void pop_back(void);
-      void pop_front(void);
-
-    private:
-      std::list<std::string> array_;
-};
+extern string get_fileline();
 
 #endif /* IVL_vvp_darray_H */
